#%%
from cmath import nan
import os
import numpy as np
import pandas as pd
import pyfas as fa
import matplotlib.pyplot as plt
from libraries import gas
from libraries import water
from libraries import datapreprocessing as dpp
from libraries import criticalvelocity as cv
from openpyxl import load_workbook
import mpl_axes_aligner
import warnings
from scipy.interpolate import interp1d
import scipy.optimize as so 

warnings.filterwarnings("ignore", category=RuntimeWarning)

# import sklearn as sklearn
# import scipy.optimize as sciop
# import scipy.integrate as scint
# from sklearn.preprocessing import PolynomialFeatures
# from sklearn.linear_model import LinearRegression

# Load simulation data, input data

L_tubing = 3000  # m, do not change unit
ID_tubing = np.array([2.25,2.25,2.25,2.25])  # in, do not change unit

units = "metric"  # "field"

T0 = 15.56  # standrd temp, degC
T_R = 50  # reserv temp degC
P_R = 70  # bara
SG_gas = 0.61
SG_water = 1.0

WGR_step = 1

test_property = "Gas Production Rate"
test_property_unit = "Sm3/d"

# test_property = "Choke opening "
# test_property_unit = "/64 in"

WGR = (
    WGR_step*[24]+
    WGR_step*[28]+
    WGR_step*[32]+
    WGR_step*[36])#+
    # WGR_step*[40]+
    # WGR_step*[44]+
    # WGR_step*[120])
# )
# QGinj = [0.11, 0.105, 0.1, 0.098, 0.093, 0.087, 0.08, 0.01]
P_wh = 100/14.5 # bara

# Backpressure equation reservoir properties
sD = 1  # 1/sm3/s
k = 100  # md
h = 100  # m
Z = 1
rw = 2.25 / 2  # in
re = 1500  # m
s = 0
n = 1
c = 10


j = 0
l = 0
ll = 0
idx_ppl = 0
degree = 7  # polynomial degree
tol = 0.03
kernel_size = 25
kernel = np.ones(kernel_size) / kernel_size

V_tubing = L_tubing * np.pi / 4 * (0.0254 * ID_tubing) ** 2  # m3

tpl_files = []
ppl_files = []
PT_list = []
QGST_list = []
lst = []

crit_idx = []
slug_idx = []

QgTPR = []
PTTPR = []


end_stable_idx = -1
end_stable_rate = -1
end_stable_PT = -1

start = 10
#%%
path_of_directory = str(input("Type in the name of the simulation data folder:"))
if path_of_directory == "":
    path_of_directory = "OLGA"
ext = ".tpl"
for files in os.listdir(path_of_directory):
    if files.endswith(".tpl"):
        tpl_files.append(files)
    elif files.endswith(".ppl"):
        ppl_files.append(files)
    else:
        continue

constantpressuresim = str(input("Constant pressure simulation data? (y/n)"))
if constantpressuresim == "":
    constantpressuresim = "y"


def find_nearest(array, value):
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    return idx, array[idx]


def find_slug(array, value):
    array = np.asarray(array)
    idx = int((np.abs(array - value)).argmin() / len(ID[1][1]))
    return idx, array[idx]


def C(k, h, T, vis, Z, sD, re, rw, s, n=1):
    hf = h / 0.3048
    Tf = (T + 273.15) * 1.8
    ref = re / 0.3048
    rwf = rw / 12

    c = (0.703 * k * hf) ** n / (
        (Tf * vis * Z) ** n
        * sD ** (1 - n)
        * (np.log(ref / rwf) - 0.75 + s) ** (2 * n - 1)
    )
    return c * 14.5 ** (2 * n) / 35.3146667


def backpressure(P_R, P_wf, C, n=1):
    q = C * (P_R ** 2 - P_wf ** 2) ** n
    return q

def nodal_point_solver(guess):

    x = np.array(QGST)
    y = np.array(PT)
    X_Y_spline = interp1d(x, y, kind="quadratic")

    P_wf = guess[0]
    Qg = guess [1]

    eq1 = Qg - c*(P_R**2-P_wf**2)**n
    eq2 = P_wf - X_Y_spline(Qg)

    return [eq1, eq2]


for i, file in enumerate(tpl_files):
    time, LIQC, QGST, QWST, PT = dpp.getTPLData(
        path_of_directory + "/", file
    )  # metric units: s, m3, Sm3/s, Pa

    try:
        # if ((1+tol) <= PT[int(len(PT)/4)] / PT[int(len(PT)/3)]) or ((1-tol) >= PT[int(len(PT)/4)] / PT[int(len(PT)/4)]):
        if constantpressuresim == "n":
            QGST_diff = [
                np.abs(
                    ((QGST[k + 1] - QGST[k]) / (time[k + 1] - time[k]))
                    / ((QGST[k + 2] - QGST[k + 1]) / (time[k + 2] - time[k + 1]))
                )
                for k in range(1, len(time) - 2)
            ]
            QGST_convolved = np.convolve(QGST_diff, kernel, mode="same")
            QWST_diff = [
                np.abs(
                    ((QWST[k + 1] - QWST[k]) / (time[k + 1] - time[k]))
                    / ((QWST[k + 2] - QWST[k + 1]) / (time[k + 2] - time[k + 1]))
                )
                for k in range(1, len(time) - 2)
            ]
            QWST_convolved = np.convolve(QWST_diff, kernel, mode="same")
            k = 0
            for k in range(start, len(time) - 2):
                if (
                    (QGST_convolved[k] <= (1 - tol))
                    and (QWST_convolved[k] <= (1 - tol))
                ) or QGST[k] <= 0:
                    idx_crit = k
                    tc = time[k]
                    Qw_crit = QWST[k]
                    Qg_crit = QGST[k]
                    crit_idx.append(k)
                    break

                elif (
                    (QGST_convolved[k] >= (1 + tol))
                    and (QWST_convolved[k] >= (1 + tol))
                ) or QGST[k] <= 0:
                    idx_crit = k
                    tc = time[k]
                    Qw_crit = QWST[k]
                    Qg_crit = QGST[k]
                    break
            try:
                for k in range(start, len(time)-1):            
                    if QGST[k] <= 0 or PT[k+1] >= PT[k]*1.1 or QGST[k+1] >= QGST[k]*2:
                        end_stable_idx = k
                        end_stable_rate = QGST[start:k-1]*86400
                        end_stable_PT = PT[start:k-1]*1e-5
                        QgTPR.append(end_stable_rate)
                        PTTPR.append(end_stable_PT)
                        crit_idx_plot = np.where(end_stable_PT == end_stable_PT.min())
                        crit_rate_plot = QGST[crit_idx_plot][0]*86400
                        if end_stable_idx >= len(QGST):
                            end_stable_idx = len(QGST)-1
                        break
                    elif k == len(time)-2:
                        end_stable_idx = -2
                        end_stable_rate = QGST[start:-2]*86400
                        end_stable_PT = PT[start:-2]*1e-5
                        QgTPR.append(end_stable_rate)
                        PTTPR.append(end_stable_PT)
                        crit_idx_plot = np.where(end_stable_PT == end_stable_PT.min())
                        crit_rate_plot = QGST[crit_idx_plot][0]*86400
                        break
                    # else:
                    #     print("fail")
            except:
                end_stable_idx = False #-1
                end_stable_rate = QGST[10:end_stable_idx]*86400
                end_stable_PT = PT[10:-2]*1e-5
                QgTPR.append(end_stable_rate)
                PTTPR.append(end_stable_PT)
                break
        else:
            qgst_avg_idx = np.array(
                [
                    find_nearest(
                        QGST,
                        np.average(
                            QGST[5 * j : 5 + 5 * j]
                            == np.average(QGST[5 + 5 * j : 10 + 5 * j])
                        ),
                    )
                    for j in range(0, len(QGST[::5]))
                ]
            )
            qgst_avg_idx = qgst_avg_idx[qgst_avg_idx > 0]
            k = int(qgst_avg_idx[0])
            print(k, qgst_avg_idx[1] * 86400)
            # for k in range(2, len(time)-2):
            #     if ((1- tol/2) <= np.abs(QGST[k + 1] / QGST[k]) <=  (1+ tol/2)): # or (np.abs(QGST[k + 1] / QGST[k]) >=  (1- tol/2)) :
            #         tss = time[k]
            #         QgSS = QGST[k]
            #         # k = -1
            #         break
            #     else:
            #         tss = False
            #         QgSS = False
            idx_crit = False
            # crit_idx_plot = [[False]]
            # crit_rate_plot = False

    except:
        print(f"Critical velocity is not found for case {file[0:2]}")
        # tc = False
        # crit_idx.append(0)
        # stabilization_rate = False

    
    # if tc == False:
    #     time_idx = -1
    # else:
    #     time_idx = k
    # time_value = find_nearest(time, 4 * 86400)[1] / 86400  # days
    # PVT

    if units == "field":
        LIQC = LIQC * 6.28981077  # m3 -> bbl
        QGST = QGST * 86400 * 35.3147 / 1000  # Sm3/s -> Mscf/d
        QWST = QWST * 86400 * 6.28981077  # Sm3/s -> STB/d
        PT = PT / 1e5 * 14.5  # Pa -> psia
        pressure_axis = "Pressure $[psia]$"
        water_axis = "Water Production Rate $[STB/D]$"
        gas_axis = "Gas Production Rate $[Mscf/D]$"
        holdup_axis = "Fraction of Liquid in Tubing, $V_l/V_t$ $[\%]$"
        height_axis = "Height of Liquid $[ft]$"
        unit_type = 1
    else:
        QGST = QGST * 86400  # Sm3/s -> Sm3/d
        QWST = QWST * 86400  # Sm3/s -> Sm3/d
        PT = PT / 1e5  # Pa -> bara
        pressure_axis = "Pressure $[bara]$"
        water_axis = "Water Production Rate $[Sm^3/d]$"
        gas_axis = "Gas Production Rate $[Sm^3/d]$"
        holdup_axis = "Fraction of Liquid in Tubing, $V_l/V_t$ $[\%]$"
        height_axis = "Equivalent Liquid Height $[m]$"
        unit_type = 2

    Z = np.array([gas.ZfacStanding(n, T_R, SG_gas, unit_type) for n in PT])
    density_gas = np.array(
        [gas.deng(PT[n], T_R, Z[n], SG_gas, unit_type) for n in range(len(Z))]
    )  # kg/m3
    Bg = np.array(
        [gas.Bg(PT[n], T_R, Z[n], unit_type) for n in range(len(Z))]
    )  # m3/Sm3
    # Qg_inj = np.array([QGinj[l]] * len(PT))  # kg/s

    # Qw_inj = np.array(WGR[ll]) * Qg_inj  # kg/s

    if units == "field":
        # QGST_inj = Qg_inj / density_gas * 86400 * 35.3147 / 1000 / Bg  # mscf/d
        Bw = np.array(
            [
                water.water_properties(PT[n] / 14.5, (T_R - 32) / 1.8, T0)
                for n in range(len(PT))
            ]
        )  # m3/Sm3
        # QWST_inj = (Qw_inj * 86400 / 158.987)  # *Bw # bbl/d this is not correct .....................
    else:
        # QGST_inj = Qg_inj / density_gas * 86400 / Bg  # Sm3/d
        Bw = np.array(
            [water.water_properties(PT[n], T_R, T0) for n in range(len(PT))]
        )  # m3/Sm3
        denw = (
            np.array([water.water_properties(PT[n], T_R, T0) for n in range(len(PT))])
            * 1000
        )
        # QWST_inj = (Qw_inj / denw * 86400 / Bw)  # *Bw # bbl/d this is not correct .................

    ppl_time, ID, UG, UL, DPZA, DPZF, DPZG, HOL, PTb = dpp.getPPLData(
        path_of_directory + "/", ppl_files[i]
    )
    # idx_ppl += 1

    # find slug
    dl_slug = ID[0][1] - ID[0][0]
    get_indexes = lambda x, xs: [z for (y, z) in zip(xs, range(len(xs))) if x == y]

    slug_0 = -1
    slug_0 = min(find_slug(ID[1], 3)[0], find_slug(ID[1], 4)[0])
    if slug_0 == 0:
        slug_0 = -1

    slug_flow = []
    bubbly_flow = []
    # indexes = get_indexes(3,ID[1][1])
    for step in ID[1]:
        idx3 = get_indexes(3, step)
        idx4 = get_indexes(4, step)
        if idx4 == []:
            bubbly_flow.append(0)
            if idx3 == []:
                slug_flow.append(0)
            else:
                slug_flow.append(len(idx3) * dl_slug)
        else:
            bubbly_flow.append(len(idx4) * dl_slug)
            if idx3 == []:
                slug_flow.append(0)
            else:
                slug_flow.append(len(idx3) * dl_slug)

    flow_regime = np.array(slug_flow) + np.array(bubbly_flow)
    # def find_indices(list_to_check, item_to_find):
    #     array = np.array(list_to_check)
    #     indices = np.where(array == item_to_find)[0]
    #     return indices

    # statistical calcs ##### ADD FFT

    # PT_std = np.std(PT)
    # PT_avg = np.average(PT)
    # PT_avgtrans = np.average(PT[0 : k + 1])
    # PT_avgss = np.average(PT[k:])

    QWAS_std = np.std(QWST[slug_0:])  # after slug initialization
    QWAS_avg = np.average(QWST[slug_0:])  # after slug initialization
    QWBS_std = np.std(QWST[3 : slug_0])  # before slug initialization
    QWBS_avg = np.average(QWST[3 : slug_0])  # before slug initialization

    QGAS_std = np.std(QGST[slug_0:])  # after slug initialization
    QGAS_avg = np.average(QGST[slug_0:])  # after slug initialization
    QGBS_std = np.std(QGST[3 : slug_0])  # before slug initialization
    QGBS_avg = np.average(QGST[3 : slug_0])  # before slug initialization

    WGR_avg = QWST / QGST
    # WGR_avgss = QWST_avgss / QGST_avgss
    # WGR_avgtrans = QWST_avgtrans / QGST_avgtrans

    if constantpressuresim == "y":
        if QGBS_avg == "nan":
            QgTPR.append(np.average(QGST[k : 5 + k]))
            PTTPR.append(np.average(PT[k : 5 + k]))
            # QgTPR.append(np.average(QGST[int(len(QGST)/5):int(2*len(QGST)/2)]))
            # PTTPR.append(np.average(PT[int(len(QGST)/5):int(2*len(QGST)/3)]))
            end_stable_idx = False # -1
            end_stable_rate = False # QGST[4:-2]*86400
            end_stable_PT = False # PT[4:-2]*1e-5
            # QgTPR.append(end_stable_rate)
            # PTTPR.append(end_stable_PT)
            crit_idx_plot = [[False]] # np.where(end_stable_PT == end_stable_PT.min())
            crit_rate_plot = False # QGST[crit_idx_plot][0]*86400
            idx_crit = False
        else:
            QgTPR.append(QGAS_avg)
            PTTPR.append(np.average(PT[slug_0:]))
            end_stable_idx = False # -1
            end_stable_rate = False # QGST[4:-2]*86400
            end_stable_PT = False # PT[4:-2]*1e-5
            # QgTPR.append(end_stable_rate)
            # PTTPR.append(end_stable_PT)
            crit_idx_plot = [[False]] # np.where(end_stable_PT == end_stable_PT.min())
            crit_rate_plot = False # QGST[crit_idx_plot][0]*86400
            idx_crit = False
    try:
        nodal_point = so.root(nodal_point_solver, [np.average(end_stable_PT[200:-20]), np.average(end_stable_rate[200:-20])])
        if nodal_point["success"] == True:
            Pwf_nodal = nodal_point["x"][0]
            Qg_nodal = nodal_point["x"][1]
        else:
            Qg_nodal = "Does not exist"
            Pwf_nodal = "Does not exist"
    except:
        Pwf_nodal = False
        Qg_nodal = False

    if units == "field":
        # VW_acc = (VW_inj - VW_prod) + LIQC[0]  # bbl
        # HW_acc = VW_acc * 5.614583333 / (np.pi / 4 * (12 * ID_tubing) ** 2)  # ft
        HW_liqc = LIQC / (np.pi / 4 * (12 * ID_tubing[i]) ** 2)
        VW_LIQC = LIQC[-1]  # bbl
        # holdup_calcfrac = VW_acc[-1] / (V_tubing * 6.28981077)
        holdup_simfrac = LIQC / (V_tubing[i] * 6.28981077)
        DPZF = DPZF[-1][0:-1][-1] * 1e-5 * 14.5
        DPZG = DPZG_[-1][0:-1][-1] * 1e-5 * 14.5
        DPZA = DPZA[-1][0:-1][-1] * 1e-5 * 14.5
        PT_branch = PTb[1][-1] * 1e-5 * 14.5
        dpzg = np.array([DPZG_[1][i][-1] for i in range(len(time))]) * 1e-5 * 14.5
        Z1 = np.array([gas.ZfacStanding(n, T_R, SG_gas, unit_type) for n in PT_branch])
        deng = np.array(
            [
                gas.deng(PT_branch[n], T_R, Z1[n], SG_gas, unit_type)
                for n in range(len(Z1))
            ]
        )
        deng_avg = np.average(deng) * 0.008345404265
        h_DPZG = (
            dpzg * 144 / 32.174 - deng_avg * 0.008345404265 * L_tubing * 3.28 * 7.48
        ) / ((62.4 - deng_avg * 0.008345404265) * 7.48)
        # Qgc = cv.Qc(PT_avgss, 1.01325, T_R, T0, SG_gas, SG_water, ID_tubing, 1)

        columns1 = [
            "Time [s]",
            "Holdup [-] (sim)",
            # "Injected QGST [Mscf/d] (calc)",
            "Produced QGST [Mscf/d] (sim)",
            # "Injected QWST [STB/d] (calc)",
            "Produced QWST [STB/d] (sim)",
            "BHP [psia] (sim)",
        ]

        columns2 = [
            "Case",
            "Time until Steady State [s]",
            "PT_std [psia]",
            "PT_avg [psia]",
            "PT_avg at SS [psia]",
            "PT_avg at TS [psia]",
            "Integrated Gravity Pressure Drop [psia]",
            "Integrated Friction Pressure Drop [psia]",
            "Integrated Additional Pressure Drop [psia]",
            "QWST_std [STB/d]",
            "QWST_avg [STB/d]",
            "QWST_avg at SS [STB/d]",
            "QWST_avg at TS [STB/d]",
            "QGST_std [Mscf/d]",
            "QGST_avg [Mscf/d]",
            "QGST_avg at SS [Mscf/d]",
            "QGST_avg at TS [Mscf/d]",
            "WGR_avg [STB/Mscf]",
            "WGR_avg at SS [STB/Mscf]",
            "WGR_avg at TS [STB/Mscf]",
            "Holdup fraction [-] (calc)",
            "Holdup fraction [-] (sim)",
            "Water Volume Accumulated [STB]",
            "LIQC [STB]",
            "Liquid Column Heigh from Accumulated volumes [ft]",
            "Liquid Column Heigh from LIQC [ft]",
            "Liquid Column Heigh from DPZG [ft]",
            "Average Gas Density [kg/m3]",
            "Turner Critical Rate at SC [Mscf/d]",
        ]
    else:
        HW_liqc = LIQC / (np.pi / 4 * (0.0254 * ID_tubing[i]) ** 2)
        VW_LIQC = LIQC[-1]  # m3
        holdup_simfrac = LIQC / V_tubing[i]
        # DPZF = DPZF[-1][0:-1][-1] * 1e-5
        # DPZG = DPZG_[-1][0:-1][-1] * 1e-5
        # DPZA = DPZA[-1][0:-1][-1] * 1e-5
        PT_branch = PTb[1][-1] * 1e-5

        dpzg = np.array([DPZG[1][t][-1] for t in range(len(time))]) * 1e-5
        dpzf = np.array([DPZF[1][t][-1] for t in range(len(time))]) * 1e-5
        dpza = np.array([DPZA[1][t][-1] for t in range(len(time))]) * 1e-5

        PTavg = [np.average(PTb[1][i] * 1e-5) for i in range(len(time))]
        Z1 = np.array([gas.ZfacStanding(n, T_R, SG_gas, unit_type) for n in PTavg])
        deng = np.array(
            [gas.deng(p, T_R, Z1[n], SG_gas, unit_type) for n, p in enumerate(PTavg)]
        )
        deng_avg = np.average(deng)

        Bw = [water.Bw1(pavg, T_R) for pavg in PTavg]
        Rsw = water.Rsw(PTavg)
        denw = (
            np.array(
                [
                    water.denw1(SG_water, SG_gas, Bw[i], Rsw[i])
                    for i, val in enumerate(PTavg)
                ]
            )
            * 1000
        )
        denw_avg = np.average(denw)  # average of liquid along branch a

        h_DPZG = (dpzg * 1e5 / 9.81 - deng * L_tubing) / (denw - deng)

        p_turner = [
            P_wh,
            PTb[1][10][0] * 1e-5,
            PTb[1][k][0] * 1e-5,
            PTb[1][slug_0][0] * 1e-5,
        ]
        Qgc = [
            cv.Qc(pturner, 1.01325, T_R, T0, SG_gas, SG_water, ID_tubing[i], 2) for pturner in p_turner
        ]

        columns1 = [
            "Time [s]",
            "Holdup [-] (sim)",
            # "Injected QGST [Sm3/d] (calc)",
            "Produced QGST [Sm3/d] (sim)",
            # "Injected QWST [Sm3/d] (calc)",
            "Produced QWST [Sm3/d] (sim)",
            "BHP [bara] (sim)",
        ]

        columns2 = [
            "Case",
            "Qg_nodal",
            "Pwf_nodal",
            "Index of Liquid Accumulation",
            "Index of Lowest Pressure",
            "Index of First Slug",
            "Index of Final Stable Iteration",
            "Time until Liquid Loading [s]",
            "Time until First Slug [s]",
            "Time until Final Stable Iteration [s]",
            "QWST_std after slug initiation [Sm3/d]",
            "QWST_avg after slug initiation [Sm3/d]",
            "QWST_std before slug initiation [Sm3/d]",
            "QWST_avg before slug initiation [Sm3/d]",
            "QGST_std after slug initiation [Sm3/d]",
            "QGST_avg after slug initiation [Sm3/d]",
            "QGST_std before slug initiation [Sm3/d]",
            "QGST_avg before slug initiation [Sm3/d]",
            "Holdup fraction [-] (sim)",
            "LIQC [m3]",
            "Liquid Column Heigh from LIQC [m]",
            "Liquid Column Heigh from DPZG [m]",
            "Average Gas Density [kg/m3]",
            "Gas Rate at Slug Initialization [Sm3/d]",
            "Critical Rate from Plot [Sm3/d]",
            "Critical Rate from Bottom of TPR Curve [Sm3/d]",
            "Turner Critical Rate using Pwh [Sm3/d]",
            "Turner Critical Rate using Pwf [Sm3/d]",
            "Turner Critical Rate using Pwf at Tc (OLGA) [Sm3/d]",
            "Turner Critical Rate using Pwf at Tslug (OLGA) [Sm3/d]",
        ]

        columns3 = [
            "Time [d]",
            "Height from Gravitational Pressure Drop [m]",
            "Height from LIQC [m]",
            "Pwf [bara]",
            "DPZG [bara]",
            "DPZF [bara]",
            "DPZA [bara]",
            "Average Pressure [bara]",
            "Z - factor",
            "Gas Density [kg/m3]",
            "Bw [m3/Sm3]",
            "Rsw [Sm3/Sm3]",
            "water Density [kg/m3]",
        ]

    # Data export to Excel
    data = [
        time,
        holdup_simfrac,
        # QGST_inj,
        QGST,
        # QWST_inj,
        QWST,
        PT,
    ]
    outputdata = pd.DataFrame(
        columns=columns1,
        data=np.transpose(data),
    )

    data2 = [
        file[0:2],
        Qg_nodal,
        Pwf_nodal,
        idx_crit,
        crit_idx_plot[0][0],
        slug_0,
        end_stable_idx,
        time[idx_crit],
        time[slug_0],
        time[end_stable_idx],
        QWAS_std,
        QWAS_avg,
        QWBS_std,
        QWBS_avg,
        QGAS_std,
        QGAS_avg,
        QGBS_std,
        QGBS_avg,
        holdup_simfrac[-1],
        VW_LIQC,
        HW_liqc[-1],
        h_DPZG[-1],
        deng_avg,
        QGST[slug_0],
        QGST[min(k, slug_0)],
        crit_rate_plot,
        Qgc[0][0],
        Qgc[1][0],
        Qgc[2][0],
        Qgc[3][0],
    ]

    data3 = [
        time / 86400,
        h_DPZG,
        HW_liqc,
        PT,
        dpzg,
        dpzf,
        dpza,
        PTavg,
        Z1,
        deng,
        Bw,
        Rsw,
        denw,
    ]
    outputdata3 = pd.DataFrame(
        columns=columns3,
        data=np.transpose(data3),
    )

    lst.append(data2)

    if i == 0:  # Write trend data to excel
        writer = pd.ExcelWriter("LL_IPR.xlsx", engine="xlsxwriter")
        outputdata.to_excel(writer, sheet_name="Case " + file[0:2] + ", Trend data")
        outputdata3.to_excel(writer, sheet_name="Case " + file[0:2] + ", Profile data")
        writer.save()
        # writer.close()
        # j += 1
    else:
        # j += 1
        book = load_workbook("LL_IPR.xlsx")
        writer = pd.ExcelWriter("LL_IPR.xlsx", engine="openpyxl")
        writer.book = book
        outputdata.to_excel(writer, sheet_name="Case " + file[0:2] + ", Trend data")
        outputdata3.to_excel(writer, sheet_name="Case " + file[0:2] + ", Profile data")
        writer.save()

    # plots
    fsize = 16
    tsize = 18
    tdir = "out"  # "in" "out" "inout"
    major = 5
    minor = 3
    style = "default"  # plt.style.available
    xsize = 8
    ysize = 5

    plt.style.use(style)
    plt.rcParams["text.usetex"] = True
    plt.rcParams["font.size"] = fsize
    plt.rcParams["legend.fontsize"] = fsize
    plt.rcParams["legend.title_fontsize"] = tsize
    plt.rcParams["xtick.direction"] = tdir
    plt.rcParams["ytick.direction"] = tdir
    plt.rcParams["xtick.major.size"] = major
    plt.rcParams["xtick.minor.size"] = minor
    plt.rcParams["ytick.major.size"] = major
    plt.rcParams["ytick.minor.size"] = minor

    if k == -1:
        plot_list = [-1]
    else:
        plot_list = [slug_0, -1]

    try:

        for iii in plot_list:  # [time_idx, -1]:  # plotting
            fig, ax = plt.subplots(figsize=(xsize, ysize))
            twin1 = ax.twinx()
            (p1,) = ax.plot(
                time[:iii] / 86400, QGST[:iii], "r-", label="QGST", linewidth=2
            )
            # plt.fill_between(time/86400, QGST-QGST_std, QGST+QGST_std, alpha=0.5)
            (p2,) = twin1.plot(
                time[:iii] / 86400, QWST[:iii], "b-", label="QWST", linewidth=1
            )
            twin1.set_ylabel(water_axis)  # , fontsize=fsize)
            ax.set_ylabel(gas_axis, labelpad=10)
            ax.set_xlabel("Time $[d]$", labelpad=10)
            ax.yaxis.label.set_color(p1.get_color())
            twin1.yaxis.label.set_color(p2.get_color())
            twin1.tick_params(axis="y", color=p2.get_color())
            ax.legend(handles=[p1, p2])
            mpl_axes_aligner.align.yaxes(ax, 0, twin1, 0)
            # plt.xlabel("Time $[D]$", labelpad=10)
            # plt.ylabel("Production rate $[Mscf/D]$", labelpad=10)
            plt.title("Rate of Produced Fluids, " + "Case " + file[0:2])
            plt.savefig(
                "plots/" + "Case " + file[0:2] + f" timestep {iii} " + " prodrate.png",
                bbox_inches="tight",
            )

            fig, ax = plt.subplots(figsize=(xsize, ysize))
            twin1 = ax.twinx()
            (p1,) = ax.plot(
                time[1:iii] / 86400, PT[1:iii], "k-", label="Pressure", linewidth=3
            )
            # plt.fill_between(time/86400, QGST-QGST_std, QGST+QGST_std, alpha=0.5)
            (p2,) = twin1.plot(
                time[1:iii] / 86400,
                holdup_simfrac[1:iii] * 100,
                "g-",
                label="Liquid Fraction",
                linewidth=2,
            )
            twin1.set_ylabel(holdup_axis)  # , fontsize=fsize)
            ax.set_ylabel(pressure_axis, labelpad=10)
            ax.set_xlabel("Time $[d]$", labelpad=10)
            ax.yaxis.label.set_color(p1.get_color())
            twin1.yaxis.label.set_color(p2.get_color())
            twin1.tick_params(axis="y", color=p2.get_color())
            ax.legend(handles=[p1, p2])
            plt.title(
                "Fraction of Accumulated Liquids and Bottom Hole Pressure Over Time, "
                + "Case "
                + file[0:2]
            )
            plt.savefig(
                "plots/"
                + "Case "
                + file[0:2]
                + f" timestep {iii} "
                + " pressureholdup.png",
                bbox_inches="tight",
            )

            fig, ax = plt.subplots(figsize=(xsize, ysize))
            # twin1 = ax.twinx()
            # (p1,) = plt.plot(
            #     time[:iii] / 86400,
            #     HW_acc[:iii],
            #     "r:",
            #     label="H from integrated QWST",
            #     linewidth=5,
            # )
            (p2,) = plt.plot(
                time[:iii] / 86400,
                HW_liqc[:iii],
                "b-",
                label="H from Liquid Content in the Tubing",
            )
            # (p3,) = plt.plot(
            #     np.array(ppl_time[:iii]) / 86400,
            #     h_DPZG[:iii],
            #     "g-",
            #     label="H from Gravitational Pressure Drop",
            # )
            # twin1.set_ylabel("Height of Liquid $[m]$") #, fontsize=fsize)
            ax.set_ylabel(height_axis, labelpad=10)
            ax.set_xlabel("Time $[d]$", labelpad=10)
            # ax.yaxis.label.set_color(p1.get_color())
            # twin1.yaxis.label.set_color(p2.get_color())
            # twin1.tick_params(axis="y", color=p2.get_color())
            ax.legend(handles=[p2])#, p3])
            # plt.xlabel("Time $[D]$", labelpad=10)
            # plt.ylabel("Production rate $[Mscf/D]$", labelpad=10)
            plt.title(
                "Equivalent Height of Liquid Column Over Time, " + "Case " + file[0:2]
            )
            plt.savefig(
                "plots/" + "Case " + file[0:2] + f" timestep {iii} " + " LiquidHeight.png",
                bbox_inches="tight",
            )

            fig, ax = plt.subplots(figsize=(xsize, ysize))
            flow_regime_convolved = np.convolve(flow_regime, kernel, mode="same")
            plt.plot(
                time[:iii] / 86400,
                flow_regime_convolved[:iii],
                "b-",
                label=f"Slug and Bubbly Flow",
                linewidth=2,
            )
            # plt.plot(
            #     np.linspace(0, time[-1], len(flow_regime)) / 86400,
            #     slug_flow,
            #     "r-",
            #     label=f"Slug Flow",
            #     linewidth=1,
            # )
            # plt.plot(
            #     np.linspace(0, time[-1], len(flow_regime)) / 86400,
            #     bubbly_flow,
            #     "g-",
            #     label=f"Bubbly Flow",
            #     linewidth=3,
            # )
            ax.set_ylabel("Length of Flow Regime, $[m]$ ", labelpad=10)
            ax.set_xlabel("Time $[d]$", labelpad=10)
            plt.title("Combined Length of Slug and Bubbly flow vs Time")
            ax.legend()
            plt.savefig(
                "plots/" + "Case " + file[0:2] + f" timestep {iii} loaded flow.png",
                bbox_inches="tight",
            )
    except:
        print(f"unable to plot case {file[0:2]}, step {iii}")

    plt.close("all")

    # plt.show()


# %%

# low pass filter
# kernel_size = 30
# kernel = np.ones(kernel_size) / kernel_size
# data_convolved = np.convolve(flow_regime, kernel, mode='same')

# plt.plot(np.linspace(0,time[-1], len(flow1))/86400, data_convolved)


#%% TPR - IPR plot
# P_R = 70
P_wf = np.linspace(0, P_R, 100)
QgIPR = [backpressure(P_R, wf, c, n) for wf in P_wf]
try: 
    plt.rcParams["legend.fontsize"] = 10
    if constantpressuresim == "n":
        fig, ax = plt.subplots(figsize=(xsize*1.3, ysize*1.1))
        ax.plot(QgIPR, P_wf, "r", label="IPR")
        for i, file in enumerate(tpl_files):
            # time, LIQC, QGST, QWST, PT = dpp.getTPLData(path_of_directory + "/", file)

            # QGST = 86400 * QGST
            # PT = 1e-5 * PT
            kernel_size = 20
            kernel = np.ones(kernel_size) / kernel_size
            # QGST = np.convolve(QGST, kernel, mode="same")
            # PT = np.convolve(PT, kernel, mode="same")

            # idx = find_nearest(QGST[2:], 5500)[0]
            # qg0 = find_nearest(QGST[2:], max(QgIPR))[0]

            # ax.plot(QGST[1:], PT[1:], label=f"TPR, WGR = {WGR[i]}")
            x = np.convolve(QgTPR[i][:-3], kernel, mode="same") #[QgTPR[i]<max(QgIPR)]
            y = np.convolve(PTTPR[i][:len(x)], kernel, mode="same")
            ax.plot(x[15:-15], y[15:-15], label=f"TPR, {test_property} = {WGR[i]} {test_property_unit}")
        plt.xlim(0, max(QgIPR) * 1.2)
        plt.ylim(0, max(P_wf) * 1.2)
        ax.set_xlabel("Gas Production Rate, $QGST$ $[Sm^3/d]$")
        ax.set_ylabel("Flowing Bottomhole Pressure, $P_{wf}$ $[bara]$")
        ax.set_title("Nodal Analysis of IPR and TPR")
        ax.grid()
        ax.legend()
        plt.savefig(
            "plots/" + f"IPR and TPR.png",
            bbox_inches="tight",
        )

    else:
        fig, ax = plt.subplots(figsize=(xsize, ysize))
        ax.plot(QgIPR, P_wf, "r", label="IPR")
        for i, wgr in enumerate(WGR[::WGR_step]):
            ax.plot(
                QgTPR[i * WGR_step : (i + 1) * WGR_step],
                PTTPR[i * WGR_step : (i + 1) * WGR_step],
                label=f"TPR, {test_property} = {wgr} {test_property_unit}",
            )
        # plt.xlim(0,max(QgIPR)*1.2)
        # plt.ylim(0,max(P_wf)*1.2)
        ax.set_xlabel("Gas Production Rate, $QGST$ $[Sm^3/d]$")
        ax.set_ylabel("Flowing Bottomhole Pressure, $P_{wf}$ $[bara]$")
        ax.set_title("Nodal Analysis of IPR and TPR. $P_{wh}$ " + f"= {P_wh}")
        ax.grid()
        ax.legend()
        plt.savefig(
            "plots/" + f"IPR and TPR.png",
            bbox_inches="tight",
        )
except:
    print("Unable to plot Nodal plot")

# %%
book = load_workbook("LL_IPR.xlsx")  # write statdata to excel
writer = pd.ExcelWriter("LL_IPR.xlsx", engine="openpyxl")
writer.book = book
statdata = pd.DataFrame(lst, columns=columns2)
statdata.to_excel(writer, sheet_name="Summary Data")
writer.save()
writer.close()


# from scipy.interpolate import interp1d
# P_R = 80
# P_wf = np.linspace(0, P_R, 100)
# QgIPR = [backpressure(P_R, i, c, n) for i in P_wf]
# WGR = 5*[1e-6]+5*[7.5e-5]
# fig,ax = plt.subplots(figsize=(xsize,ysize))
# ax.plot(QgIPR, P_wf, "r", label="IPR")

# for i,wgr in enumerate(WGR[::5]):

#     x = np.array(QgTPR[i*5:(i+1)*5])
#     # x = x[x>0]
#     # x = np.where(x<0, 0, x)
#     y = np.array(PTTPR[i*5:(i+1)*5])
#     y = y[y<100]
#     x = x[:len(y)]
#     # y = y[:len(x)]
#     # y = np.where(y>90, 90, y)

#     try:
#         X_Y_spline = interp1d(x, y, kind="quadratic")
#         X_ = np.linspace(x.min(), x.max(), 50)
#         Y_ = X_Y_spline(X_)

#         ax.plot(X_,Y_, label=f"TPR, WGR = {wgr}")
#     except:
#         print("lol")
#     # break

# plt.xlim(0,max(QgIPR)*1.2)
# plt.ylim(0,max(P_wf)*1.2)
# ax.set_xlabel("Gas Production Rate, $QGST$ $[Sm^3/d]$")
# ax.set_ylabel("Flowing Bottomhole Pressure, $P_{wf}$ $[bara]$")
# ax.set_title("Nodal Analysis of IPR and TPR")
# ax.grid()
# ax.legend()


# from scipy.interpolate import interp1d
# from sklearn.preprocessing import PolynomialFeatures
# from sklearn.linear_model import LinearRegression

# degree = 2
# P_R = 80
# P_wf = np.linspace(0, P_R, 100)
# QgIPR = [backpressure(P_R, i, c, n) for i in P_wf]

# WGR = 5*[1e-6]+5*[7.5e-5]

# fig,ax = plt.subplots(figsize=(xsize,ysize))
# ax.plot(QgIPR, P_wf, "r", label="IPR")

# for i,wgr in enumerate(WGR[::5]):

#     x = np.array(QgTPR[i*5:(i+1)*5])
#     # x = x[x>0]
#     # x = np.where(x<0, 0, x)
#     y = np.array(PTTPR[i*5:(i+1)*5])
#     y = y[y<100]
#     x = x[:len(y)]
#     # y = np.where(y>90, 90, y)

#     # X_Y_spline = interp1d(x, y, kind="cubic")
#     # X_ = np.linspace(x.min(), x.max(), 50)
#     # Y_ = X_Y_spline(X_)

#     poly = PolynomialFeatures(degree, include_bias=False)
#     poly_features = poly.fit_transform(x.reshape(-1,1))
#     poly_reg_model = LinearRegression()
#     poly_reg_model.fit(poly_features, y)
#     y_predicted = poly_reg_model.predict(poly_features)
#     xx = np.linspace(x.min(), x.max(), 50).reshape(-1,1)
#     X_ = poly.fit_transform(xx)
#     Y_ = poly_reg_model.predict(X_)
#     # plt.scatter(x,y)
#     # plt.plot(x,y_predicted, c="red")

#     ax.plot(X_,Y_, label=f"TPR, WGR = {wgr}")
#     # break

# plt.xlim(0,max(QgIPR)*1.2)
# plt.ylim(0,max(P_wf)*1.2)
# ax.set_xlabel("Gas Production Rate, $QGST$ $[Sm^3/d]$")
# ax.set_ylabel("Flowing Bottomhole Pressure, $P_{wf}$ $[bara]$")
# ax.set_title("Nodal Analysis of IPR and TPR")
# ax.grid()
# ax.legend()


#  P_wf = np.linspace(0, P_R, 100)
#             QgIPR = [backpressure(P_R, wf, c, n) for wf in P_wf]

#             plt.rcParams["legend.fontsize"] = 10

#             if constantpressuresim == "n":
#                 if i == 0:
#                     fig, ax = plt.subplots(figsize=(xsize, ysize))
#                     ax.plot(QgIPR, P_wf, "r", label="IPR")
#                 else:
#                     ax.plot(end_stable_rate, end_stable_PT, label=f"TPR, WGR = {WGR[i]}")
#                 plt.xlim(0, max(QgIPR) * 1.2)
#                 plt.ylim(0, max(P_wf) * 1.2)
#                 ax.set_xlabel("Gas Production Rate, $QGST$ $[Sm^3/d]$")
#                 ax.set_ylabel("Flowing Bottomhole Pressure, $P_{wf}$ $[bara]$")
#                 ax.set_title("Nodal Analysis of IPR and TPR")
#                 ax.grid()
#                 ax.legend()
#                 plt.savefig(
#                     "plots/" + f"IPR and TPR.png",
#                     bbox_inches="tight",
#                 )

#             else:
#                 if i == 0:
#                     fig, ax = plt.subplots(figsize=(xsize, ysize))
#                     ax.plot(QgIPR, P_wf, "r", label="IPR")
#                 else:
#                     for i, wgr in enumerate(WGR[::WGR_step]):
#                         ax.plot(
#                             QgTPR[i * WGR_step : (i + 1) * WGR_step],
#                             PTTPR[i * WGR_step : (i + 1) * WGR_step],
#                             label=f"TPR, WGR = {wgr}",
#                         )
#                 plt.xlim(0,max(QgIPR)*1.2)
#                 plt.ylim(0,max(P_wf)*1.2)
#                 ax.set_xlabel("Gas Production Rate, $QGST$ $[Sm^3/d]$")
#                 ax.set_ylabel("Flowing Bottomhole Pressure, $P_{wf}$ $[bara]$")
#                 ax.set_title("Nodal Analysis of IPR and TPR")
#                 ax.grid()
#                 ax.legend()
#                 plt.savefig(
#                     "plots/" + f"IPR and TPR.png",
#                     bbox_inches="tight",
#                 )