#%%
import os
import numpy as np

# import sklearn as sklearn
# import scipy.optimize as sciop
# import scipy.integrate as scint
import pandas as pd
import pyfas as fa
import matplotlib.pyplot as plt
from libraries import gas
from libraries import water
from libraries import datapreprocessing as dpp
from libraries import criticalvelocity as cv
from openpyxl import load_workbook

# from sklearn.preprocessing import PolynomialFeatures
# from sklearn.linear_model import LinearRegression

# Load simulation data, input data

L_tubing = 3000  # m, do not change unit
ID_tubing = 2.25  # in, do not change unit

units = "metric"  # "field"

T0 = 15.56  # standrd temp, degC
T_R = 70  # reserv temp degC


WGR = [
    0.020909091,
    0.016727273,
    0.012545455,
    0.008363636,
    0.004181818,
]  # from injection rates
QGinj = [0.11, 0.105, 0.1, 0.098, 0.093, 0.087, 0.08, 0.01]
P_wh = [25] * len(QGinj * len(WGR))  # psia


SG_gas = 0.609
SG_water = 1.00

j = 0
l = 0
ll = 0
idx_ppl = 0
degree = 7  # polynomial degree
tol = 8e-13

V_tubing = L_tubing * np.pi / 4 * (0.0254 * ID_tubing) ** 2  # m3

tpl_files = []
ppl_files = []

lst = []

# path_of_directory = "OLGA - G1W1"
#%%
path_of_directory = str(input("Type in the name of the simulation data folder:"))
if path_of_directory == "":
    path_of_directory = "OLGA"
ext = ".tpl"
for files in os.listdir(path_of_directory):
    if files.endswith(".tpl"):
        tpl_files.append(files)
    elif files.endswith(".ppl"):
        ppl_files.append(files)
    else:
        continue


for i in tpl_files[0:1]:
    time, LIQC, QGST, QWST, PT = dpp.getTPLData(
        path_of_directory + "/", i
    )  # metric units, s, Sm3/s, Pa

    try:
        for k in range(len(time)):
            if np.abs(QWST[k + 1] - QWST[k]) / ((time[k + 1] - time[k])) < tol:
                stabilization_time = time[k]
                stabilization_rate = QWST[k]
                if (
                    np.abs(stabilization_rate - QWST[-1]) / ((time[k + 1] - time[k]))
                    < tol
                ):
                    break
    except:
        print("Not stable flow")
        stabilization_time = "Not stable flow"
        stabilization_rate = "Not stable flow"

    def find_nearest(array, value):
        array = np.asarray(array)
        idx = (np.abs(array - value)).argmin()
        return idx, array[idx]

    if stabilization_time == "Not stable flow":
        time_idx = -1
    else:
        time_idx = find_nearest(time, stabilization_time)[0]
    # time_value = find_nearest(time, 4 * 86400)[1] / 86400  # days
    # PVT

    if units == "field":
        LIQC = LIQC * 6.28981077  # m3 -> bbl
        QGST = QGST * 86400 * 35.3147 / 1000  # Sm3/s -> Mscf/d
        QWST = QWST * 86400 * 6.28981077  # Sm3/s -> STB/d
        PT = PT / 1e5 * 14.5  # Pa -> psia
        pressure_axis = "Pressure $[psia]$"
        water_axis = "Water Production Rate $[STB/D]$"
        gas_axis = "Gas Production Rate $[Mscf/D]$"
        holdup_axis = "Fraction of Liquid in Tubing, $V_l/V_t$ $[\%]$"
        height_axis = "Height of Liquid $[ft]$"
        unit_type = 1
    else:
        QGST = QGST * 86400  # Sm3/s -> Sm3/d
        QWST = QWST * 86400  # Sm3/s -> Sm3/d
        PT = PT / 1e5  # Pa -> bara
        pressure_axis = "Pressure $[bara]$"
        water_axis = "Water Production Rate $[Sm^3/D]$"
        gas_axis = "Gas Production Rate $[Sm^3/D]$"
        holdup_axis = "Fraction of Liquid in Tubing, $V_l/V_t$ $[\%]$"
        height_axis = "Equivalent Liquid Height $[m]$"
        unit_type = 2

    Z = np.array([gas.ZfacStanding(n, T_R, SG_gas, unit_type) for n in PT])
    density_gas = np.array(
        [gas.deng(PT[n], T_R, Z[n], SG_gas, unit_type) for n in range(len(Z))]
    )  # kg/m3
    Bg = np.array(
        [gas.Bg(PT[n], T_R, Z[n], unit_type) for n in range(len(Z))]
    )  # m3/Sm3
    Qg_inj = np.array([QGinj[l]] * len(PT))  # kg/s

    Qw_inj = np.array(WGR[ll]) * Qg_inj  # kg/s

    if units == "field":
        QGST_inj = Qg_inj / density_gas * 86400 * 35.3147 / 1000 / Bg  # mscf/d
        Bw = np.array(
            [
                water.water_properties(PT[n] / 14.5, (T_R - 32) / 1.8, T0)
                for n in range(len(PT))
            ]
        )  # m3/Sm3
        QWST_inj = (
            Qw_inj * 86400 / 158.987
        )  # *Bw # bbl/d this is not correct .....................
    else:
        QGST_inj = Qg_inj / density_gas * 86400 / Bg  # Sm3/d
        Bw = np.array(
            [water.water_properties(PT[n], T_R, T0) for n in range(len(PT))]
        )  # m3/Sm3
        denw = (
            np.array([water.water_properties(PT[n], T_R, T0) for n in range(len(PT))])
            * 1000
        )
        QWST_inj = (
            Qw_inj / denw * 86400 / Bw
        )  # *Bw # bbl/d this is not correct .................

    l += 1
    if l == len(QGinj) - 1:
        l = 0  # reset Qginj index
        ll += 1  # next WGR index
        if ll >= len(WGR):
            ll = len(WGR) - 1

    # total flow, accumulation

    VW_prod = np.array(
        [np.trapz(QWST[0:ii], time[0:ii] / 86400) for ii in range(len(time))]
    )
    VW_inj = np.array(
        [
            np.trapz([QWST[-1]] * len(time[:ii]), time[:ii] / 86400)
            for ii in range(len(time))
        ]
    )

    ppl_time, ID, UG, UL, DPZA, DPZF, DPZG_, HOL, PTb = dpp.getPPLData(
        path_of_directory + "/", ppl_files[idx_ppl]
    )
    idx_ppl += 1

    # ID plotting

    x = ID[0]
    ID_start = ID[1][0]
    ID_midtrans = ID[1][int(time_idx / 2)]
    ID_endtrans = ID[1][int(time_idx)]
    ID_end = ID[1][-1]

    # find slug
    dl_slug = ID[0][1]-ID[0][0]
    get_indexes = lambda x, xs: [i for (y, i) in zip(xs, range(len(xs))) if x == y]

    slug_flow = []
    bubbly_flow = []
    # indexes = get_indexes(3,ID[1][1])
    for step in ID[1]:
        idx3 = get_indexes(3,step)
        idx4 = get_indexes(4,step)
        if idx4 == []:
            bubbly_flow.append(0)
            if idx3 == []:
                slug_flow.append(0)
            else:
                slug_flow.append(len(idx3)*dl_slug)
        else:
            bubbly_flow.append(len(idx4) * dl_slug)

    flow_regime = np.array(slug_flow) + np.array(bubbly_flow)
    # def find_indices(list_to_check, item_to_find):
    #     array = np.array(list_to_check)
    #     indices = np.where(array == item_to_find)[0]
    #     return indices

    # statistical calcs ##### ADD FFT

    PT_std = np.std(PT)
    PT_avg = np.average(PT)
    PT_avgtrans = np.average(PT[0 : k + 1])
    PT_avgss = np.average(PT[k:])

    QWST_std = np.std(QWST)
    QWST_avg = np.average(QWST)
    QWST_avgss = np.average(QWST[k:])
    QWST_avgtrans = np.average(QWST[0 : k + 1])

    QGST_std = np.std(QGST)
    QGST_avg = np.average(QGST)
    QGST_avgss = np.average(QGST[k:])
    QGST_avgtrans = np.average(QGST[0 : k + 1])

    WGR_avg = QWST_avg / QGST_avg
    WGR_avgss = QWST_avgss / QGST_avgss
    WGR_avgtrans = QWST_avgtrans / QGST_avgtrans

    if units == "field":
        VW_acc = (VW_inj - VW_prod) + LIQC[0]  # bbl
        HW_acc = VW_acc * 5.614583333 / (np.pi / 4 * (12 * ID_tubing) ** 2)  # ft
        HW_liqc = LIQC / (np.pi / 4 * (12 * ID_tubing) ** 2)
        VW_LIQC = LIQC[-1]  # bbl
        holdup_calcfrac = VW_acc[-1] / (V_tubing * 6.28981077)
        holdup_simfrac = LIQC / (V_tubing * 6.28981077)
        DPZF = DPZF[-1][0:-1][-1] * 1e-5 * 14.5
        DPZG = DPZG_[-1][0:-1][-1] * 1e-5 * 14.5
        DPZA = DPZA[-1][0:-1][-1] * 1e-5 * 14.5
        PT_branch = PTb[1][-1] * 1e-5 * 14.5
        dpzg = np.array([DPZG_[1][i][-1] for i in range(len(time))]) * 1e-5 * 14.5
        Z1 = np.array([gas.ZfacStanding(n, T_R, SG_gas, unit_type) for n in PT_branch])
        deng = np.array(
            [
                gas.deng(PT_branch[n], T_R, Z1[n], SG_gas, unit_type)
                for n in range(len(Z1))
            ]
        )
        deng_avg = np.average(deng) * 0.008345404265
        h_DPZG = (
            dpzg * 144 / 32.174 - deng_avg * 0.008345404265 * L_tubing * 3.28 * 7.48
        ) / ((62.4 - deng_avg * 0.008345404265) * 7.48)
        Qgc = cv.Qc(PT_avgss, 1.01325, T_R, T0, SG_gas, SG_water, ID_tubing, 1)
        columns1 = [
            "Time [s]",
            "Holdup [-] (sim)",
            "Injected QGST [Mscf/d] (calc)",
            "Produced QGST [Mscf/d] (sim)",
            "Injected QWST [STB/d] (calc)",
            "Produced QWST [STB/d] (sim)",
            "BHP [psia] (sim)",
        ]
        columns2 = [
            "Case",
            "Time until Steady State [s]",
            "PT_std [psia]",
            "PT_avg [psia]",
            "PT_avg at SS [psia]",
            "PT_avg at TS [psia]",
            "Integrated Gravity Pressure Drop [psia]",
            "Integrated Friction Pressure Drop [psia]",
            "Integrated Additional Pressure Drop [psia]",
            "QWST_std [STB/d]",
            "QWST_avg [STB/d]",
            "QWST_avg at SS [STB/d]",
            "QWST_avg at TS [STB/d]",
            "QGST_std [Mscf/d]",
            "QGST_avg [Mscf/d]",
            "QGST_avg at SS [Mscf/d]",
            "QGST_avg at TS [Mscf/d]",
            "WGR_avg [STB/Mscf]",
            "WGR_avg at SS [STB/Mscf]",
            "WGR_avg at TS [STB/Mscf]",
            "Holdup fraction [-] (calc)",
            "Holdup fraction [-] (sim)",
            "Water Volume Accumulated [STB]",
            "LIQC [STB]",
            "Liquid Column Heigh from Accumulated volumes [ft]",
            "Liquid Column Heigh from LIQC [ft]",
            "Liquid Column Heigh from DPZG [ft]",
            "Average Gas Density [kg/m3]",
            "Turner Critical Rate at SC [Mscf/d]",
        ]
    else:
        VW_acc = (VW_inj - VW_prod) + LIQC[0]  # m3
        HW_acc = VW_acc / (np.pi / 4 * (0.0254 * ID_tubing) ** 2)  # m
        HW_liqc = LIQC / (np.pi / 4 * (0.0254 * ID_tubing) ** 2)
        VW_LIQC = LIQC[-1]  # m3
        holdup_calcfrac = VW_acc[-1] / V_tubing
        holdup_simfrac = LIQC / V_tubing
        DPZF = DPZF[-1][0:-1][-1] * 1e-5
        DPZG = DPZG_[-1][0:-1][-1] * 1e-5
        DPZA = DPZA[-1][0:-1][-1] * 1e-5
        PT_branch = PTb[1][-1] * 1e-5

        dpzg = np.array([DPZG_[1][i][-1] for i in range(len(time))]) * 1e-5
        Z1 = np.array([gas.ZfacStanding(n, T_R, SG_gas, unit_type) for n in PT_branch])
        deng = np.array(
            [
                gas.deng(PT_branch[n], T_R, Z1[n], SG_gas, unit_type)
                for n in range(len(Z1))
            ]
        )
        deng_avg = np.average(deng)

        Bw = [water.Bw1(i, T_R) for i in PT_branch]
        Rsw = water.Rsw(PT_branch)
        denw = (
            np.array(
                [
                    water.denw1(SG_water, SG_gas, Bw[i], Rsw[i])
                    for i, val in enumerate(PT_branch)
                ]
            )
            * 1000
        )
        denw_avg = np.average(denw)  # average of liquid along branch a

        h_DPZG = (dpzg * 1e5 / 9.81 - deng_avg * L_tubing) / (denw_avg - deng_avg)
        Qgc = cv.Qc(PT_branch[0], 1.01325, T_R, T0, SG_gas, SG_water, ID_tubing, 2)

        columns1 = [
            "Time [s]",
            "Holdup [-] (sim)",
            "Injected QGST [Sm3/d] (calc)",
            "Produced QGST [Sm3/d] (sim)",
            "Injected QWST [Sm3/d] (calc)",
            "Produced QWST [Sm3/d] (sim)",
            "BHP [bara] (sim)",
        ]
        columns2 = [
            "Case",
            "Time until Steady State [s]",
            "PT_std [bara]",
            "PT_avg [bara]",
            "PT_avg at SS [bara]",
            "PT_avg at TS [bara]",
            "Integrated Gravity Pressure Drop [bara]",
            "Integrated Friction Pressure Drop [bara]",
            "Integrated Additional Pressure Drop [bara]",
            "QWST_std [Sm3/d]",
            "QWST_avg [Sm3/d]",
            "QWST_avg at SS [Sm3/d]",
            "QWST_avg at TS [Sm3/d]",
            "QGST_std [Sm3/d]",
            "QGST_avg [Sm3/d]",
            "QGST_avg at SS [Sm3/d]",
            "QGST_avg at TS [Sm3/d]",
            "WGR_avg [Sm3/Sm3]",
            "WGR_avg at SS [Sm3/Sm3]",
            "WGR_avg at TS [Sm3/Sm3]",
            "Holdup fraction [-] (calc)",
            "Holdup fraction [-] (sim)",
            "Water Volume Accumulated [Sm3]",
            "LIQC [m3]",
            "Liquid Column Heigh from Accumulated volumes [m]",
            "Liquid Column Heigh from LIQC [m]",
            "Liquid Column Heigh from DPZG [m]",
            "Average Gas Density [kg/m3]",
            "Turner Critical Rate at SC [Sm3/d]",
        ]

    # Data export to Excel
    data = [time, holdup_simfrac, QGST_inj, QGST, QWST_inj, QWST, PT]
    outputdata = pd.DataFrame(
        columns=columns1,
        data=np.transpose(data),
    )

    data2 = [
        i[0:2],
        stabilization_time,
        PT_std,
        PT_avg,
        PT_avgss,
        PT_avgtrans,
        DPZG[-1],
        DPZF[-1],
        DPZA[-1],
        QWST_std,
        QWST_avg,
        QWST_avgss,
        QWST_avgtrans,
        QGST_std,
        QGST_avg,
        QGST_avgss,
        QGST_avgtrans,
        WGR_avg,
        WGR_avgss,
        WGR_avgtrans,
        holdup_calcfrac,
        holdup_simfrac[-1],
        VW_acc[-1],
        VW_LIQC,
        HW_acc[-1],
        HW_liqc[-1],
        h_DPZG[-1],
        deng_avg,
        Qgc[0],
    ]

    lst.append(data2)

    if j == 0:  # Write trend data to excel
        writer = pd.ExcelWriter("LL_source.xlsx", engine="xlsxwriter")
        outputdata.to_excel(writer, sheet_name="Case " + i[0:2] + ", trend data")
        writer.save()
        writer.close()
        j += 1
    else:
        j += 1
        book = load_workbook("LL_source.xlsx")
        writer = pd.ExcelWriter("LL_source.xlsx", engine="openpyxl")
        writer.book = book
        outputdata.to_excel(writer, sheet_name="Case " + i[0:2] + ", trend data")
        writer.save()

    # plots
    fsize = 16
    tsize = 18
    tdir = "out"  # "in" "out" "inout"
    major = 5
    minor = 3
    style = "default"  # plt.style.available
    xsize = 8
    ysize = 5

    plt.style.use(style)
    plt.rcParams["text.usetex"] = True
    plt.rcParams["font.size"] = fsize
    plt.rcParams["legend.fontsize"] = fsize
    plt.rcParams["legend.title_fontsize"] = tsize
    plt.rcParams["xtick.direction"] = tdir
    plt.rcParams["ytick.direction"] = tdir
    plt.rcParams["xtick.major.size"] = major
    plt.rcParams["xtick.minor.size"] = minor
    plt.rcParams["ytick.major.size"] = major
    plt.rcParams["ytick.minor.size"] = minor

    for iii in [time_idx, -1]:  # plotting
        fig, ax = plt.subplots(figsize=(xsize, ysize))
        twin1 = ax.twinx()
        (p1,) = ax.plot(time[:iii] / 86400, QGST[:iii], "r-", label="QGST")
        # plt.fill_between(time/86400, QGST-QGST_std, QGST+QGST_std, alpha=0.5)
        (p2,) = twin1.plot(time[:iii] / 86400, QWST[:iii], "b-", label="QWST")
        twin1.set_ylabel(water_axis)  # , fontsize=fsize)
        ax.set_ylabel(gas_axis, labelpad=10)
        ax.set_xlabel("Time $[D]$", labelpad=10)
        ax.yaxis.label.set_color(p1.get_color())
        twin1.yaxis.label.set_color(p2.get_color())
        twin1.tick_params(axis="y", color=p2.get_color())
        ax.legend(handles=[p1, p2])
        # plt.xlabel("Time $[D]$", labelpad=10)
        # plt.ylabel("Production rate $[Mscf/D]$", labelpad=10)
        plt.title("Rate of Produced Fluids, " + "Case " + i[0:2])
        plt.savefig(
            "plots/" + "Case " + i[0:2] + f" timestep {iii} " + " prodrate.png",
            bbox_inches="tight",
        )

        fig, ax = plt.subplots(figsize=(xsize, ysize))
        twin1 = ax.twinx()
        (p1,) = ax.plot(
            time[:iii] / 86400, PT[:iii], "k-", label="Pressure", linewidth=3
        )
        # plt.fill_between(time/86400, QGST-QGST_std, QGST+QGST_std, alpha=0.5)
        (p2,) = twin1.plot(
            time[:iii] / 86400,
            holdup_simfrac[:iii] * 100,
            "g-",
            label="Liquid Fraction",
            linewidth=2,
        )
        twin1.set_ylabel(holdup_axis)  # , fontsize=fsize)
        ax.set_ylabel(pressure_axis, labelpad=10)
        ax.set_xlabel("Time $[D]$", labelpad=10)
        ax.yaxis.label.set_color(p1.get_color())
        twin1.yaxis.label.set_color(p2.get_color())
        twin1.tick_params(axis="y", color=p2.get_color())
        ax.legend(handles=[p1, p2])
        plt.title(
            "Fraction of Accumulated Liquids and Bottom Hole Pressure Over Time, "
            + "Case "
            + i[0:2]
        )
        plt.savefig(
            "plots/" + "Case " + i[0:2] + f" timestep {iii} " + " pressureholdup.png",
            bbox_inches="tight",
        )

        fig, ax = plt.subplots(figsize=(xsize, ysize))
        # twin1 = ax.twinx()
        (p1,) = plt.plot(
            time[:iii] / 86400,
            HW_acc[:iii],
            "r:",
            label="H from integrated QWST",
            linewidth=5,
        )
        (p2,) = plt.plot(
            time[:iii] / 86400, HW_liqc[:iii], "b-", label="H from LIQC", linewidth=3
        )
        (p3,) = plt.plot(
            np.array(ppl_time) / 86400,
            h_DPZG,
            "g^",
            label="H from Gravitational Pressure Drop",
            linewidth=1,
        )
        # twin1.set_ylabel("Height of Liquid $[m]$") #, fontsize=fsize)
        ax.set_ylabel(height_axis, labelpad=10)
        ax.set_xlabel("Time $[D]$", labelpad=10)
        # ax.yaxis.label.set_color(p1.get_color())
        # twin1.yaxis.label.set_color(p2.get_color())
        # twin1.tick_params(axis="y", color=p2.get_color())
        ax.legend(handles=[p1, p2, p3])
        # plt.xlabel("Time $[D]$", labelpad=10)
        # plt.ylabel("Production rate $[Mscf/D]$", labelpad=10)
        plt.title("Height of Liquid Column Over Time, " + "Case " + i[0:2])
        plt.savefig(
            "plots/" + "Case " + i[0:2] + f" timestep {iii} " + " LiquidHeight.png",
            bbox_inches="tight",
        )

        # fig, ax = plt.subplots(figsize=(xsize, ysize))
        # # twin1 = ax.twinx()
        # (p1,) = plt.plot(
        #     time / 86400, HW_acc, "r:", label="H from integrated QWST", linewidth=5
        # )
        # (p2,) = plt.plot(time / 86400, HW_liqc, "b-", label="H from LIQC", linewidth=3)
        # (p3,) = plt.plot(np.array(ppl_time) / 86400, h_DPZG,"b^", label="H from Gravitational Pressure Drop", linewidth=3)
        # # twin1.set_ylabel("Height of Liquid $[m]$") #, fontsize=fsize)
        # ax.set_ylabel(height_axis, labelpad=10)
        # ax.set_xlabel("Time $[D]$", labelpad=10)
        # # ax.yaxis.label.set_color(p1.get_color())
        # # twin1.yaxis.label.set_color(p2.get_color())
        # # twin1.tick_params(axis="y", color=p2.get_color())
        # ax.legend(handles=[p1, p2, p3])
        # # plt.xlabel("Time $[D]$", labelpad=10)
        # # plt.ylabel("Production rate $[Mscf/D]$", labelpad=10)
        # plt.title("Height of Liquid Column Over Time, " + "Case " + i[0:2])
        # plt.savefig("plots/" + "Case " + i[0:2] + " LiquidHeight.png", bbox_inches="tight")

        fig, ax = plt.subplots(figsize=(xsize, ysize))
        plt.plot(x, ID_start, "-", label=f"Start, time = {0} days", linewidth=4)
        plt.plot(
            x,
            ID_midtrans,
            "-",
            label=f"Mid Transient, time = {int(time[int(time_idx/2)]/86400)} days",
            linewidth=3,
        )
        plt.plot(
            x,
            ID_endtrans,
            "-",
            label=f"End Transient, time = {int(time[int(time_idx)]/86400)} days",
            linewidth=2,
        )
        plt.plot(
            x, ID_end, "-", label=f"End, time = {int(time[-1]/86400)} days", linewidth=1
        )
        ax.set_ylabel("Flow Regime Identifier", labelpad=10)
        ax.set_xlabel("Length along branch, x $[m]$", labelpad=10)
        plt.title(
            "1: Stratified flow, 2: Annular flow, 3: Slug flow, 4: Bubbly flow"
        )
        ax.legend()
        plt.savefig(
            "plots/" + "Case " + i[0:2] + f" timestep {iii} " + " ID.png",
            bbox_inches="tight",
        )


    fig, ax = plt.subplots(figsize=(xsize, ysize))
    plt.plot(np.linspace(0,time[-1], len(flow_regime))/86400, flow_regime, "b-", label=f"Slug & Bubbly Flow", linewidth=3)
    plt.plot(np.linspace(0,time[-1], len(flow_regime))/86400, slug_flow, "r-", label=f"Slug Flow", linewidth=1)
    plt.plot(np.linspace(0,time[-1], len(flow_regime))/86400, bubbly_flow, "g-", label=f"Bubbly Flow", linewidth=3)
    ax.set_ylabel("Length of Flow Regime, $[m]$ ", labelpad=10)
    ax.set_xlabel("Time $[D]$", labelpad=10)
    plt.title("Length of Slug & Bubbly flow in Time")
    ax.legend()
    plt.savefig("plots/" + "Case " + i[0:2] + f"loaded flow.png", bbox_inches="tight",)

    plt.close("all")

    # plt.show()

book = load_workbook("LL_source.xlsx")  # write statdata to excel
writer = pd.ExcelWriter("LL_source.xlsx", engine="openpyxl")
writer.book = book
statdata = pd.DataFrame(lst, columns=columns2)
statdata.to_excel(writer, sheet_name="Statistics")
writer.save()


# %%
