#%%
%config Completer.use_jedi = False
import numpy as np
from libraries import gas
import pandas as pd
from scipy import interpolate
import os
import matplotlib.pyplot as plt
import mpl_axes_aligner
import ipywidgets.widgets as widgets
from ipywidgets import interact
import warnings
import pyfas as fa
from mpl_toolkits.mplot3d import Axes3D
import itertools as it
import matplotlib.cm as cm
# import fluids

"""
Fetch filenames
"""
# sim_folder = (
#     os.path.abspath("D:\Liquid Loading\\03-23-water gas\\1L - numeric depletion") + "/"
# )
# sim_folder = os.path.abspath("D:\Liquid Loading\\03-23-wet gas\\1L") + "/"
# sim_folder = os.path.abspath("D:\Liquid Loading\\03-23-wet gas\\2La") + "/"
# sim_folder = os.path.abspath("D:\Liquid Loading\\03-23-water gas\\1L") + "/"
sim_folder = os.path.abspath("G:\Liquid Loading\wet gas\\1L - long") + "/"

def filename(file_type):
    file = []
    for files in os.listdir(sim_folder):  # "..\eclipse-plot"
        if files.endswith(file_type):
            file.append(files)
        else:
            continue
    return file


"""
Aquire data Trend and Profile data from OLGA
Tune indexes and properties to specific simulations
"""


def getTPLData(sim_folder, f_name):
    tplpath = sim_folder
    fname = f_name
    tpl = fa.Tpl(tplpath + fname)
    # fielter_rend = tpl.filter_trends(trend)
    # view_trend = tpl.view_trends(trend)
    # tpl.label # find number to extract in tpl.extract()
    # keys =tpl.data.keys # find number to extract in tpl.extract()
    tpl.extract(0)
    tpl.extract(1)
    tpl.extract(2)
    tpl.extract(3)
    tpl.extract(4)
    tpl.extract(5)

    time = tpl.time
    LIQFR = tpl.data[1]
    QGST = tpl.data[2]
    QWST = tpl.data[3]
    QOST = tpl.data[4]
    PT = tpl.data[5]

    return np.array(time), LIQFR, QGST, QWST, QOST, PT


time, LIQFR, QGST, QWST, QOST, PT = getTPLData(sim_folder, filename("tpl")[0])

def getPPLData(sim_folder, f_name):
    # hard coded indexes. check ppl.filter_data("1") etc.
    # ppldata geometry data[index][0]
    # ppldata timeseries data[index][1][timestep]
    pplpath = sim_folder
    fname = f_name
    ppl = fa.Ppl(pplpath + fname)

    ppl.time
    ppl.extract(1)
    ppl.extract(2)
    ppl.extract(3)
    ppl.extract(4)
    ppl.extract(5)
    ppl.extract(6)
    ppl.extract(7)
    ppl.extract(8)
    ppl.extract(9)
    ppl.extract(10)
    ppl.extract(11)
    ppl.extract(14)
    ppl.extract(15)

    time = ppl.time
    ID = ppl.data[1]
    UD = ppl.data[2]
    UG = ppl.data[3]
    UL = ppl.data[4]
    USD = ppl.data[5]
    USG = ppl.data[6]
    USL = ppl.data[7]
    DPZA = ppl.data[8]
    DPZF = ppl.data[9]
    DPZG = ppl.data[10]
    HOL = ppl.data[11]
    PT = ppl.data[14]
    TM = ppl.data[15]

    return np.array(time), ID, UD, UG, UL, USD, USG, USL, DPZA, DPZF, DPZG, HOL, PT, TM


(
    time,
    ID,
    UD,
    UG,
    UL,
    USD,
    USG,
    USL,
    DPZA,
    DPZF,
    DPZG,
    HOL,
    PT_branch,
    TM_branch,
) = getPPLData(sim_folder, filename("ppl")[0])

"""
Aquire data from ROCX
"""


def prd(filename, nrows=1000):
    # Production data at defined boundaries in ROCX.
    # Returns dataframe
    # Time, waterprod-p5, oilprod-p5, gasprod-p5, waterprod-p4, oilprod-p4, gasprod-p4, ..., totwater, totoil, totgas, timestep
    df = pd.read_fwf(filename, infer_nrows=nrows)
    return df


def prf(filename):
    # Get pressure or saturation data, returns time array, property array
    # x, y, z, pw, po, pg (.pprf)
    # x, y, z, sw, so, sg (.sprf)

    time = []
    data = []

    with open(filename, "r") as f:
        id_list, x_list, y_list, z_list, prop1_list, prop2_list, prop3_list = (
            [],
            [],
            [],
            [],
            [],
            [],
            [],
        )
        for idx, line in enumerate(f):
            if line.strip():
                if line.startswith("##"):
                    time.append(float(line[7:-1]))
                    if idx > 1:
                        data.append(
                            [
                                id_list,
                                x_list,
                                y_list,
                                z_list,
                                prop1_list,
                                prop2_list,
                                prop3_list,
                            ]
                        )
                        (
                            id_list,
                            x_list,
                            y_list,
                            z_list,
                            prop1_list,
                            prop2_list,
                            prop3_list,
                        ) = ([], [], [], [], [], [], [])

                elif not line.startswith("#%"):
                    id, x, y, z, prop1, prop2, prop3 = map(float, line.split())
                    id_list.append(id)
                    x_list.append(x)
                    y_list.append(y)
                    z_list.append(z)
                    prop1_list.append(prop1)
                    prop2_list.append(prop2)
                    prop3_list.append(prop3)

        data.append(
            [id_list, x_list, y_list, z_list, prop1_list, prop2_list, prop3_list]
        )
    return np.array(time), np.array(data)


"""
Fluid data from TAB-file
"""

def getTABdata(simfolder, f_name):
    tabpath = simfolder
    fname = f_name
    tab = fa.Tab(tabpath + fname)
    tab.export_all()
    # p_array = tab.metadata["p_array"]
    # t_array = tab.metadata["t_array"]
    pressure = tab.data.T["PT"].values[0]  # Pa
    temperature = tab.data.T["TM"].values[0]  # C

    try:
        ROG = np.array(tab.data.T["ROG"].values[0])  # dens gas, kg/m3
        ROHL = np.array(tab.data.T["ROHL"].values[0])  # dens oil (liquid), kg/m3
        ROWT = np.array(tab.data.T["ROWT"].values[0])
        RS = np.array(tab.data.T["RS"].values[0])  # gas oil ratio, -
        RSW = np.array(tab.data.T["RSW"].values[0])
        SIGGHL = np.array(tab.data.T["SIGGHL"].values[0])  # IFT gas oil, N/m
        SIGGWT = np.array(tab.data.T["SIGGWT"].values[0])
        SIGGHLWT = np.array(tab.data.T["SIGGHLWT"].values[0])
        VISG = np.array(tab.data.T["VISG"].values[0])  # visc gas, CP
        VISHL = np.array(tab.data.T["VISHL"].values[0])  # visc oil, CP
        VISWT = np.array(tab.data.T["VISWT"].values[0])
        return (
            pressure,
            temperature,
            ROG,
            ROHL,
            ROWT,
            RS,
            RSW,
            SIGGHL,
            SIGGWT,
            SIGGHLWT,
            VISG,
            VISHL,
            VISWT,
        )

    except:
        try:
            ROG = np.array(tab.data.T["ROG"].values[0])  # dens gas, kg/m3
            ROHL = np.array(tab.data.T["ROHL"].values[0])  # dens oil (liquid), kg/m3
            RS = np.array(tab.data.T["RS"].values[0])  # gas oil ratio, -
            SIGGHL = np.array(tab.data.T["SIGGHL"].values[0])  # IFT gas oil, N/m
            VISG = np.array(tab.data.T["VISG"].values[0])  # visc gas, CP
            VISHL = np.array(tab.data.T["VISHL"].values[0])  # visc oil, CP
            return pressure, temperature, ROG, ROHL, RS, SIGGHL, VISG, VISHL

        except:
            ROG = np.array(tab.data.T["ROG"].values[0])  # dens gas, kg/m3
            ROWT = np.array(tab.data.T["ROWT"].values[0])
            RSW = np.array(tab.data.T["RSW"].values[0])
            SIGGWT = np.array(tab.data.T["SIGGWT"].values[0])
            VISG = np.array(tab.data.T["VISG"].values[0])  # visc gas, CP
            VISWT = np.array(tab.data.T["VISWT"].values[0])
            return pressure, temperature, ROG, ROWT, RSW, SIGGWT, VISG, VISWT


try:
    (
        pressure,
        temperature,
        ROG,
        ROHL,
        ROWT,
        RS,
        RSW,
        SIGGHL,
        SIGGWT,
        SIGGHLWT,
        VISG,
        VISHL,
        VISWT,
    ) = getTABdata(sim_folder, filename("tab")[0])
except:
    try:
        pressure, temperature, ROG, ROHL, RS, SIGGHL, VISG, VISHL = getTABdata(
            sim_folder, filename("tab")[0]
        )

    except:
        pressure, temperature, ROG, ROWT, RSW, SIGGWT, VISG, VISWT = getTABdata(
            sim_folder, filename("tab")[0]
        )


def tab_interp(
    p_input, T_input, property, pressure_array=pressure, temperature_array=temperature
):
    x = np.unique(pressure_array) / 1e5
    y = np.unique(temperature_array)
    z = property
    f = interpolate.interp2d(x, y, z, kind="linear")
    return f(p_input, T_input)


"""
Find index of slug and nearest index of a value
"""


def find_nearest(array, value):
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    return idx, array[idx]


def find_slug(array, value):
    array = np.asarray(array)
    idx = int((np.abs(array - value)).argmin() / len(ID[1][1]))
    return idx, array[idx]


"""
Backpressure equation
"""


def C(k, h, T, vis, Z, sD, re, rw, s, n=1):
    hf = h / 0.3048
    Tf = (T + 273.15) * 1.8
    ref = re / 0.3048
    rwf = rw / 12

    c = (0.703 * k * hf) ** n / (
        (Tf * vis * Z) ** n * (np.log(0.472 * ref / rwf + s) ** (2 * n - 1))
    )
    return c * 14.5 ** (2 * n) / 35.3146667


def backpressure(P_R, P_wf, C, n=1):
    q = C * (P_R ** 2 - P_wf ** 2) ** n
    return q


"""
Intersection of IPR and TPC using quadratic spline interpolation
"""


def nodal_point_solver(guess):

    x = np.array(QGST)
    y = np.array(PT)
    X_Y_spline = interp1d(x, y, kind="quadratic")

    P_wf = guess[0]
    Qg = guess[1]

    eq1 = Qg - c * (P_R ** 2 - P_wf ** 2) ** n
    eq2 = P_wf - X_Y_spline(Qg)

    return [eq1, eq2]


# def backpressure_solver(guess):
#     p_Rwf = [presdata[i][6][0] for i, p in enumerate(presdata)]
#     p_Ravg = [np.average(presdata[i][6]) for i, p in enumerate(presdata)]
#     p_Ri = p_Ravg[0]

#     i= 10
#     press = p_Rwf[i]
#     temp = TM_branch[1][i][0]
#     yg = tab_interp(1.01325, 20, ROG)[0]/1.205
#     Z = gas.ZfacStanding(press, temp, yg, 2)
#     visg = tab_interp(press, temp, VISG)
#     press, temp, yg, Z, visg[0]
#     C(10, 100, temp, visg, Z, 0, 500, 0.1/0.0254, 0, 1 )

"""
Turner critical velocity/rate
"""


def vc(liquid_density, gas_density, surface_tension, model="Turner"):
    # metric units, critical velocity returned in m/s
    model = str.upper(model)
    if model == "TURNER":
        k = 6.558  # turner value
    else:
        k = 5.465  # Coleman value

    rhol = liquid_density
    rhog = gas_density

    return k * (surface_tension * (rhol - rhog) / rhog ** 2) ** (0.25)


def Qc(p1, p2, T1, T2, d, unit=2, model="Turner"):

    A = np.pi / 4 * (d) ** 2  # m2
    Yg = tab_interp(1.01325, 20, ROG)[0] / 1.205
    Z1 = np.array([gas.ZfacStanding(p1[x], T1[x], Yg, 2) for x in range(len(p1))])
    Z2 = gas.ZfacStanding(p2, T2, Yg, 2)
    # Rs = tab_interp(p1, T1, RS)
    rho_g = np.array(
        [tab_interp(p1[x], T1[x], ROG)[0] for x in range(len(p1))]
    )  # kg/m3
    try:
        rho_l = np.array(
            [tab_interp(p1[x], T1[x], ROWT)[0] for x in range(len(p1))]
        )  # kg/m3
        sigma = np.array(
            [tab_interp(p1[x], T1[x], SIGGWT)[0] for x in range(len(p1))]
        )  # N/m

        v_c = vc(rho_l, rho_g, sigma, model)  # m/s
    except:
        rho_l = np.array(
            [tab_interp(p1[x], T1[x], ROHL)[0] for x in range(len(p1))]
        )  # kg/m3
        sigma = np.array(
            [tab_interp(p1[x], T1[x], SIGGHL)[0] for x in range(len(p1))]
        )  # N/m

        v_c = vc(rho_l, rho_g, sigma, model)  # m/s

    Qc = [
        (
            (p1[x] * v_c[x] * A * Z2 * (T2 + 273.15))
            / (Z1[x] * p2 * (T1[x] + 273.15))
            * 86400
        )
        for x in range(len(p1))
    ]  # Sm3/d
    return Qc, v_c

sattime, satdata = prf(sim_folder + filename("sprf")[0])


prestime, presdata = prf(sim_folder + filename("pprf")[0])


Q_sandface = prd(sim_folder + filename("prd")[0], len(sattime))
time3 = Q_sandface["Time"].to_numpy()


x_grid = np.unique(satdata[0][1]).shape[0]
y_grid = np.unique(satdata[0][2]).shape[0]
z_grid = np.unique(satdata[0][3]).shape[0]


#%% liquid loading identifier indexes
try:
    rate_gradient_search_idx = (
        # np.where(abs(np.gradient(QGST[50:])) > 0.00088)[0][0] + 50
        np.where(abs(np.gradient(QGST[50:], time[50:])) > 0.00001)[0][0]
        + 50
    )
    rate_gradient_search_rate = QGST[rate_gradient_search_idx]
    rate_gradient_search_pressure = PT[rate_gradient_search_idx]
    print(
        f"Rate gradient search index={rate_gradient_search_idx}, rate={rate_gradient_search_rate*86400}, pressure={rate_gradient_search_pressure/1e5}, time={time[rate_gradient_search_idx]/3600}"
    )
except:
    rate_gradient_search_idx = 0
    rate_gradient_search_rate = QGST[rate_gradient_search_idx] * 86400
    rate_gradient_search_pressure = PT[rate_gradient_search_idx] * 1e-5
    print("dq/dt < 0 does not exist")

slug_identifier_idx = find_slug(ID[1], 3)[0]
slug_identifier_rate = QGST[slug_identifier_idx]
slug_identifier_pressure = PT[slug_identifier_idx]
if slug_identifier_idx == 0:
    print("Slug does not exist")
else:
    print(
        f"Slug identifier index={slug_identifier_idx}, rate={slug_identifier_rate*86400}, pressure={slug_identifier_pressure/1e5}, time={time[slug_identifier_idx]/3600}"
    )

try:
    liquidfilm_reversal_idx = np.where(np.asarray(UL[1]) < 0)[0][0]
    liquidfilm_reversal_rate = QGST[liquidfilm_reversal_idx]
    liquidfilm_reversal_pressure = PT[liquidfilm_reversal_idx]
    print(
        f"Liquid film reversal index={liquidfilm_reversal_idx}, rate={liquidfilm_reversal_rate*86400}, pressure={liquidfilm_reversal_pressure/1e5}, time={time[liquidfilm_reversal_idx]/3600}"
    )
    cr1 = Qc(
        PT_branch[1][liquidfilm_reversal_idx - 10] * 1e-5,
        1.01325,
        TM_branch[1][liquidfilm_reversal_idx - 10],
        15.556,
        0.05715,
    )[0]
    cr2 = Qc(
        PT_branch[1][liquidfilm_reversal_idx + 10] * 1e-5,
        1.01325,
        TM_branch[1][liquidfilm_reversal_idx + 10],
        15.556,
        0.05715,
    )[0]
    print(
        f"Turner critical rate pre/post liquid film reversal = {np.max(cr1), np.max(cr2)}"
    )
except:
    print("UL > 0")
    liquidfilm_reversal_idx = 0

try:
    droplet_reversal_idx = np.where(np.asarray(UD[1]) < 0)[0][0]
    droplet_reversal_rate = QGST[droplet_reversal_idx]
    droplet_reversal_pressure = PT[droplet_reversal_idx]
    print(
        f"Droplet reversal index={droplet_reversal_idx}, rate={droplet_reversal_rate*86400}, pressure={droplet_reversal_pressure/1e5}, time={time[droplet_reversal_idx]/3600}"
    )
    cr1 = Qc(
        PT_branch[1][droplet_reversal_idx - 10] * 1e-5,
        1.01325,
        TM_branch[1][droplet_reversal_idx - 10],
        15.556,
        0.05715,
    )[0]
    cr2 = Qc(
        PT_branch[1][droplet_reversal_idx + 10] * 1e-5,
        1.01325,
        TM_branch[1][droplet_reversal_idx + 10],
        15.556,
        0.05715,
    )[0]
    print(
        f"Turner critical rate pre/post droplet reversal = {np.max(cr1), np.max(cr2)}"
    )

except:
    droplet_reversal_idx = 0
    print("UL > 0")

oil_backflow_index = []
for (i, Q) in enumerate(Q_sandface.columns[Q_sandface.columns.str.startswith("Oil")]):
    try:
        idx = np.where(Q_sandface[Q] < 0)[0][0]
        oil_backflow_index.append(idx)
        print(f"Layer {z_grid-i} has oil backflow @ index={idx}, time={time3[idx]/3600} hr")
    except:
        # print(f"Layer {z_grid-i} does not have oil backflow")
        pass
oil_backflow_index = min(oil_backflow_index)

for (i, Q) in enumerate(Q_sandface.columns[Q_sandface.columns.str.startswith("Wat")]):
    try:
        idx = np.where(Q_sandface[Q] < 0)[0][0]
        print(f"Layer {z_grid-i} has water backflow @ index={idx}, time={time3[idx]/3600} hr")
    except:
        # print(f"Layer {z_grid-i} does not have water backflow")
        pass

for (i, Q) in enumerate(Q_sandface.columns[Q_sandface.columns.str.startswith("Gas")]):
    try:
        idx = np.where(Q_sandface[Q] < 0)[0][0]
        print(f"Layer {z_grid-i} has gas backflow @ index={idx}, time={time3[idx]/3600} hr")
    except:
        # print(f"Layer {z_grid-i} does not have gas backflow")
        pass

try:
    post_transient_idx = []
    for (i, Q) in enumerate(Q_sandface.columns[Q_sandface.columns.str.startswith("Gas")]):
        try:
            post_transient_idx.append(np.where(
                    abs(np.gradient(Q_sandface[Q][oil_backflow_index:], time3[oil_backflow_index:]))
                    <= 0.00000000001
                )[0][0])
        except:
            pass

    post_transient_idx = (min(post_transient_idx) + oil_backflow_index)
    pre_transient_idx = (rate_gradient_search_idx)-3

    for (i, Q) in enumerate(Q_sandface.columns[Q_sandface.columns.str.startswith("Gas")]):
        try:
            print(f"Gas mass rate layer {z_grid-i} pre/post/change = {Q_sandface[Q][pre_transient_idx], Q_sandface[Q][post_transient_idx], Q_sandface[Q][post_transient_idx]/Q_sandface[Q][pre_transient_idx]}")
        except:
            # print(f"error layer {z_grid-i}")
            pass
    for (i, Q) in enumerate(Q_sandface.columns[Q_sandface.columns.str.startswith("Oil")]):
        try:
            if Q_sandface[Q][post_transient_idx] != 0:
                print(f"Oil mass rate layer {z_grid-i} pre/post = {Q_sandface[Q][pre_transient_idx], Q_sandface[Q][post_transient_idx]}")
        except:
            # print(f"error layer {z_grid-i}")
            pass
    for (i, Q) in enumerate(Q_sandface.columns[Q_sandface.columns.str.startswith("Wat")]):
        try:
            if Q_sandface[Q][post_transient_idx] != 0:
                print(f"Water mass rate layer {z_grid-i} pre/post = {Q_sandface[Q][pre_transient_idx], Q_sandface[Q][post_transient_idx]}")
        except:
            # print(f"error layer {z_grid-i}")
            pass
except:
    print("error")

try:
    i = post_transient_idx
    layer1 = np.where(satdata[i][5][0:x_grid]==0)[0][0]
    layer2 = np.where(satdata[i][5][x_grid:2*x_grid]==0)[0][0]
    layer3 = np.where(satdata[i][5][2*x_grid:3*x_grid]==0)[0][0]
    layer4 = np.where(satdata[i][5][3*x_grid:4*x_grid]==0)[0][0]
    layer5 = np.where(satdata[i][5][4*x_grid:5*x_grid]==0)[0][0]

    layer1, layer2, layer3, layer4, layer5

    # make minimum radial penetration function (min (l1, ..., l5)). insert instead of layer5 below

    avg_oilsat1 = np.average(satdata[i][5][0:layer5], weights=np.diff(satdata[i][1][0:layer5+1]))
    avg_oilsat2 = np.average(satdata[i][5][x_grid:x_grid+layer5], weights=np.diff(satdata[i][1][0:layer5+1]))
    avg_oilsat3 = np.average(satdata[i][5][2*x_grid:2*x_grid+layer5], weights=np.diff(satdata[i][1][0:layer5+1]))
    avg_oilsat4 = np.average(satdata[i][5][3*x_grid:3*x_grid+layer5], weights=np.diff(satdata[i][1][0:layer5+1]))
    avg_oilsat5 = np.average(satdata[i][5][4*x_grid:4*x_grid+layer5], weights=np.diff(satdata[i][1][0:layer5+1]))
    oildepth1 = satdata[i][1][layer1]
    oildepth2 = satdata[i][1][layer2]
    oildepth3 = satdata[i][1][layer3]
    oildepth4 = satdata[i][1][layer4]
    oildepth5 = satdata[i][1][layer5]

    print(avg_oilsat1, avg_oilsat2, avg_oilsat3, avg_oilsat4, avg_oilsat5, oildepth1, oildepth2, oildepth3, oildepth4, oildepth5)
except:
    print("error")

#%%


# plotting
fsize = 14
tsize = 16
tdir = "out"  # "in" "out" "inout"
major = 5
minor = 3
style = "default"  # plt.style.available
xsize = 8
ysize = 5

plt.style.use(style)
plt.rcParams["text.usetex"] = True
plt.rcParams["font.size"] = fsize
plt.rcParams["legend.fontsize"] = 9
plt.rcParams["legend.title_fontsize"] = tsize
plt.rcParams["legend.loc"] = "best"
plt.rcParams["xtick.direction"] = tdir
plt.rcParams["ytick.direction"] = tdir
plt.rcParams["xtick.major.size"] = major
plt.rcParams["xtick.minor.size"] = minor
plt.rcParams["ytick.major.size"] = major
plt.rcParams["ytick.minor.size"] = minor

gaspressurelabels=["$p_{g, L 1}$",
    "$p_{g, L 2}$",
    "$p_{g, L 3}$",
    "$p_{g, L 4}$",
    "$p_{g, L 5}$",
    "$p_{g, L 6}$",
    "$p_{g, L 7}$",
    "$p_{g, L 8}$",
    "$p_{g, L 9}$",
    "$p_{g, L 10}$",
    "$p_{g, L 11}$",]

oilpressurelabels=["$p_{o, L 1}$",
    "$p_{o, L 2}$",
    "$p_{o, L 3}$",
    "$p_{o, L 4}$",
    "$p_{o, L 5}$",
    "$p_{o, L 6}$",
    "$p_{o, L 7}$",
    "$p_{o, L 8}$",
    "$p_{o, L 9}$",
    "$p_{o, L 10}$",
    "$p_{o, L 11}$",]

waterpressurelabels=["$p_{w, L 1}$",
    "$p_{w, L 2}$",
    "$p_{w, L 3}$",
    "$p_{w, L 4}$",
    "$p_{w, L 5}$",
    "$p_{w, L 6}$",
    "$p_{w, L 7}$",
    "$p_{w, L 8}$",
    "$p_{w, L 9}$",
    "$p_{w, L 10}$",
    "$p_{w, L 11}$",]

gasmasslabels = ["$\dot{m}_{g, L 1}$",
"$\dot{m}_{g, L 2}$",
"$\dot{m}_{g, L 3}$",
"$\dot{m}_{g, L 4}$",
"$\dot{m}_{g, L 5}$",
"$\dot{m}_{g, L 6}$",
"$\dot{m}_{g, L 7}$",
"$\dot{m}_{g, L 8}$",
"$\dot{m}_{g, L 9}$",
"$\dot{m}_{g, L 10}$",
"$\dot{m}_{g, L 11}$"]

oilmasslabels = ["$\dot{m}_{o, L 1}$",
"$\dot{m}_{o, L 2}$",
"$\dot{m}_{o, L 3}$",
"$\dot{m}_{o, L 4}$",
"$\dot{m}_{o, L 5}$",
"$\dot{m}_{o, L 6}$",
"$\dot{m}_{o, L 7}$",
"$\dot{m}_{o, L 8}$",
"$\dot{m}_{o, L 9}$",
"$\dot{m}_{o, L 10}$",
"$\dot{m}_{o, L 11}$"]

watermasslabels = ["$\dot{m}_{w, L 1}$",
"$\dot{m}_{w, L 2}$",
"$\dot{m}_{w, L 3}$",
"$\dot{m}_{w, L 4}$",
"$\dot{m}_{w, L 5}$",
"$\dot{m}_{w, L 6}$",
"$\dot{m}_{w, L 7}$",
"$\dot{m}_{w, L 8}$",
"$\dot{m}_{w, L 9}$",
"$\dot{m}_{w, L 10}$",
"$\dot{m}_{w, L 11}$"]


@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time3) / 3600),
             save=widgets.ToggleButton()):

    fig, ax = plt.subplots(figsize=(xsize, ysize))
    for Q, lbl in zip(Q_sandface.columns[Q_sandface.columns.str.startswith("Gas")][::-1], gasmasslabels[0:z_grid]):
        if np.sum(Q_sandface[Q]) != 0:
            ax.plot(time3/3600, Q_sandface[Q], label=lbl)
    ax.set_ylabel("Gas mass rate, $\dot{m}_g$ $[kg/s]$", labelpad=10)
    ax.set_xlabel("Time, $t$ $[hr]$", labelpad=10)
    ax.set_xlim(xlim)
    ax.legend(bbox_to_anchor=(1.15,1),loc="upper right", ncol=1)
    ax.set_title("Mass Rate of Gas Through Sandface")
    if save==True:
        fig.savefig("plots/gasmassrate.png", dpi=1000, bbox_inches="tight")

@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time3) / 3600),
             save=widgets.ToggleButton()
             ):

    fig, ax = plt.subplots(figsize=(xsize, ysize))
    for Q, lbl in zip(Q_sandface.columns[Q_sandface.columns.str.startswith("Oil")][::-1], oilmasslabels[0:z_grid]):
        if np.sum(Q_sandface[Q]) != 0:
            ax.plot(time3/3600, Q_sandface[Q], label=lbl)
    ax.set_ylabel("Oil mass rate, $\dot{m}_g$ $[kg/s]$", labelpad=10)
    ax.set_xlabel("Time, $t$ $[hr]$", labelpad=10)
    ax.set_xlim(xlim)
    ax.legend(bbox_to_anchor=(1.15,1),loc="upper right", ncol=1)
    ax.set_title("Mass Rate of Oil Through Sandface")
    if save==True:
        fig.savefig("plots/oilmassrate.png", dpi=1000, bbox_inches="tight")

@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time3) / 3600),
             save=widgets.ToggleButton()
             ):

    fig, ax = plt.subplots(figsize=(xsize, ysize))
    for Q, lbl in zip(Q_sandface.columns[Q_sandface.columns.str.startswith("Water")][::-1], watermasslabels[0:z_grid]):
        if np.sum(Q_sandface[Q]) != 0:
            ax.plot(time3/3600, Q_sandface[Q], label=lbl)
    ax.set_ylabel("Water mass rate, $\dot{m}_g$ $[kg/s]$", labelpad=10)
    ax.set_xlabel("Time, $t$ $[hr]$", labelpad=10)
    ax.set_xlim(xlim)
    ax.legend(bbox_to_anchor=(1.15,1),loc="upper right", ncol=1)
    ax.set_title("Mass Rate of Water Through Sandface")
    if save==True:
        fig.savefig("plots/watermassrate.png", dpi=1000, bbox_inches="tight")


@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time3) / 3600)):
    fig, ax = plt.subplots(figsize=(xsize, ysize))
    ax.plot(time3 / 3600, Q_sandface["TotOilprod"], "g", label="$\dot{m}_{o, tot}$")
    ax.plot(time3 / 3600, Q_sandface["TotGasprod"], "r", label="$\dot{m}_{g, tot}$")
    if np.sum(Q_sandface["TotWatprod"]) != 0:
        ax.plot(time3 / 3600, Q_sandface["TotWatprod"], "b", label="$\dot{m}_{w, tot}$")
    ax.set_ylabel("Mass rate, $\dot{m}$ $[kg/s]$", labelpad=10)
    ax.set_xlabel("Time, $t$ $[hr]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax.set_xlim(xlim)
    ax.legend(bbox_to_anchor=(1.15,1),loc="upper right", ncol=1)
    ax.set_title("Total Mass Rates Through Sandface")


# %%
# OLGA profile plot
timesteps = len(time)

#%%
# OLGA Trend data plot
@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time) / 3600),
    save=widgets.ToggleButton()
    ):

    variables = [[time/3600,QGST*86400], [time/3600,QOST*86400]]
    positions = ['left', 'right']
    colors = ['red', 'green']
    ylabels = ['$q_{g}$', '$q_{o}$']
    xlabel = 'Time, $t$ $[hr]$'
    yaxislabels = ["Gas flow rate, $q_{g}$ $[Sm^3/d]$", 'Oil flow rate, $q_{o}$ $[Sm^3/d]$']

    y_axis_dist = 0.2

    def make_patch_spines_invisible(ax):
        ax.set_frame_on(True)
        ax.patch.set_visible(False)
        for sp in ax.spines.values():
            sp.set_visible(False)


    fig, host = plt.subplots(figsize=(xsize,ysize))
    fig.subplots_adjust(right=0.75)


    ###### HOST PLOTTING
    tkw = dict(size=4, width=1.5, labelsize=fsize)


    p1, = host.plot(variables[0][0], variables[0][1], colors[0], label=ylabels[0])
    host.set_xlabel(xlabel, fontsize=fsize)
    host.set_ylabel(yaxislabels[0], fontsize=fsize)
    host.yaxis.label.set_color(p1.get_color())
    host.tick_params(axis='y', colors=p1.get_color(), **tkw)
    host.tick_params(axis='x', **tkw)
    # host.set_xlim(0, 2)

    lines = [p1]

#     y_axis_dist = 0.2
    inc_r = 1
    inc_l = -y_axis_dist

    for ix, i in enumerate(variables):


        if ix != 0:
            par = host.twinx()
            if positions[ix] == 'right':
                par.spines[positions[ix]].set_position(("axes", inc_r))
                inc_r += y_axis_dist
            elif positions[ix] == 'left':
                par.spines[positions[ix]].set_position(("axes", inc_l))
                inc_l -= y_axis_dist

            make_patch_spines_invisible(par)
            par.spines[positions[ix]].set_visible(True)
            par.yaxis.set_label_position(positions[ix])
            par.yaxis.set_ticks_position(positions[ix])
            p, = par.plot(variables[ix][0], variables[ix][1], colors[ix], label=ylabels[ix])
            par.set_ylabel(yaxislabels[ix], fontsize=fsize)
            par.yaxis.label.set_color(p.get_color())
            par.tick_params(axis='y', colors=p.get_color(), **tkw)
            lines.append(p)

    host.legend(lines, [l.get_label() for l in lines], fontsize=12, loc='best')
    par.set_xlim(xlim)
    par.set_title("Surface Rates of Gas and Oil")
    if save==True:
        fig.savefig("plots/surfrates.png", dpi=1000, bbox_inches="tight")


flowregime_identifier = []
for i in range(len(ID[1])):
    slugflow_idx = np.where(ID[1][i]>=3)[0]
    slugflow_length=[]
    if len(slugflow_idx) != 0:
        for j in slugflow_idx:
            if j==len(ID[0]):
                slugflow_length.append((ID[0][j] - ID[0][j-1]))
            else:
                slugflow_length.append((ID[0][j+1] - ID[0][j]))
    else:
        slugflow_length.append(0)
    flowregime_identifier.append(sum(slugflow_length))


@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time) / 3600),
    save=widgets.ToggleButton()
    ):

    variables = [[time/3600,LIQFR], [time/3600,flowregime_identifier], [time/3600,PT*1e-5]]
    positions = ['right', 'left', 'right']
    colors = ['green', 'blue', 'red']
    ylabels = ['$\phi_L$', '$L_{slug \ flow}$', '$p_{wf}$']
    xlabel = 'Time, $t$ $[hr]$'
    yaxislabels = ["Liquid fraction, $\phi_L$  $[-]$ ", 'Length of slug flow regime, $L_{slug \ flow}$ $[m]$ ', 'Bottomhole pressure, $p_{wf}$ $[bara]$']

    y_axis_dist = 0.2

    def make_patch_spines_invisible(ax):
        ax.set_frame_on(True)
        ax.patch.set_visible(False)
        for sp in ax.spines.values():
            sp.set_visible(False)


    fig, host = plt.subplots(figsize=(xsize,ysize))
    fig.subplots_adjust(right=0.75)


    ###### HOST PLOTTING
    tkw = dict(size=4, width=1.5, labelsize=fsize)


    p1, = host.plot(variables[0][0], variables[0][1], colors[0], label=ylabels[0])
    host.set_xlabel(xlabel, fontsize=fsize)
    host.set_ylabel(yaxislabels[0], fontsize=fsize)
    host.yaxis.label.set_color(p1.get_color())
    host.tick_params(axis='y', colors=p1.get_color(), **tkw)
    host.tick_params(axis='x', **tkw)
    # host.set_xlim(0, 2)

    lines = [p1]

#     y_axis_dist = 0.2
    inc_r = 1
    inc_l = -y_axis_dist

    for ix, i in enumerate(variables):


        if ix != 0:
            par = host.twinx()
            if positions[ix] == 'right':
                par.spines[positions[ix]].set_position(("axes", inc_r))
                inc_r += y_axis_dist
            elif positions[ix] == 'left':
                par.spines[positions[ix]].set_position(("axes", inc_l))
                inc_l -= y_axis_dist

            make_patch_spines_invisible(par)
            par.spines[positions[ix]].set_visible(True)
            par.yaxis.set_label_position(positions[ix])
            par.yaxis.set_ticks_position(positions[ix])
            p, = par.plot(variables[ix][0], variables[ix][1], colors[ix], label=ylabels[ix])
            par.set_ylabel(yaxislabels[ix], fontsize=fsize)
            par.yaxis.label.set_color(p.get_color())
            par.tick_params(axis='y', colors=p.get_color(), **tkw)
            lines.append(p)

    host.legend(lines, [l.get_label() for l in lines], fontsize=12, loc='lower right')
    par.set_xlim(xlim)
    par.set_title("BHP, Flow Regime, and Liquid Fraction")
    if save==True:
        fig.savefig("plots/slugbhpfrac.png", dpi=1000, bbox_inches="tight")


@interact
def ppl_plot(
    ts=widgets.IntSlider(min=0, max=len(time) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(ID[0])),
    ylim=widgets.FloatRangeSlider(
        min=np.min([UG[1], UD[1], UL[1]]), max=np.max([UG[1], UD[1], UL[1]])
    ),
    save=widgets.ToggleButton(),
):
    critical_rate = Qc(
        PT_branch[1][ts] * 1e-5, 1.01325, TM_branch[1][ts], 15.556, 0.05715
    )[1]
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    # ax0.set_ylim(1, 4)
    # ax0.invert_yaxis()
    # ax0.set_yticklabels(["Stratified", "Annular", "Slug", "Bubbly"])
    (p0,) = ax0.plot(PT_branch[0], critical_rate, "k", label="$u_{c, Turner}$")
    (p1,) = ax0.plot(UG[0], UG[1][ts], "r-", label="$u_{g}$")
    (p2,) = ax0.plot(UD[0], UD[1][ts], "g-", label="$u_{d}$")
    (p3,) = ax0.plot(UL[0], UL[1][ts], "b-", label="$u_{l}$")
    ax0.set_title(
        f"Phase Velocity and Turner Critical Velocity @ t = {round(time[ts]/3600,1)} hr"
    )
    ax0.set_ylabel("Velocity, $u$ $[m/s]$", fontsize=fsize)
    ax0.set_xlabel("Distance along tubing , $z$ $[m]$", fontsize=fsize)
    ax0.set_ylim(ylim)
    ax0.set_xlim(xlim)
    fig.legend(borderaxespad=3, fontsize=12)
    if save==True:
        fig.savefig("plots/phasevelocity.png", dpi=800)


@interact
def sprf_plot(
    ts=widgets.IntSlider(min=0, max=len(sattime) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(satdata[0][1])),
    save=widgets.ToggleButton()
):
    fig, ax = plt.subplots(nrows=2, ncols=1)
    ax[1].pcolormesh(
        satdata[ts][1].reshape(z_grid, -1)[0],
        satdata[ts][3].reshape(z_grid, -1)[:, 0],
        satdata[ts][6].reshape(z_grid, -1),
        cmap="jet",
        shading="auto",
    )
    # ax.grid()
    ax[1].set_ylabel("Vertical distance, $z$ $[m]$", labelpad=10)
    ax[1].set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax[1].set_xlim(xlim)
    # ax[1].yaxis.set_label_position("right")
    # ax[1].set_title(
    #     "Gas Saturation Profile @ time = {} $hr$".format(round(sattime[ts] / 3600, 1))
    # )
    fig.colorbar(cm.ScalarMappable(norm=None, cmap="jet"), ax=ax, location="right",anchor=(0.1,-0.01), shrink=0.45)
    for data, lbl in zip(presdata[ts][6].reshape(z_grid, -1), gaspressurelabels):
    # data, lbl = zip(presdata[ts][6].reshape(z_grid, -1), labels)
        if data[0]/data[-1] != 1:
            ax[0].plot(
                presdata[0][1].reshape(z_grid, -1)[0],
                data,
                label=lbl,)
    ax[0].set_ylabel("Gas pressure, $p_g$ $[bara]$", labelpad=10)
    ax[0].set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    ax[0].set_xlim(xlim)
    # ax.set_ylim(ylim)
    ax[0].legend(bbox_to_anchor=(1.23,1.1),loc="upper right", ncol=1)
    ax[0].set_title(
        f"Gas Pressure and Saturation @ t = {round(sattime[ts]/3600,1)} hrs"
    )
    if save == True:
        fig.savefig("plots/gas_saturation.png", dpi=1000, bbox_inches="tight")

    # fig.tight_layout()


#%%
@interact
def sprf_plot(
    ts=widgets.IntSlider(min=0, max=len(sattime) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(satdata[0][1])),
    save=widgets.ToggleButton()
):
    fig, ax = plt.subplots(nrows=2, ncols=1)
    ax[1].pcolormesh(
        satdata[ts][1].reshape(z_grid, -1)[0],
        satdata[ts][3].reshape(z_grid, -1)[:, 0],
        satdata[ts][5].reshape(z_grid, -1),
        cmap="jet_r",
        shading="auto",
    )
    # ax.grid()
    ax[1].set_ylabel("Vertical distance, $z$ $[m]$", labelpad=10)
    ax[1].set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax[1].set_xlim(xlim)
    # ax[1].yaxis.set_label_position("right")
    # ax[1].set_title(
    #     "Gas Saturation Profile @ time = {} $hr$".format(round(sattime[ts] / 3600, 1))
    # )
    fig.colorbar(cm.ScalarMappable(norm=None, cmap="jet_r"), ax=ax, location="right",anchor=(0.1,-0.01), shrink=0.45)
    # fig.tight_layout(pad=0.2)

    for data, lbl in zip(presdata[ts][5].reshape(z_grid, -1), oilpressurelabels):
    # data, lbl = zip(presdata[ts][6].reshape(z_grid, -1), labels)
        if data[0]/data[-1] != 1:
            ax[0].plot(
                presdata[0][1].reshape(z_grid, -1)[0],
                data,
                label=lbl,)
    ax[0].set_ylabel("Oil pressure, $p_g$ $[bara]$", labelpad=10)
    ax[0].set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    ax[0].set_xlim(xlim)
    # ax.set_ylim(ylim)
    ax[0].legend(bbox_to_anchor=(1.23,1.1),loc="upper right", ncol=1)
    ax[0].set_title(
        f"Oil Pressure and Saturation @ t = {round(sattime[ts]/3600,1)} hrs"
    )
    if save==True:
        fig.savefig("plots/oil_saturation.png", dpi=1000, bbox_inches="tight")

    # ax[0].set_xscale("log")
    # fig.tight_layout()


# %%
@interact
def sprf_plot(
    ts=widgets.IntSlider(min=0, max=len(sattime) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(satdata[0][1])),
    save=widgets.ToggleButton()
):
    fig, ax = plt.subplots(nrows=2, ncols=1)
    ax[1].pcolormesh(
        satdata[ts][1].reshape(z_grid, -1)[0],
        satdata[ts][3].reshape(z_grid, -1)[:, 0],
        satdata[ts][4].reshape(z_grid, -1),
        cmap="jet_r",
        shading="auto",
    )
    # ax.grid()
    ax[1].set_ylabel("Vertical distance, $z$ $[m]$", labelpad=10)
    ax[1].set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax[1].set_xlim(xlim)
    # ax[1].yaxis.set_label_position("right")
    # ax[1].set_title(
    #     "Gas Saturation Profile @ time = {} $hr$".format(round(sattime[ts] / 3600, 1))
    # )
    fig.colorbar(cm.ScalarMappable(norm=None, cmap="jet_r"), ax=ax, location="right", anchor=(0.1,-0.01), shrink=0.45)
    # fig.tight_layout(pad=0.2)

    for data, lbl in zip(presdata[ts][4].reshape(z_grid, -1), waterpressurelabels):
    # data, lbl = zip(presdata[ts][6].reshape(z_grid, -1), labels)
        if data[0]/data[-1] != 1:
            ax[0].plot(
                presdata[0][1].reshape(z_grid, -1)[0],
                data,
                label=lbl,)
    ax[0].set_ylabel("Water pressure, $p_g$ $[bara]$", labelpad=10)
    ax[0].set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    ax[0].set_xlim(xlim)
    # ax.set_ylim(ylim)
    ax[0].legend(bbox_to_anchor=(1.23,1.1),loc="upper right", ncol=1)
    ax[0].set_title(
        f"Water Pressure and Saturation @ t = {round(sattime[ts]/3600,1)} hrs"
    )
    if save==True:
        fig.savefig("plots/water_saturation.png", dpi=1000, bbox_inches="tight")
    # ax[0].set_xscale("log")
    # fig.tight_layout()



# %%
