#%%¨
%config Completer.use_jedi = False
import numpy as np
from libraries import gas
import pandas as pd
from scipy import interpolate
import os
import matplotlib.pyplot as plt
import mpl_axes_aligner
import ipywidgets.widgets as widgets
from ipywidgets import interact
import warnings
import pyfas as fa
from mpl_toolkits.mplot3d import Axes3D
import itertools as it
import matplotlib.cm as cm
# import fluids

"""
Fetch filenames
"""
# sim_folder = (
#     os.path.abspath("D:\Liquid Loading\\03-23-water gas\\1L - numeric depletion") + "/"
# )
# sim_folder = os.path.abspath("D:\Liquid Loading\\03-23-wet gas\\1L") + "/"
# sim_folder = os.path.abspath("D:\Liquid Loading\\03-23-wet gas\\2La") + "/"
# sim_folder = os.path.abspath("D:\Liquid Loading\\03-23-water gas\\1L") + "/"
sim_folder = os.path.abspath("D:\Liquid Loading\wet gas\\1L - long") + "/"


x_grid = 30
y_grid = 1
z_grid = 10

def filename(file_type):
    file = []
    for files in os.listdir(sim_folder):  # "..\eclipse-plot"
        if files.endswith(file_type):
            file.append(files)
        else:
            continue
    return file


"""
Aquire data Trend and Profile data from OLGA
Tune indexes and properties to specific simulations
"""


def getTPLData(sim_folder, f_name):
    tplpath = sim_folder
    fname = f_name
    tpl = fa.Tpl(tplpath + fname)
    # fielter_rend = tpl.filter_trends(trend)
    # view_trend = tpl.view_trends(trend)
    # tpl.label # find number to extract in tpl.extract()
    # keys =tpl.data.keys # find number to extract in tpl.extract()
    tpl.extract(0)
    tpl.extract(1)
    tpl.extract(2)
    tpl.extract(3)
    tpl.extract(4)
    tpl.extract(5)

    time = tpl.time
    LIQFR = tpl.data[1]
    QGST = tpl.data[2]
    QWST = tpl.data[3]
    QOST = tpl.data[4]
    PT = tpl.data[5]

    return np.array(time), LIQFR, QGST, QWST, QOST, PT


time, LIQFR, QGST, QWST, QOST, PT = getTPLData(sim_folder, filename("tpl")[0])

def getPPLData(sim_folder, f_name):
    # hard coded indexes. check ppl.filter_data("1") etc.
    # ppldata geometry data[index][0]
    # ppldata timeseries data[index][1][timestep]
    pplpath = sim_folder
    fname = f_name
    ppl = fa.Ppl(pplpath + fname)

    ppl.time
    ppl.extract(1)
    ppl.extract(2)
    ppl.extract(3)
    ppl.extract(4)
    ppl.extract(5)
    ppl.extract(6)
    ppl.extract(7)
    ppl.extract(8)
    ppl.extract(9)
    ppl.extract(10)
    ppl.extract(11)
    ppl.extract(14)
    ppl.extract(15)

    time = ppl.time
    ID = ppl.data[1]
    UD = ppl.data[2]
    UG = ppl.data[3]
    UL = ppl.data[4]
    USD = ppl.data[5]
    USG = ppl.data[6]
    USL = ppl.data[7]
    DPZA = ppl.data[8]
    DPZF = ppl.data[9]
    DPZG = ppl.data[10]
    HOL = ppl.data[11]
    PT = ppl.data[14]
    TM = ppl.data[15]

    return np.array(time), ID, UD, UG, UL, USD, USG, USL, DPZA, DPZF, DPZG, HOL, PT, TM


(
    time,
    ID,
    UD,
    UG,
    UL,
    USD,
    USG,
    USL,
    DPZA,
    DPZF,
    DPZG,
    HOL,
    PT_branch,
    TM_branch,
) = getPPLData(sim_folder, filename("ppl")[0])

"""
Aquire data from ROCX
"""


def prd(filename, nrows=1000):
    # Production data at defined boundaries in ROCX.
    # Returns dataframe
    # Time, waterprod-p5, oilprod-p5, gasprod-p5, waterprod-p4, oilprod-p4, gasprod-p4, ..., totwater, totoil, totgas, timestep
    df = pd.read_fwf(filename, infer_nrows=nrows)
    return df


def prf(filename):
    # Get pressure or saturation data, returns time array, property array
    # x, y, z, pw, po, pg (.pprf)
    # x, y, z, sw, so, sg (.sprf)

    time = []
    data = []

    with open(filename, "r") as f:
        id_list, x_list, y_list, z_list, prop1_list, prop2_list, prop3_list = (
            [],
            [],
            [],
            [],
            [],
            [],
            [],
        )
        for idx, line in enumerate(f):
            if line.strip():
                if line.startswith("##"):
                    time.append(float(line[7:-1]))
                    if idx > 1:
                        data.append(
                            [
                                id_list,
                                x_list,
                                y_list,
                                z_list,
                                prop1_list,
                                prop2_list,
                                prop3_list,
                            ]
                        )
                        (
                            id_list,
                            x_list,
                            y_list,
                            z_list,
                            prop1_list,
                            prop2_list,
                            prop3_list,
                        ) = ([], [], [], [], [], [], [])

                elif not line.startswith("#%"):
                    id, x, y, z, prop1, prop2, prop3 = map(float, line.split())
                    id_list.append(id)
                    x_list.append(x)
                    y_list.append(y)
                    z_list.append(z)
                    prop1_list.append(prop1)
                    prop2_list.append(prop2)
                    prop3_list.append(prop3)

        data.append(
            [id_list, x_list, y_list, z_list, prop1_list, prop2_list, prop3_list]
        )
    return np.array(time), np.array(data)


"""
Fluid data from TAB-file
"""

def getTABdata(simfolder, f_name):
    tabpath = simfolder
    fname = f_name
    tab = fa.Tab(tabpath + fname)
    tab.export_all()
    # p_array = tab.metadata["p_array"]
    # t_array = tab.metadata["t_array"]
    pressure = tab.data.T["PT"].values[0]  # Pa
    temperature = tab.data.T["TM"].values[0]  # C

    try:
        ROG = np.array(tab.data.T["ROG"].values[0])  # dens gas, kg/m3
        ROHL = np.array(tab.data.T["ROHL"].values[0])  # dens oil (liquid), kg/m3
        ROWT = np.array(tab.data.T["ROWT"].values[0])
        RS = np.array(tab.data.T["RS"].values[0])  # gas oil ratio, -
        RSW = np.array(tab.data.T["RSW"].values[0])
        SIGGHL = np.array(tab.data.T["SIGGHL"].values[0])  # IFT gas oil, N/m
        SIGGWT = np.array(tab.data.T["SIGGWT"].values[0])
        SIGGHLWT = np.array(tab.data.T["SIGGHLWT"].values[0])
        VISG = np.array(tab.data.T["VISG"].values[0])  # visc gas, CP
        VISHL = np.array(tab.data.T["VISHL"].values[0])  # visc oil, CP
        VISWT = np.array(tab.data.T["VISWT"].values[0])
        return (
            pressure,
            temperature,
            ROG,
            ROHL,
            ROWT,
            RS,
            RSW,
            SIGGHL,
            SIGGWT,
            SIGGHLWT,
            VISG,
            VISHL,
            VISWT,
        )

    except:
        try:
            ROG = np.array(tab.data.T["ROG"].values[0])  # dens gas, kg/m3
            ROHL = np.array(tab.data.T["ROHL"].values[0])  # dens oil (liquid), kg/m3
            RS = np.array(tab.data.T["RS"].values[0])  # gas oil ratio, -
            SIGGHL = np.array(tab.data.T["SIGGHL"].values[0])  # IFT gas oil, N/m
            VISG = np.array(tab.data.T["VISG"].values[0])  # visc gas, CP
            VISHL = np.array(tab.data.T["VISHL"].values[0])  # visc oil, CP
            return pressure, temperature, ROG, ROHL, RS, SIGGHL, VISG, VISHL

        except:
            ROG = np.array(tab.data.T["ROG"].values[0])  # dens gas, kg/m3
            ROWT = np.array(tab.data.T["ROWT"].values[0])
            RSW = np.array(tab.data.T["RSW"].values[0])
            SIGGWT = np.array(tab.data.T["SIGGWT"].values[0])
            VISG = np.array(tab.data.T["VISG"].values[0])  # visc gas, CP
            VISWT = np.array(tab.data.T["VISWT"].values[0])
            return pressure, temperature, ROG, ROWT, RSW, SIGGWT, VISG, VISWT


try:
    (
        pressure,
        temperature,
        ROG,
        ROHL,
        ROWT,
        RS,
        RSW,
        SIGGHL,
        SIGGWT,
        SIGGHLWT,
        VISG,
        VISHL,
        VISWT,
    ) = getTABdata(sim_folder, filename("tab")[0])
except:
    try:
        pressure, temperature, ROG, ROHL, RS, SIGGHL, VISG, VISHL = getTABdata(
            sim_folder, filename("tab")[0]
        )

    except:
        pressure, temperature, ROG, ROWT, RSW, SIGGWT, VISG, VISWT = getTABdata(
            sim_folder, filename("tab")[0]
        )


def tab_interp(
    p_input, T_input, property, pressure_array=pressure, temperature_array=temperature
):
    x = np.unique(pressure_array) / 1e5
    y = np.unique(temperature_array)
    z = property
    f = interpolate.interp2d(x, y, z, kind="linear")
    return f(p_input, T_input)


"""
Find index of slug and nearest index of a value
"""


def find_nearest(array, value):
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    return idx, array[idx]


def find_slug(array, value):
    array = np.asarray(array)
    idx = int((np.abs(array - value)).argmin() / len(ID[1][1]))
    return idx, array[idx]


"""
Backpressure equation
"""


def C(k, h, T, vis, Z, sD, re, rw, s, n=1):
    hf = h / 0.3048
    Tf = (T + 273.15) * 1.8
    ref = re / 0.3048
    rwf = rw / 12

    c = (0.703 * k * hf) ** n / (
        (Tf * vis * Z) ** n * (np.log(0.472 * ref / rwf + s) ** (2 * n - 1))
    )
    return c * 14.5 ** (2 * n) / 35.3146667


def backpressure(P_R, P_wf, C, n=1):
    q = C * (P_R ** 2 - P_wf ** 2) ** n
    return q


"""
Intersection of IPR and TPC using quadratic spline interpolation
"""


def nodal_point_solver(guess):

    x = np.array(QGST)
    y = np.array(PT)
    X_Y_spline = interp1d(x, y, kind="quadratic")

    P_wf = guess[0]
    Qg = guess[1]

    eq1 = Qg - c * (P_R ** 2 - P_wf ** 2) ** n
    eq2 = P_wf - X_Y_spline(Qg)

    return [eq1, eq2]


# def backpressure_solver(guess):
#     p_Rwf = [presdata[i][6][0] for i, p in enumerate(presdata)]
#     p_Ravg = [np.average(presdata[i][6]) for i, p in enumerate(presdata)]
#     p_Ri = p_Ravg[0]

#     i= 10
#     press = p_Rwf[i]
#     temp = TM_branch[1][i][0]
#     yg = tab_interp(1.01325, 20, ROG)[0]/1.205
#     Z = gas.ZfacStanding(press, temp, yg, 2)
#     visg = tab_interp(press, temp, VISG)
#     press, temp, yg, Z, visg[0]
#     C(10, 100, temp, visg, Z, 0, 500, 0.1/0.0254, 0, 1 )

"""
Turner critical velocity/rate
"""


def vc(liquid_density, gas_density, surface_tension, model="Turner"):
    # metric units, critical velocity returned in m/s
    model = str.upper(model)
    if model == "TURNER":
        k = 6.558  # turner value
    else:
        k = 5.465  # Coleman value

    rhol = liquid_density
    rhog = gas_density

    return k * (surface_tension * (rhol - rhog) / rhog ** 2) ** (0.25)


def Qc(p1, p2, T1, T2, d, unit=2, model="Turner"):

    A = np.pi / 4 * (d) ** 2  # m2
    Yg = tab_interp(1.01325, 20, ROG)[0] / 1.205
    Z1 = np.array([gas.ZfacStanding(p1[x], T1[x], Yg, 2) for x in range(len(p1))])
    Z2 = gas.ZfacStanding(p2, T2, Yg, 2)
    # Rs = tab_interp(p1, T1, RS)
    rho_g = np.array(
        [tab_interp(p1[x], T1[x], ROG)[0] for x in range(len(p1))]
    )  # kg/m3
    try:
        rho_l = np.array(
            [tab_interp(p1[x], T1[x], ROWT)[0] for x in range(len(p1))]
        )  # kg/m3
        sigma = np.array(
            [tab_interp(p1[x], T1[x], SIGGWT)[0] for x in range(len(p1))]
        )  # N/m

        v_c = vc(rho_l, rho_g, sigma, model)  # m/s
    except:
        rho_l = np.array(
            [tab_interp(p1[x], T1[x], ROHL)[0] for x in range(len(p1))]
        )  # kg/m3
        sigma = np.array(
            [tab_interp(p1[x], T1[x], SIGGHL)[0] for x in range(len(p1))]
        )  # N/m

        v_c = vc(rho_l, rho_g, sigma, model)  # m/s

    Qc = [
        (
            (p1[x] * v_c[x] * A * Z2 * (T2 + 273.15))
            / (Z1[x] * p2 * (T1[x] + 273.15))
            * 86400
        )
        for x in range(len(p1))
    ]  # Sm3/d
    return Qc, v_c

#%%
sattime, satdata = prf(sim_folder + filename("sprf")[0])


prestime, presdata = prf(sim_folder + filename("pprf")[0])


Q_sandface = prd(sim_folder + filename("prd")[0], len(sattime))
time3 = Q_sandface["Time"].to_numpy()
QgL5, QgL4, QgL3, QgL2, QgL1, QgTot = (
    Q_sandface["Gasprod"],
    Q_sandface["Gasprod.1"],
    Q_sandface["Gasprod.2"],
    Q_sandface["Gasprod.3"],
    Q_sandface["Gasprod.4"],
    Q_sandface["TotGasprod"],
)
QoL5, QoL4, QoL3, QoL2, QoL1, QoTot = (
    Q_sandface["Oilprod"],
    Q_sandface["Oilprod.1"],
    Q_sandface["Oilprod.2"],
    Q_sandface["Oilprod.3"],
    Q_sandface["Oilprod.4"],
    Q_sandface["TotOilprod"],
)
QwL5, QwL4, QwL3, QwL2, QwL1, QwTot = (
    Q_sandface["Waterprod"],
    Q_sandface["Waterprod.1"],
    Q_sandface["Waterprod.2"],
    Q_sandface["Waterprod.3"],
    Q_sandface["Waterprod.4"],
    Q_sandface["TotWatprod"],
)


#%%


# plotting
fsize = 16
tsize = 16
tdir = "out"  # "in" "out" "inout"
major = 5
minor = 3
style = "default"  # plt.style.available
xsize = 8
ysize = 5

plt.style.use(style)
plt.rcParams["text.usetex"] = True
plt.rcParams["font.size"] = fsize
plt.rcParams["legend.fontsize"] = 14
plt.rcParams["legend.title_fontsize"] = tsize
plt.rcParams["legend.loc"] = "best"
plt.rcParams["xtick.direction"] = tdir
plt.rcParams["ytick.direction"] = tdir
plt.rcParams["xtick.major.size"] = major
plt.rcParams["xtick.minor.size"] = minor
plt.rcParams["ytick.major.size"] = major
plt.rcParams["ytick.minor.size"] = minor

#%% Saturation profile plot


@interact
def sprf_plot(
    ts=widgets.IntSlider(min=0, max=len(sattime) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(satdata[0][1])),
):
    fig, ax = plt.subplots(figsize=(xsize, ysize))
    fig.colorbar(cm.ScalarMappable(norm=None, cmap="jet"), ax=ax)
    ax.pcolormesh(
        satdata[ts][1].reshape(z_grid, -1)[0],
        satdata[ts][3].reshape(z_grid, -1)[:, 0],
        satdata[ts][6].reshape(z_grid, -1),
        cmap="jet",
        shading="auto",
    )
    # ax.grid()
    ax.set_ylabel("Vertical distance, $z$ $[m]$", labelpad=10)
    ax.set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax.set_xlim(xlim)
    ax.set_title(
        "Gas Saturation Profile @ time = {} $hr$".format(round(sattime[ts] / 3600, 1))
    )


@interact
def sprf_plot(
    ts=widgets.IntSlider(min=0, max=len(sattime) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(satdata[0][1])),
):
    fig, ax = plt.subplots(figsize=(xsize, ysize))
    fig.colorbar(cm.ScalarMappable(norm=None, cmap="jet"), ax=ax)
    ax.pcolormesh(
        satdata[ts][1].reshape(z_grid, -1)[0],
        satdata[ts][3].reshape(z_grid, -1)[:, 0],
        satdata[ts][5].reshape(z_grid, -1),
        cmap="jet",
        shading="auto",
    )
    # ax.grid()
    ax.set_ylabel("Vertical distance, $z$ $[m]$", labelpad=10)
    ax.set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax.set_xlim(xlim)
    ax.set_title(
        "Oil Saturation Profile @ time = {} $hr$".format(round(sattime[ts] / 3600, 1))
    )


@interact
def sprf_plot(
    ts=widgets.IntSlider(min=0, max=len(sattime) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(satdata[0][1])),
):
    fig, ax = plt.subplots(figsize=(xsize, ysize))
    fig.colorbar(cm.ScalarMappable(norm=None, cmap="jet"), ax=ax)
    ax.pcolormesh(
        satdata[ts][1].reshape(z_grid, -1)[0],
        satdata[ts][3].reshape(z_grid, -1)[:, 0],
        satdata[ts][4].reshape(z_grid, -1),
        cmap="jet",
        shading="auto",
    )
    # ax.grid()
    ax.set_ylabel("Vertical distance, $z$ $[m]$", labelpad=10)
    ax.set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax.set_xlim(xlim)
    ax.set_title(
        "Water Saturation Profile @ time = {} $hr$".format(round(sattime[ts] / 3600, 1))
    )


@interact
def sprf_plot(
    ts=widgets.IntSlider(min=0, max=len(sattime) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(satdata[0][1])),
):
    fig, ax = plt.subplots(figsize=(xsize, ysize))
    fig.colorbar(cm.ScalarMappable(norm=None, cmap="jet"), ax=ax)
    ax.pcolormesh(
        satdata[ts][1].reshape(z_grid, -1)[0],
        satdata[ts][3].reshape(z_grid, -1)[:, 0],
        (satdata[ts][5] + satdata[ts][4]).reshape(z_grid, -1),
        cmap="jet",
        shading="auto",
    )
    # ax.grid()
    ax.set_ylabel("Vertical distance, $z$ $[m]$", labelpad=10)
    ax.set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax.set_xlim(xlim)
    ax.set_title(
        "Liquid Saturation Profile @ time = {} $hr$".format(
            round(sattime[ts] / 3600, 1)
        )
    )


#%% Pressure profile plot


@interact
def pprf_plot(
    ts=widgets.IntSlider(min=0, max=len(prestime) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(presdata[0][1])),
    ylim=widgets.IntRangeSlider(
        min=np.min(PT_branch[1]) / 1e5 - 5, max=np.max(PT_branch[1]) / 1e5 + 5
    ),
):

    fig, ax = plt.subplots(figsize=(xsize, ysize))
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[0],
        presdata[ts][6].reshape(z_grid, -1)[0],
        label="$p_{g, L 1}$",
    )
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[1],
        presdata[ts][6].reshape(z_grid, -1)[1],
        label="$p_{g, L 2}$",
    )
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[2],
        presdata[ts][6].reshape(z_grid, -1)[2],
        label="$p_{g, L 3}$",
    )
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[3],
        presdata[ts][6].reshape(z_grid, -1)[3],
        label="$p_{g, L 4}$",
    )
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[4],
        presdata[ts][6].reshape(z_grid, -1)[4],
        label="$p_{g, L 5}$",
    )
    try:
        ax.plot(
            presdata[0][1].reshape(z_grid, -1)[5],
            presdata[ts][6].reshape(z_grid, -1)[5],
            label="$p_{g, L 6}$",
        )
        ax.plot(
            presdata[0][1].reshape(z_grid, -1)[6],
            presdata[ts][6].reshape(z_grid, -1)[6],
            label="$p_{g, L 7}$",
        )
        ax.plot(
            presdata[0][1].reshape(z_grid, -1)[7],
            presdata[ts][6].reshape(z_grid, -1)[7],
            label="$p_{g, L 8}$",
        )
        ax.plot(
            presdata[0][1].reshape(z_grid, -1)[8],
            presdata[ts][6].reshape(z_grid, -1)[8],
            label="$p_{g, L 9}$",
        )
        ax.plot(
            presdata[0][1].reshape(z_grid, -1)[9],
            presdata[ts][6].reshape(z_grid, -1)[9],
            label="$p_{g, L 10}$",
        )
        ax.plot(
            presdata[0][1].reshape(z_grid, -1)[10],
            presdata[ts][6].reshape(z_grid, -1)[10],
            label="$p_{g, L 11}$",
        )
    except:
        pass

    ax.set_ylabel("Gas pressure, $p_g$ $[bara]$", labelpad=10)
    ax.set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    ax.set_xlim(xlim)
    # ax.set_ylim(ylim)
    ax.legend()
    ax.set_title(
        "Gas Pressure Profile @ time = {} $hr$".format(round(prestime[ts] / 3600, 1))
    )


@interact
def pprf_plot(
    ts=widgets.IntSlider(min=0, max=len(prestime) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(presdata[0][1])),
    ylim=widgets.IntRangeSlider(
        min=np.min(PT_branch[1]) / 1e5 - 5, max=np.max(PT_branch[1]) / 1e5 + 5
    ),
):

    fig, ax = plt.subplots(figsize=(xsize, ysize))
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[0],
        presdata[ts][5].reshape(z_grid, -1)[0],
        label="$p_{o, L 1}$",
    )
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[1],
        presdata[ts][5].reshape(z_grid, -1)[1],
        label="$p_{o, L 2}$",
    )
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[2],
        presdata[ts][5].reshape(z_grid, -1)[2],
        label="$p_{o, L 3}$",
    )
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[3],
        presdata[ts][5].reshape(z_grid, -1)[3],
        label="$p_{o, L 4}$",
    )
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[4],
        presdata[ts][5].reshape(z_grid, -1)[4],
        label="$p_{o, L 5}$",
    )

    try:
        ax.plot(
            presdata[0][1].reshape(z_grid, -1)[5],
            presdata[ts][5].reshape(z_grid, -1)[5],
            label="$p_{o, L 6}$",
        )
        ax.plot(
            presdata[0][1].reshape(z_grid, -1)[6],
            presdata[ts][5].reshape(z_grid, -1)[6],
            label="$p_{o, L 7}$",
        )
        ax.plot(
            presdata[0][1].reshape(z_grid, -1)[7],
            presdata[ts][5].reshape(z_grid, -1)[7],
            label="$p_{o, L 8}$",
        )
        ax.plot(
            presdata[0][1].reshape(z_grid, -1)[8],
            presdata[ts][5].reshape(z_grid, -1)[8],
            label="$p_{o, L 9}$",
        )
        ax.plot(
            presdata[0][1].reshape(z_grid, -1)[9],
            presdata[ts][5].reshape(z_grid, -1)[9],
            label="$p_{o, L 10}$",
        )
        ax.plot(
            presdata[0][1].reshape(z_grid, -1)[10],
            presdata[ts][5].reshape(z_grid, -1)[10],
            label="$p_{o, L 11}$",
        )
    except:
        pass
    
    ax.set_ylabel("Oil pressure, $p_g$ $[bara]$", labelpad=10)
    ax.set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    ax.set_xlim(xlim)
    # ax.set_ylim(ylim)
    ax.legend()
    ax.set_title(
        "Oil Pressure Profile @ time = {} $hr$".format(round(prestime[ts] / 3600, 1))
    )


@interact
def pprf_plot(
    ts=widgets.IntSlider(min=0, max=len(prestime) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(presdata[0][1])),
    ylim=widgets.IntRangeSlider(
        min=np.min(PT_branch[1]) / 1e5 - 5, max=np.max(PT_branch[1]) / 1e5 + 5
    ),
):

    fig, ax = plt.subplots(figsize=(xsize, ysize))
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[0],
        presdata[ts][4].reshape(z_grid, -1)[0],
        label="$p_{w, L 1}$",
    )
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[1],
        presdata[ts][4].reshape(z_grid, -1)[1],
        label="$p_{w, L 2}$",
    )
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[2],
        presdata[ts][4].reshape(z_grid, -1)[2],
        label="$p_{w, L 3}$",
    )
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[3],
        presdata[ts][4].reshape(z_grid, -1)[3],
        label="$p_{w, L 4}$",
    )
    ax.plot(
        presdata[0][1].reshape(z_grid, -1)[4],
        presdata[ts][4].reshape(z_grid, -1)[4],
        label="$p_{w, L 5}$",
    )


    ax.set_ylabel("Water pressure, $p_g$ $[bara]$", labelpad=10)
    ax.set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    ax.legend()
    ax.set_title(
        "Water Pressure Profile @ time = {} $hr$".format(round(prestime[ts] / 3600, 1))
    )


#%% Flow through sand-face plot


@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time3) / 3600)):

    fig, ax = plt.subplots(figsize=(xsize, ysize))
    ax.plot(time3 / 3600, QgL1, label="$\dot{m}_{g, L 1}$")
    ax.plot(time3 / 3600, QgL2, label="$\dot{m}_{g, L 2}$")
    ax.plot(time3 / 3600, QgL3, label="$\dot{m}_{g, L 3}$")
    ax.plot(time3 / 3600, QgL4, label="$\dot{m}_{g, L 4}$")
    ax.plot(time3 / 3600, QgL5, label="$\dot{m}_{g, L 5}$")
    ax.set_ylabel("Gas mass rate, $\dot{m}_g$ $[kg/s]$", labelpad=10)
    ax.set_xlabel("Time, $t$ $[hr]$", labelpad=10)
    ax.set_xlim(xlim)
    ax.legend()
    ax.set_title("Mass Rate of Gas Through Sandface, Numeric Layer 1-5")
    fig.savefig("plots/gasmassrate.png", dpi=1000)


@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time3) / 3600)):
    fig, ax = plt.subplots(figsize=(xsize, ysize))
    ax.plot(time3 / 3600, QoL1, label="$\dot{m}_{o, L 1}$")
    ax.plot(time3 / 3600, QoL2, label="$\dot{m}_{o, L 2}$")
    ax.plot(time3 / 3600, QoL3, label="$\dot{m}_{o, L 3}$")
    ax.plot(time3 / 3600, QoL4, label="$\dot{m}_{o, L 4}$")
    ax.plot(time3 / 3600, QoL5, label="$\dot{m}_{o, L 5}$")
    ax.set_ylabel("Oil mass rate, $\dot{m}_o$ $[kg/s]$", labelpad=10)
    ax.set_xlabel("Time, $t$ $[hr]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax.set_xlim(xlim)
    ax.legend()
    ax.set_title("Mass Rate of Oil Through Sandface, Numeric Layer 1-5")
    fig.savefig("plots/oilmassrate.png", dpi=1000)


@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time3) / 3600)):
    fig, ax = plt.subplots(figsize=(xsize, ysize))
    ax.plot(time3 / 3600, QwL1, label="$\dot{m}_{w, L 1}$")
    ax.plot(time3 / 3600, QwL2, label="$\dot{m}_{w, L 2}$")
    ax.plot(time3 / 3600, QwL3, label="$\dot{m}_{w, L 3}$")
    ax.plot(time3 / 3600, QwL4, label="$\dot{m}_{w, L 4}$")
    ax.plot(time3 / 3600, QwL5, label="$\dot{m}_{w, L 5}$")
    ax.set_ylabel("Water mass rate, $\dot{m}_o$ $[kg/s]$", labelpad=10)
    ax.set_xlabel("Time, $t$ $[hr]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax.set_xlim(xlim)
    ax.legend()
    ax.set_title("Mass Rate of Water Through Sandface, Numeric Layer 1-5")
    fig.savefig("plots/watermassrate.png", dpi=1000)


@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time3) / 3600)):
    fig, ax = plt.subplots(figsize=(xsize, ysize))
    ax.plot(time3 / 3600, QoTot, "g", label="$\dot{m}_{o, tot}$")
    ax.plot(time3 / 3600, QgTot, "r", label="$\dot{m}_{g, tot}$")
    ax.plot(time3 / 3600, QwTot, "b", label="$\dot{m}_{w, tot}$")
    ax.set_ylabel("Mass rate, $\dot{m}$ $[kg/s]$", labelpad=10)
    ax.set_xlabel("Time, $t$ $[hr]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax.set_xlim(xlim)
    ax.legend()
    ax.set_title("Total Mass Rates Through Sandface")


#%% Nodal plot

p_ri = np.average(presdata[0][6])
p_r = np.array([np.average(presdata[i][6]) for i in range(len(presdata))])
# P_wf = np.linspace(0, P_R, 100)
# QgIPR = [backpressure(P_R, wf, c, n) for wf in P_wf]
# try:
#     plt.rcParams["legend.fontsize"] = 10
#     fig, ax = plt.subplots(figsize=(xsize*1.3, ysize*1.1))
#     ax.plot(QgIPR, P_wf, "r", label="IPR")
#     for i, file in enumerate(tpl_files):
#         # time, LIQC, QGST, QWST, PT = dpp.getTPLData(path_of_directory + "/", file)

#         # QGST = 86400 * QGST
#         # PT = 1e-5 * PT
#         kernel_size = 20
#         kernel = np.ones(kernel_size) / kernel_size
#         # QGST = np.convolve(QGST, kernel, mode="same")
#         # PT = np.convolve(PT, kernel, mode="same")

#         # idx = find_nearest(QGST[2:], 5500)[0]
#         # qg0 = find_nearest(QGST[2:], max(QgIPR))[0]

#         # ax.plot(QGST[1:], PT[1:], label=f"TPR, WGR = {WGR[i]}")
#         x = np.convolve(QgTPR[i][:-3], kernel, mode="same") #[QgTPR[i]<max(QgIPR)]
#         y = np.convolve(PTTPR[i][:len(x)], kernel, mode="same")
#         ax.plot(x[15:-15], y[15:-15], label=f"TPR, {test_property} = {WGR[i]} {test_property_unit}")
#     plt.xlim(0, max(QgIPR) * 1.2)
#     plt.ylim(0, max(P_wf) * 1.2)
#     ax.set_xlabel("Gas Production Rate, $QGST$ $[Sm^3/d]$")
#     ax.set_ylabel("Flowing Bottomhole Pressure, $P_{wf}$ $[bara]$")
#     ax.set_title("Nodal Analysis of IPR and TPR")
#     ax.grid()
#     ax.legend()


# @interact
# def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=np.max(time)/3600)):
#     fig, ax0 = plt.subplots(figsize=(xsize, ysize))
#     # ax0.grid(True)
#     (p0,) = ax0.plot(QGST[xlim[0]:xlim[1]]*86400, PT[xlim[0]:xlim[1]]/1e5, label="$TPC$")
#     ax0.set_ylabel("Pressure, $p_{wf}$ $[bara]$", fontsize=fsize)
#     ax0.set_xlabel("Gas flow rate, $q_g$ $[Sm^3/d]$", fontsize=fsize)
#     # ax0.set_ylim(1, 4)
#     # ax0.set_yticks([1, 2, 3, 4])
#     # ax0.set_yticklabels(["Stratified", "Annular", "Slug", "Bubbly"])

#     ax1 = ax0.twinx()
#     # ax1.set_ylim(90, 130)
#     (p1,) = ax1.plot(QGST[xlim[0]:xlim[1]]*86400, p_r[xlim[0]:xlim[1]], "r", label="$T$")
#     # # ax1.grid(False)
#     # ax1.set_ylabel("Temperature, $T$ $[^oC]$", fontsize=16)
#     # ax1.tick_params(axis="both", labelsize=16)
#     # ax0.set_xlim(xlim)
#     ax0.set_title(
#         "Nodal Plot of Tubing and Reservoir Performance"
#     )
#     fig.legend(borderaxespad=3)


@interact
def ppl_plot(
    xlim=widgets.IntRangeSlider(min=0, max=np.max(time) / 3600),
    n=widgets.FloatSlider(min=0.5, max=1, step=0.01),
):
    p_Rwf = [presdata[i][6][0] for i, p in enumerate(presdata)]
    p_Ravg = [np.average(presdata[i][6]) for i, p in enumerate(presdata)]
    p_Ri = p_Ravg[0]
    p_wf = np.linspace(0, p_Ri, 100)
    i = 10
    press = p_Rwf[i]
    temp = TM_branch[1][i][0]
    yg = tab_interp(1.01325, 20, ROG)[0] / 1.205
    Z = gas.ZfacStanding(press, temp, yg, 2)
    visg = tab_interp(press, temp, VISG) * 1000
    QGRES = (
        C(10, 100, temp, visg, Z, 0, 500, 0.05715 / 2 / 0.0254, 0, n)
        * (p_Ri ** 2 - p_wf ** 2) ** n
    )
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    # ax0.grid(True)
    (p0,) = ax0.plot(
        QGST[xlim[0] : xlim[1]] * 86400, PT[xlim[0] : xlim[1]] / 1e5, label="$TPC$"
    )
    ax0.set_ylabel("Pressure, $p_{wf}$ $[bara]$", fontsize=fsize)
    ax0.set_xlabel("Gas flow rate, $q_g$ $[Sm^3/d]$", fontsize=fsize)
    (p0,) = ax0.plot(QGRES, p_wf, "r", label="$IPR$")
    fig.legend(borderaxespad=3)
    ax0.set_title("Nodal Plot of Tubing and Reservoir Performance")


# %%
# OLGA profile plot
timesteps = len(time)


@interact
def ppl_plot(
    ts=widgets.IntSlider(min=0, max=timesteps - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(ID[0])),
    loc=widgets.IntSlider(min=1, max=10),
):
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    # ax0.grid(False)
    (p0,) = ax0.plot(ID[0], ID[1][ts], label="Flow regime")
    ax0.set_ylabel("Flow regime identification", fontsize=fsize)
    ax0.set_xlabel("Distance along tubing, $z$ $[m]$", fontsize=fsize)
    ax0.set_ylim(1, 4)
    ax0.set_yticks([1, 2, 3, 4])
    ax0.set_yticklabels(["Stratified", "Annular", "Slug", "Bubbly"])

    ax1 = ax0.twinx()
    # ax1.set_ylim(90, 130)
    (p1,) = ax1.plot(HOL[0], HOL[1][ts], "r", label="$H_L$")
    ax1.grid()
    ax1.set_ylabel("Liquid holdup, $H_L$ $[-]$", fontsize=fsize)
    ax1.tick_params(axis="both", labelsize=fsize)
    ax0.set_xlim(xlim)
    fig.legend(loc=loc, borderaxespad=3)
    ax0.set_title(
        "Flow Regime and Liquid Holdup @ time = {} hr".format(round(time[ts] / 3600))
    )
    fig.savefig("plots/flowpattern.png", dpi=1000)



@interact
def ppl_plot(
    ts=widgets.IntSlider(min=0, max=timesteps - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(ID[0])),
):
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    ax0.grid(True)
    (p0,) = ax0.plot(PT_branch[0], PT_branch[1][ts] / 1e5, label="$p$")
    ax0.set_ylabel("Pressure, $p$ $[bara]$", fontsize=fsize)
    ax0.set_xlabel("Distance along tubing, $z$ $[m]$", fontsize=fsize)
    ax0.set_ylim(np.min(PT_branch[1]) * 1e-5 - 5, np.max(PT_branch[1]) * 1e-5 + 5)
    # ax0.set_ylim(1, 4)
    # ax0.set_yticks([1, 2, 3, 4])
    # ax0.set_yticklabels(["Stratified", "Annular", "Slug", "Bubbly"])

    ax1 = ax0.twinx()
    # ax1.set_ylim(90, 130)
    (p1,) = ax1.plot(TM_branch[0], TM_branch[1][ts], "r", label="$T$")
    # ax1.grid(False)
    ax1.set_ylabel("Temperature, $T$ $[^oC]$", fontsize=fsize)
    ax1.set_ylim(np.min(TM_branch[1]) - 5, np.max(TM_branch[1]) + 5)
    ax1.tick_params(axis="both", labelsize=fsize)
    ax0.set_xlim(xlim)
    ax0.set_title(
        "Pressure and Temperature @ time = {} hr".format(round(time[ts] / 3600))
    )
    fig.legend(borderaxespad=3)


@interact
def ppl_plot(
    ts=widgets.IntSlider(min=0, max=timesteps - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(ID[0])),
):
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    ax0.grid(True)
    (p0,) = ax0.plot(UG[0], UG[1][ts], "r-", label="$u_{g}$")
    (p0,) = ax0.plot(UD[0], UD[1][ts], "g-", label="$u_{d}$")
    (p0,) = ax0.plot(UL[0], UL[1][ts], "b-", label="$u_{l}$")
    ax0.set_ylabel("Velocity, $u$ $[m/s]$", fontsize=fsize)
    ax0.set_xlabel("Distance along tubing , $z$ $[m]$", fontsize=fsize)
    ax0.set_title(
        "Velocity of gas, liquid film, and droplets @ time = {} hr".format(
            round(time[ts] / 3600)
        )
    )
    ax0.set_xlim(xlim)
    fig.legend(borderaxespad=3)
    # ax0.set_ylim(1, 4)
    # ax0.set_yticks([1, 2, 3, 4])
    # ax0.set_yticklabels(["Stratified", "Annular", "Slug", "Bubbly"])


#%%
# OLGA Trend data plot
@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time) / 3600)):
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    (p0,) = ax0.plot(time / 3600, QGST * 86400, "r", label="$q_{g}$")
    ax0.set_ylabel("Gas flow rate, $q_{g}$ $[Sm^3/d]$ ", fontsize=fsize)
    ax0.set_xlabel("Time, $t$ $[hr]$", fontsize=fsize)

    ax1 = ax0.twinx()
    (p1,) = ax1.plot(time / 3600, QOST * 86400, "g", label="$q_{o}$")
    # ax1.grid()
    ax1.set_ylabel("Oil flow rate, $q_{o}$ $[Sm^3/d]$ ", fontsize=16)
    ax1.tick_params(axis="both", labelsize=16)
    ax0.set_xlim(xlim)
    fig.legend(borderaxespad=4)
    ax0.set_title("Surface Flow Rates of Oil and Gas")
    fig.savefig("plots/surfacerate.png", dpi=1000)


@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time) / 3600)):
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    (p0,) = ax0.plot(time / 3600, PT * 1e-5, "k", label="$p_{wf}$")
    ax0.set_ylabel("Bottomhole pressure, $p_{wf}$ $[bara]$ ", fontsize=fsize)
    ax0.set_xlabel("Time, $t$ $[hr]$", fontsize=fsize)

    ax1 = ax0.twinx()
    (p1,) = ax1.plot(time / 3600, LIQFR, "b", label="$\phi_L$")
    # ax1.grid()
    ax1.set_ylabel("Liquid fraction, $\phi_L$  $[-]$ ", fontsize=fsize)
    ax1.tick_params(axis="both", labelsize=fsize)
    ax0.set_xlim(xlim)
    fig.legend(borderaxespad=3)
    ax0.set_title("Bottomhole Pressure and Liquid Fraction")
    fig.savefig("plots/bhp.png", dpi=1000)


@interact
def ppl_plot(
    ts=widgets.IntSlider(min=0, max=len(time) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(ID[0])),
    ylim=widgets.FloatRangeSlider(
        min=np.min([UG[1], UD[1], UL[1]]), max=np.max([UG[1], UD[1], UL[1]])
    ),
):
    critical_rate = Qc(
        PT_branch[1][ts] * 1e-5, 1.01325, TM_branch[1][ts], 15.556, 0.05715
    )[1]
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    # ax0.set_ylim(1, 4)
    # ax0.invert_yaxis()
    # ax0.set_yticklabels(["Stratified", "Annular", "Slug", "Bubbly"])
    (p0,) = ax0.plot(PT_branch[0], critical_rate, "k", label="$u_{c, Turner}$")
    (p1,) = ax0.plot(UG[0], UG[1][ts], "r-", label="$u_{g}$")
    (p2,) = ax0.plot(UD[0], UD[1][ts], "g-", label="$u_{d}$")
    (p3,) = ax0.plot(UL[0], UL[1][ts], "b-", label="$u_{l}$")
    ax0.set_title(
        f"Phase Velocity and Turner Critical Velocity @ t = {round(time[ts]/3600,1)} hr"
    )
    ax0.set_ylabel("Velocity, $u$ $[m/s]$", fontsize=fsize)
    ax0.set_xlabel("Distance along tubing , $z$ $[m]$", fontsize=fsize)
    ax0.set_ylim(ylim)
    ax0.set_xlim(xlim)
    fig.legend(borderaxespad=3)
    fig.savefig("plots/phasevelocity.png", dpi=800)


# #%%
# @interact
# def ppl_plot(
#     ts=widgets.IntSlider(min=0, max=len(time) - 1),
#     xlim=widgets.IntRangeSlider(min=0, max=np.max(ID[0])),
#     ylim=widgets.FloatRangeSlider(
#         min=np.min([UG[1], UD[1], UL[1]]), max=np.max([UG[1], UD[1], UL[1]])
#     ),
# ):
#     critical_rate = Qc(
#         PT_branch[1][ts] * 1e-5, 1.01325, TM_branch[1][ts], 15.556, 0.05715
#     )[1]
#     fig, ax0 = plt.subplots(figsize=(xsize, ysize))
#     # ax0.set_ylim(1, 4)
#     # ax0.invert_yaxis()
#     # ax0.set_yticklabels(["Stratified", "Annular", "Slug", "Bubbly"])
#     # (p0,) = ax0.plot(PT_branch[0], critical_rate, "k", label="$u_{c, Turner}$")
#     (p1,) = ax0.plot(USG[0], USG[1][ts], "r-", label="$u_{sg}$")
#     (p2,) = ax0.plot(USD[0], USD[1][ts], "g-", label="$u_{sd}$")
#     (p3,) = ax0.plot(USL[0], USL[1][ts], "b-", label="$u_{sl}$")
#     ax0.set_ylabel("Superficial velocity, $u_s$ $[m/s]$", fontsize=fsize)
#     ax0.set_xlabel("Distance along tubing , $z$ $[m]$", fontsize=fsize)
#     ax0.set_title("Superficial Phase Velocity")
#     ax0.set_ylim(ylim)
#     ax0.set_xlim(xlim)
#     fig.legend(borderaxespad=3)


# # %%
# # Fluid prop plot
# def plot_property_keyword(pressure, temperature, thermo_property):
#     fig = plt.figure(figsize=(16, 12))
#     ax = fig.add_subplot(111, projection="3d")
#     X = []
#     Y = []
#     for x, y in it.product(pressure, temperature):
#         X.append(x / 1e5)
#         Y.append(y)
#     ax.scatter(X, Y, thermo_property)
#     ax.set_ylabel("Temperature [C]")
#     ax.set_xlabel("Pressure [bar]")
#     ax.set_xlim(
#         0,
#     )
#     ax.set_title("RS")
#     return fig


# pressure, temperature, ROG, ROHL, RS, SIGGHL, VISG, VISHL = getTABdata(sim_folder, filename(".tab")[0])

# plot_property_keyword(np.array(pressure).reshape(len(np.unique(pressure)),len(np.unique(temperature)))[:,0],
#                       np.array(temperature).reshape(len(np.unique(pressure)),len(np.unique(temperature)))[0,:],
#                       np.array(RS).reshape(len(np.unique(pressure)),len(np.unique(temperature))))


#%% liquid loading identifier indexes
try:
    rate_gradient_search_idx = (
        # np.where(abs(np.gradient(QGST[50:])) > 0.00088)[0][0] + 50
        np.where(abs(np.gradient(QGST[50:], time[50:])) > 0.00001)[0][0]
        + 50
    )
    rate_gradient_search_rate = QGST[rate_gradient_search_idx]
    rate_gradient_search_pressure = PT[rate_gradient_search_idx]
    print(
        f"Rate gradient search index={rate_gradient_search_idx}, rate={rate_gradient_search_rate*86400}, pressure={rate_gradient_search_pressure/1e5}, time={time[rate_gradient_search_idx]/3600}"
    )
except:
    rate_gradient_search_idx = 0
    rate_gradient_search_rate = QGST[rate_gradient_search_idx] * 86400
    rate_gradient_search_pressure = PT[rate_gradient_search_idx] * 1e-5
    print("dq/dt < 0 does not exist")

slug_identifier_idx = find_slug(ID[1], 3)[0]
slug_identifier_rate = QGST[slug_identifier_idx]
slug_identifier_pressure = PT[slug_identifier_idx]
if slug_identifier_idx == 0:
    print("Slug does not exist")
else:
    print(
        f"Slug identifier index={slug_identifier_idx}, rate={slug_identifier_rate*86400}, pressure={slug_identifier_pressure/1e5}, time={time[slug_identifier_idx]/3600}"
    )

try:
    liquidfilm_reversal_idx = np.where(np.asarray(UL[1]) < 0)[0][0]
    liquidfilm_reversal_rate = QGST[liquidfilm_reversal_idx]
    liquidfilm_reversal_pressure = PT[liquidfilm_reversal_idx]
    print(
        f"Liquid film reversal index={liquidfilm_reversal_idx}, rate={liquidfilm_reversal_rate*86400}, pressure={liquidfilm_reversal_pressure/1e5}, time={time[liquidfilm_reversal_idx]/3600}"
    )
    cr1 = Qc(
        PT_branch[1][liquidfilm_reversal_idx - 10] * 1e-5,
        1.01325,
        TM_branch[1][liquidfilm_reversal_idx - 10],
        15.556,
        0.05715,
    )[0]
    cr2 = Qc(
        PT_branch[1][liquidfilm_reversal_idx + 10] * 1e-5,
        1.01325,
        TM_branch[1][liquidfilm_reversal_idx + 10],
        15.556,
        0.05715,
    )[0]
    print(
        f"Turner critical rate pre/post liquid film reversal = {np.max(cr1), np.max(cr2)}"
    )
except:
    print("UL > 0")
    liquidfilm_reversal_idx = 0

try:
    droplet_reversal_idx = np.where(np.asarray(UD[1]) < 0)[0][0]
    droplet_reversal_rate = QGST[droplet_reversal_idx]
    droplet_reversal_pressure = PT[droplet_reversal_idx]
    print(
        f"Droplet reversal index={droplet_reversal_idx}, rate={droplet_reversal_rate*86400}, pressure={droplet_reversal_pressure/1e5}, time={time[droplet_reversal_idx]/3600}"
    )
    cr1 = Qc(
        PT_branch[1][droplet_reversal_idx - 10] * 1e-5,
        1.01325,
        TM_branch[1][droplet_reversal_idx - 10],
        15.556,
        0.05715,
    )[0]
    cr2 = Qc(
        PT_branch[1][droplet_reversal_idx + 10] * 1e-5,
        1.01325,
        TM_branch[1][droplet_reversal_idx + 10],
        15.556,
        0.05715,
    )[0]
    print(
        f"Turner critical rate pre/post droplet reversal = {np.max(cr1), np.max(cr2)}"
    )

except:
    droplet_reversal_idx = 0
    print("UL > 0")


try:
    oil_backflow_index = len(QoL5)
    for i, L in enumerate([QoL5, QoL4, QoL3, QoL2, QoL1]):
        try:
            idx = np.where(L < 0)[0][0]
            print(f"QoL{5-i} has backflow @ index={idx}, time={time3[idx]/3600} hr")
            if idx < oil_backflow_index:
                oil_backflow_index = idx
        except:
            print(f"QoL{5-i} does not have backflow")
    # if oil_backflow_index != len(QoL5):
    #     print(
    #         f"Oil backflow index = {oil_backflow_index}, time={time3[oil_backflow_index]/3600} hr"
        # )
except:
    print("Oil does not flow back")

try:
    water_backflow_index = len(QwL5)
    for L in (QwL5, QwL4, QwL3, QwL2, QwL1):
        idx = np.where(L < 0)[0][0]
        if idx < water_backflow_index:
            water_backflow_index = idx
    if water_backflow_index != len(QwL5):
        print(
            f"Water backflow index = {water_backflow_index}, time={time3[water_backflow_index]/3600} hr"
        )
except:
    print("Water does not flow back")

try:
    gas_backflow_index = len(QgL5)
    for L in (QgL5, QgL4, QgL3, QgL2, QgL1):
        idx = np.where(L < 0)[0][0]
        if idx < gas_backflow_index:
            gas_backflow_index = idx
    if gas_backflow_index != len(QgL5):
        print(
            f"Gas backflow index = {gas_backflow_index}, time={time3[gas_backflow_index]/3600} hr"
        )
except:
    print("Gas does not flow back")

try:
    post_transient_idx = []
    for i in (QgL1, QgL2, QgL3, QgL4, QgL5):
        try:
            post_transient_idx.append(np.where(
                    abs(np.gradient(i[oil_backflow_index:], time3[oil_backflow_index:]))
                    <= 0.00000000001
                )[0][0])
        except:
            pass
    post_transient_idx = (np.min(post_transient_idx) + oil_backflow_index)+4*36

    pre_transient_idx = (rate_gradient_search_idx)-4*36

    print(f"gas mass rate layer 1 pre/post/change = {QgL1[pre_transient_idx], QgL1[post_transient_idx], QgL1[post_transient_idx]/QgL1[pre_transient_idx]}")
    print(f"gas mass rate layer 2 pre/post/change = {QgL2[pre_transient_idx], QgL2[post_transient_idx], QgL2[post_transient_idx]/QgL2[pre_transient_idx]}")
    # print(f"gas mass rate layer 3 pre/post/change = {QgL3[pre_transient_idx], QgL3[post_transient_idx], QgL3[post_transient_idx]/QgL3[pre_transient_idx]}")
    print(f"gas mass rate layer 4 pre/post/change = {QgL4[pre_transient_idx], QgL4[post_transient_idx], QgL4[post_transient_idx]/QgL4[pre_transient_idx]}")
    print(f"gas mass rate layer 5 pre/post/change = {QgL5[pre_transient_idx], QgL5[post_transient_idx], QgL5[post_transient_idx]/QgL5[pre_transient_idx]}")


    print(f"oil mass rate layer 1 pre/post/change = {QoL1[pre_transient_idx], np.average(QoL1[post_transient_idx:])}")
    print(f"oil mass rate layer 2 pre/post/change = {QoL2[pre_transient_idx], np.average(QoL2[post_transient_idx:])}")
    print(f"oil mass rate layer 3 pre/post/change = {QoL3[pre_transient_idx], np.average(QoL3[post_transient_idx:])}")
    print(f"oil mass rate layer 4 pre/post/change = {QoL4[pre_transient_idx], np.average(QoL4[post_transient_idx:])}")
    print(f"oil mass rate layer 5 pre/post/change = {QoL5[pre_transient_idx], np.average(QoL5[post_transient_idx:])}")

    i = post_transient_idx
    layer1 = np.where(satdata[i][5][0:30]==0)[0][0]
    layer2 = np.where(satdata[i][5][30:60]==0)[0][0]
    layer3 = np.where(satdata[i][5][60:90]==0)[0][0]
    layer4 = np.where(satdata[i][5][90:120]==0)[0][0]
    layer5 = np.where(satdata[i][5][120:150]==0)[0][0]

    layer1, layer2, layer3, layer4, layer5

    # make minimum radial penetration function (min (l1, ..., l5)). insert instead of layer5 below

    avg_oilsat1 = np.average(satdata[i][5][0:layer5], weights=np.diff(satdata[i][1][0:layer5+1]))
    avg_oilsat2 = np.average(satdata[i][5][30:30+layer5], weights=np.diff(satdata[i][1][0:layer5+1]))
    avg_oilsat3 = np.average(satdata[i][5][60:60+layer5], weights=np.diff(satdata[i][1][0:layer5+1]))
    avg_oilsat4 = np.average(satdata[i][5][90:90+layer5], weights=np.diff(satdata[i][1][0:layer5+1]))
    avg_oilsat5 = np.average(satdata[i][5][120:120+layer5], weights=np.diff(satdata[i][1][0:layer5+1]))
    oildepth1 = satdata[i][1][layer1]
    oildepth2 = satdata[i][1][layer2]
    oildepth3 = satdata[i][1][layer3]
    oildepth4 = satdata[i][1][layer4]
    oildepth5 = satdata[i][1][layer5]

    print(avg_oilsat1, avg_oilsat2, avg_oilsat3, avg_oilsat4, avg_oilsat5, oildepth1, oildepth2, oildepth3, oildepth4, oildepth5)
except:
    print("error")
# %%

# DEWPRESSURES = np.array([ 100000.000 , 281530.302 , 404081.633 , 506122.449 , 708163.265 , 912244.898 , 1012244.90 , 1316326.53 , 1318367.35 , 1620408.16 , 1724489.80 , 1924489.80 , 2130612.24 , 2228571.43 , 2532653.06 , 2536734.69 , 2836734.69 , 2942857.14 , 3140816.33 , 3348979.59 , 3444897.96 , 3748979.59 , 3755102.04 , 4053061.22 , 4161224.49 , 4357142.86 , 4567346.94 , 4661224.49 , 4965306.12 , 4973469.39 , 5269387.76 , 5379591.84 , 5573469.39 , 5785714.29 , 5877551.02 , 6181632.65 , 6191836.73 , 6485714.29 , 6597959.18 , 6789795.92 , 7004081.63 , 7093877.55 , 7397959.18 , 7410204.08 , \
# 7702040.82 , 7816326.53 , 8006122.45 , 8222448.98 , 8310204.08 , 8614285.71 , 8628571.43 , 8918367.35 , 9034693.88 , 9222448.98 , 9440816.33 , 9526530.61 , 9830612.24 , 9846938.78 , 10134693.9 , 10253061.2 , 10438775.5 , 10659183.7 , 10742857.1 , 11046938.8 , 11065306.1 , 11351020.4 , 11471428.6 , 11655102.0 , 11877551.0 , 11959183.7 , 12263265.3 , 12283673.5 , 12567346.9 , 12689795.9 , 12871428.6 , 13095918.4 , 13175510.2 , 13479591.8 , 13502040.8 , 13783673.5 , 13908163.3 , 14087755.1 , 14314285.7 , 14391836.7 , 14695918.4 , 14720408.2 , 15000000.0 , 15126530.6 , 15532653.1 , 15938775.5 ,\
# 16344898.0 , 16751020.4 , 17157142.9 , 17563265.3 , 17969387.8 , 18345471.6 , 18375510.2 , 18781632.7 , 18928579.6 , 18781632.7 , 18375510.2 , 18004278.6 , 17969387.8 , 17563265.3 , 17157142.9 , 16751020.4 , 16344898.0 , 15938775.5 , 15532653.1 , 15000000.0 , 14695918.4 , 14391836.7 , 14314285.7 , 13908163.3 , 13783673.5 , 13502040.8 , 13479591.8 , 12871428.6 , 12567346.9 , 11959183.7 , 11655102.0 , 11471428.6 , 11046938.8 , 9368233.20 , 7566346.59 , 5807597.38 , 4113991.39 , 2993534.11])
# DEWTEMPERATURES = np.array([ 38.2285945 , 56.5841627 , 63.3713423 , 67.6673246 , 74.1212112 , 78.9704259 , 80.9434187 , 85.8309168 , 85.8591984 , 89.5456851 , 90.6213393 , 92.4651775 , 94.1053610 , 94.8054633 , 96.7023879 , 96.7252867 , 98.2471957 , 98.7158861 , 99.5042590 , 100.223737 , 100.520746 , 101.332277 , 101.346714 , 101.966424 , 102.153487 , 102.444956 , 102.694094 , 102.785357 , 103.001867 , 103.006086 , 103.106221 , 103.118304 , 103.108190 , 103.053314 , 103.015971 , 102.836493 , 102.829029 , 102.575643 , 102.459820 , 102.238441 , 101.957304 , 101.829178 , 101.351530 , 101.330910 , \
# 100.808637 , 100.588290 , 100.203179 , 99.7356283 , 99.5374277 , 98.8132923 , 98.7778644 , 98.0323548 , 97.7188643 , 97.1958980 , 96.5615429 , 96.3049268 , 95.3601813 , 95.3079463 , 94.3621547 , 93.9593163 , 93.3110980 , 92.5161242 , 92.2070218 , 91.0496949 , 90.9780746 , 89.8386378 , 89.3440876 , 88.5731122 , 87.6122543 , 87.2521048 , 85.8743054 , 85.7797635 , 84.4380775 , 83.8427903 , 82.9414211 , 81.7963385 , 81.3819240 , 79.7567002 , 79.6340216 , 78.0623102 , 77.3477558 , 76.2946595 , 74.9273324 , 74.4488653 , 72.5190822 , 72.3598059 , 70.4982681 , 69.6286041 , 66.7121903 , 63.5819781 ,\
# 60.1992005 , 56.5095085 , 52.4304028 , 47.8278712 , 42.4545232 , 36.3205752 , 35.7552550 , 25.6975966 , 18.1079282 , 1.96395979 , -7.9292506 , -13.989787 , -14.490193 , -19.746013 , -24.246520 , -28.238040 , -31.856352 , -35.185773 , -38.282938 , -42.058877 , -44.091779 , -46.047339 , -46.534640 , -49.016882 , -49.755765 , -51.392182 , -51.520607 , -54.895526 , -56.513395 , -59.626098 , -61.126340 , -62.015792 , -64.028287 , -71.775304 , -78.904488 , -84.892485 , -90.391510 , -93.918222 ])

# plt.plot(DEWTEMPERATURES, DEWPRESSURES/1e5)
# plt.xlabel("Temperature, $T$ $[^oC]$")
# plt.ylabel("Pressure, $p$ $[bara]$")
# plt.title("Phase Envelope of Wet Gas")

#%% relative perm
# krw = lambda Sat_l, krwoc, N: krwoc*(Sat_l)**N

# def krw(krwc, Sw, Swc, Sor, Nw=2):
#     return krwc*((Sw - Swc)/(1-Swc-Sor))**Nw

# def krg(krgc, Sw, Swc, Sog, Ng=2):
#     return krgc*((1 - Sw - Sog)/(1-Swc-Sog))**Ng

# N = 2
# kr_irr = 1
# Sat_l = np.linspace(0,1,21)
# krw(1, Sat_l, 0.1, 0)
# Sat_l
# corey correlation
# Sw = [
#     0,
#     0.05,
#     0.1,
#     0.15,
#     0.2,
#     0.25,
#     0.3,
#     0.35,
#     0.4,
#     0.45,
#     0.5,
#     0.55,
#     0.6,
#     0.65,
#     0.7,
#     0.75,
#     0.8,
#     0.85,
#     0.9,
#     1,
# ]
# Sg = 1 - np.array(Sw)
# # simple (should use stone II)
# So = [
#     0.1,
#     0.11,
#     0.12,
#     0.15,
#     0.2,
#     0.25,
#     0.3,
#     0.35,
#     0.4,
#     0.45,
#     0.5,
#     0.55,
#     0.6,
#     0.65,
#     0.7,
#     0.75,
#     0.8,
#     0.85,
#     0.9,
#     0.95,
#     1,
# ]

# krg = [
#     0,
#     0.0030864,
#     0.012346,
#     0.027778,
#     0.049383,
#     0.07716,
#     0.11111,
#     0.15123,
#     0.19753,
#     0.25,
#     0.30864,
#     0.37346,
#     0.44444,
#     0.5216,
#     0.60494,
#     0.69444,
#     0.79012,
#     0.89198,
#     1,
#     1,
# ]
# krw = [
#     0,
#     0.0030864,
#     0.012346,
#     0.027778,
#     0.049383,
#     0.07716,
#     0.11111,
#     0.15123,
#     0.19753,
#     0.25,
#     0.30864,
#     0.37346,
#     0.44444,
#     0.5216,
#     0.60494,
#     0.69444,
#     0.79012,
#     0.89198,
#     1,
#     1,
# ]
# kro = [
#     0,
#     0.003,
#     0.005,
#     0.013,
#     0.025,
#     0.038,
#     0.05,
#     0.082,
#     0.114,
#     0.145,
#     0.177,
#     0.233,
#     0.289,
#     0.344,
#     0.4,
#     0.48,
#     0.56,
#     0.64,
#     0.72,
#     0.86,
#     1,
# ]


# plt.plot(Sw, krw)
# plt.plot(Sg, krg)
# %%
# plt.style.use(style)
# plt.rcParams["text.usetex"] = True
# plt.rcParams["font.size"] = 12
# plt.rcParams["legend.fontsize"] = 8
# plt.rcParams["legend.title_fontsize"] = 8
# plt.rcParams["legend.loc"] = "best"
# plt.rcParams["xtick.direction"] = tdir
# plt.rcParams["ytick.direction"] = tdir
# plt.rcParams["xtick.major.size"] = major
# plt.rcParams["xtick.minor.size"] = minor
# plt.rcParams["ytick.major.size"] = major
# plt.rcParams["ytick.minor.size"] = minor


@interact
def sprf_plot(
    ts=widgets.IntSlider(min=0, max=len(sattime) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(satdata[0][1])),
):
    fig, ax = plt.subplots(nrows=2, ncols=1)
    ax[1].pcolormesh(
        satdata[ts][1].reshape(z_grid, -1)[0],
        satdata[ts][3].reshape(z_grid, -1)[:, 0],
        satdata[ts][6].reshape(z_grid, -1),
        cmap="jet",
        shading="auto",
    )
    # ax.grid()
    ax[1].set_ylabel("Vertical distance, $z$ $[m]$", labelpad=10)
    ax[1].set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax[1].set_xlim(xlim)
    # ax[1].yaxis.set_label_position("right")
    # ax[1].set_title(
    #     "Gas Saturation Profile @ time = {} $hr$".format(round(sattime[ts] / 3600, 1))
    # )
    fig.colorbar(cm.ScalarMappable(norm=None, cmap="jet"), ax=ax, location="right")
    # fig.tight_layout(pad=0.2)

    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[0],
        presdata[ts][6].reshape(z_grid, -1)[0],
        label="$p_{g, L 1}$",
    )
    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[1],
        presdata[ts][6].reshape(z_grid, -1)[1],
        label="$p_{g, L 2}$",
    )
    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[2],
        presdata[ts][6].reshape(z_grid, -1)[2],
        label="$p_{g, L 3}$",
    )
    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[3],
        presdata[ts][6].reshape(z_grid, -1)[3],
        label="$p_{g, L 4}$",
    )
    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[4],
        presdata[ts][6].reshape(z_grid, -1)[4],
        label="$p_{g, L 5}$",
    )
    ax[0].set_ylabel("Gas pressure, $p_g$ $[bara]$", labelpad=10)
    ax[0].set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    ax[0].set_xlim(xlim)
    # ax.set_ylim(ylim)
    ax[0].legend()
    ax[0].set_title(
        f"Gas Pressure and Saturation @ t = {round(sattime[ts]/3600,1)} hrs"
    )
    fig.savefig("plots/gas_saturation.png", dpi=1000, bbox_inches="tight")

    # fig.tight_layout()


#%%
@interact
def sprf_plot(
    ts=widgets.IntSlider(min=0, max=len(sattime) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(satdata[0][1])),
):
    fig, ax = plt.subplots(nrows=2, ncols=1)
    ax[1].pcolormesh(
        satdata[ts][1].reshape(z_grid, -1)[0],
        satdata[ts][3].reshape(z_grid, -1)[:, 0],
        satdata[ts][5].reshape(z_grid, -1),
        cmap="jet_r",
        shading="auto",
    )
    # ax.grid()
    ax[1].set_ylabel("Vertical distance, $z$ $[m]$", labelpad=10)
    ax[1].set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax[1].set_xlim(xlim)
    # ax[1].yaxis.set_label_position("right")
    # ax[1].set_title(
    #     "Gas Saturation Profile @ time = {} $hr$".format(round(sattime[ts] / 3600, 1))
    # )
    fig.colorbar(cm.ScalarMappable(norm=None, cmap="jet_r"), ax=ax, location="right")
    # fig.tight_layout(pad=0.2)

    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[0],
        presdata[ts][5].reshape(z_grid, -1)[0],
        label="$p_{o, L 1}$",
    )
    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[1],
        presdata[ts][5].reshape(z_grid, -1)[1],
        label="$p_{o, L 2}$",
    )
    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[2],
        presdata[ts][5].reshape(z_grid, -1)[2],
        label="$p_{o, L 3}$",
    )
    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[3],
        presdata[ts][5].reshape(z_grid, -1)[3],
        label="$p_{o, L 4}$",
    )
    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[4],
        presdata[ts][5].reshape(z_grid, -1)[4],
        label="$p_{o, L 5}$",
    )
    ax[0].set_ylabel("Oil pressure, $p_g$ $[bara]$", labelpad=10)
    ax[0].set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    ax[0].set_xlim(xlim)
    # ax.set_ylim(ylim)
    ax[0].legend()
    ax[0].set_title(
        f"Oil Pressure and Saturation @ t = {round(sattime[ts]/3600,1)} hrs"
    )
    fig.savefig("plots/oil_saturation.png", dpi=1000, bbox_inches="tight")

    # ax[0].set_xscale("log")
    # fig.tight_layout()


# %%
@interact
def sprf_plot(
    ts=widgets.IntSlider(min=0, max=len(sattime) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(satdata[0][1])),
):
    fig, ax = plt.subplots(nrows=2, ncols=1)
    ax[1].pcolormesh(
        satdata[ts][1].reshape(z_grid, -1)[0],
        satdata[ts][3].reshape(z_grid, -1)[:, 0],
        satdata[ts][4].reshape(z_grid, -1),
        cmap="jet_r",
        shading="auto",
    )
    # ax.grid()
    ax[1].set_ylabel("Vertical distance, $z$ $[m]$", labelpad=10)
    ax[1].set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    # ax.set_xticks(x.reshape(5,-1)[0])
    ax[1].set_xlim(xlim)
    # ax[1].yaxis.set_label_position("right")
    # ax[1].set_title(
    #     "Gas Saturation Profile @ time = {} $hr$".format(round(sattime[ts] / 3600, 1))
    # )
    fig.colorbar(cm.ScalarMappable(norm=None, cmap="jet_r"), ax=ax, location="right")
    # fig.tight_layout(pad=0.2)

    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[0],
        presdata[ts][4].reshape(z_grid, -1)[0],
        label="$p_{w, L 1}$",
    )
    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[1],
        presdata[ts][4].reshape(z_grid, -1)[1],
        label="$p_{w, L 2}$",
    )
    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[2],
        presdata[ts][4].reshape(z_grid, -1)[2],
        label="$p_{w, L 3}$",
    )
    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[3],
        presdata[ts][4].reshape(z_grid, -1)[3],
        label="$p_{w, L 4}$",
    )
    ax[0].plot(
        presdata[0][1].reshape(z_grid, -1)[4],
        presdata[ts][4].reshape(z_grid, -1)[4],
        label="$p_{w, L 5}$",
    )
    ax[0].set_ylabel("Water pressure, $p_g$ $[bara]$", labelpad=10)
    ax[0].set_xlabel("Radial distance, $x$ $[m]$", labelpad=10)
    ax[0].set_xlim(xlim)
    # ax.set_ylim(ylim)
    ax[0].legend()
    ax[0].set_title(
        f"Water Pressure and Saturation @ t = {round(sattime[ts]/3600,1)} hrs"
    )
    fig.savefig("plots/water_saturation.png", dpi=1000, bbox_inches="tight")
    # ax[0].set_xscale("log")
    # fig.tight_layout()


# %%

# %% boundary pressure in time
# boundarypressure1 = [presdata[i][6].reshape(z_grid, x_grid)[0][-1] for i in range(len(prestime))]
# boundarypressure2 = [presdata[i][6].reshape(z_grid, x_grid)[1][-1] for i in range(len(prestime))]
# boundarypressure3 = [presdata[i][6].reshape(z_grid, x_grid)[2][-1] for i in range(len(prestime))]
# boundarypressure4 = [presdata[i][6].reshape(z_grid, x_grid)[3][-1] for i in range(len(prestime))]
# boundarypressure5 = [presdata[i][6].reshape(z_grid, x_grid)[4][-1] for i in range(len(prestime))]

# plt.plot(prestime, boundarypressure1, label="l1")
# plt.plot(prestime, boundarypressure2, label="l2")
# plt.plot(prestime, boundarypressure3, label="l3")
# plt.plot(prestime, boundarypressure4, label="l4")
# plt.plot(prestime, boundarypressure5, label="l5")
# plt.legend()


BUBBLEPRESSURES = [ 199.001353 , 937.959170 , 3321.92557 , 9469.06541 , 23232.5490 , 31882.0898 , 34424.8443 , 35077.8055 , 35626.2641 , 36728.8606 , 61721.0939 , 100000.000 , 148609.795 , 243846.432 , 377435.423 , 506122.449 , 664406.463 , 912244.898 , 1073572.62 , 1318367.35 , 1628555.92 , 1724489.80 , 2095391.71 , 2130612.24 , 2536734.69 , 2927271.52 , 2942857.14 , 3348979.59 , 3755102.04 , 4161224.49 , 4510869.69 , 4567346.94 , 4973469.39 , 5379591.84 , 5785714.29 , 6191836.73 , 6597959.18 , 7004081.63 , 7410204.08 , 7816326.53 , 8222448.98 , 8614513.69 , 8628571.43 , 9034693.88 , \
9440816.33 , 9846938.78 , 10253061.2 , 10659183.7 , 11065306.1 , 11413330.8 , 11471428.6 , 11877551.0 , 12283673.5 , 12592972.5 , 12689795.9 , 13095918.4 , 13330602.2 , 13502040.8 , 13756822.9 , 13908163.3 , 14123149.8 , 14314285.7 , 14549914.5 , 14720408.2 , 14816032.4 , 14894987.9 , 14887474.4 , 14785425.6 ] 
BUBBLETEMPERATURES = [ -204.09962 , -196.41499 , -188.61989 , -180.70638 , -172.47975 , -169.17283 , -168.33459 , -168.12707 , -167.95509 , -167.61937 , -161.52631 , -155.14791 , -149.30548 , -141.08431 , -132.81098 , -126.61064 , -120.32029 , -112.24471 , -107.75012 , -101.71861 , -95.074520 , -93.197064 , -86.563625 , -85.979047 , -79.679363 , -73.940951 , -73.709886 , -67.737725 , -62.224533 , -57.061495 , -52.772576 , -52.091308 , -47.272499 , -42.571429 , -37.960077 , -33.414527 , -28.913905 , -24.439576 , -19.974282 , -15.501621 , -11.005824 , -6.6291341 , -6.4713592 , -1.8825547 , \
2.77681101 , 7.52395896 , 12.3776483 , 17.3588536 , 22.4916610 , 27.0322137 , 27.8045092 , 33.3325969 , 39.1539846 , 43.8711075 , 45.4140241 , 52.3107225 , 56.6270097 , 59.9526589 , 65.2059108 , 68.5709998 , 73.7833742 , 79.0081118 , 86.7088780 , 94.0059949 , 99.6928888 , 108.355169 , 117.056098 , 125.841157 ]
DEWPRESSURES = [ 100000.000 , 122079.497 , 148473.120 , 179577.333 , 216061.783 , 258682.152 , 308286.813 , 365830.828 , 432394.182 , 506122.449 , 509213.827 , 597730.552 , 699626.021 , 816028.345 , 912244.898 , 949815.414 , 1103735.24 , 1281142.12 , 1318367.35 , 1486217.25 , 1724274.94 , 1724489.80 , 2002291.28 , 2130612.24 , 2329768.03 , 2536734.69 , 2720274.11 , 2942857.14 , 3194514.50 , 3348979.59 , 3755102.04 , 3773121.26 , 4161224.49 , 4354884.84 , 4567346.94 , 4923910.76 , 4973469.39 , 5379591.84 , 5481710.26 , 5785714.29 , 6031073.28 , 6191836.73 , 6573931.41 , 6597959.18 , \
7004081.63 , 7111528.82 , 7410204.08 , 7644683.86 , 7816326.53 , 8173941.05 , 8222448.98 , 8628571.43 , 8699652.23 , 9034693.88 , 9222018.93 , 9440816.33 , 9698242.30 , 9846938.78 , 10171933.8 , 10253061.2 , 10642885.0 , 10659183.7 , 11065306.1 , 11110588.8 , 11471428.6 , 11574363.2 , 11877551.0 , 12033266.3 , 12283673.5 , 12485717.9 , 12689795.9 , 12930293.2 , 13095918.4 , 13310739.0 , 13502040.8 , 13635185.6 , 13908163.3 , 13912032.1 , 14147148.0 , 14314285.7 , 14344630.0 , 14507971.3 , 14640041.4 , 14695321.2 , 14711292.0 ]  
DEWTEMPERATURES = [ 85.3030988 , 89.8903438 , 94.5219971 , 99.1503713 , 103.774177 , 108.393027 , 113.006509 , 117.614117 , 122.214765 , 126.634231 , 126.806780 , 131.389187 , 135.961117 , 140.490943 , 143.802048 , 145.006181 , 149.505070 , 153.984951 , 154.845920 , 158.442613 , 162.872990 , 162.876685 , 167.268709 , 169.067929 , 171.618691 , 173.997565 , 175.904880 , 177.993821 , 180.095031 , 181.262321 , 183.941332 , 184.048189 , 186.128045 , 187.018758 , 187.893214 , 189.137588 , 189.289696 , 190.357716 , 190.578413 , 191.128241 , 191.459913 , 191.625222 , 191.859633 , 191.867116 , \
191.867938 , 191.829205 , 191.637974 , 191.402995 , 191.184275 , 190.602855 , 190.510951 , 189.619328 , 189.440761 , 188.507957 , 187.920192 , 187.172495 , 186.206677 , 185.605409 , 184.177135 , 183.795498 , 181.815354 , 181.727121 , 179.379037 , 179.098730 , 176.722613 , 175.995993 , 173.719009 , 172.464732 , 170.316067 , 168.452777 , 166.439637 , 163.876135 , 161.973923 , 159.313800 , 156.730299 , 154.791191 , 150.363855 , 150.295950 , 145.818063 , 142.098499 , 141.359682 , 136.919136 , 132.493454 , 130.277974 , 129.581550 ] 
plt.plot(DEWTEMPERATURES, np.array(DEWPRESSURES)*1e-5)
plt.plot(BUBBLETEMPERATURES, np.array(BUBBLEPRESSURES)*1e-5)
plt.xlabel("Temperature, $T$ $[^oC]$")
plt.ylabel("Pressure, $p$ $[bara]$")
plt.title("Phase Envelope of Gas Condensate")