#%%
import numpy as np
from libraries import gas
import pandas as pd
from scipy import interpolate
import os
import matplotlib.pyplot as plt
import mpl_axes_aligner
import ipywidgets.widgets as widgets
from ipywidgets import interact
import warnings
import pyfas as fa
from mpl_toolkits.mplot3d import Axes3D
import itertools as it
import matplotlib.cm as cm
import fluids

sim_folder = os.path.abspath("D:\Github\Liquid-Loading-Analysis-Tool\OLGA") + "/"

def filename(file_type):
    file = []
    for files in os.listdir(sim_folder):  # "..\eclipse-plot"
        if files.endswith(file_type):
            file.append(files)
        else:
            continue
    return file


def getTPLData(sim_folder, f_name):
    tplpath = sim_folder
    fname = f_name
    tpl = fa.Tpl(tplpath + fname)
    # fielter_rend = tpl.filter_trends(trend)
    # view_trend = tpl.view_trends(trend)
    # tpl.label # find number to extract in tpl.extract()
    # keys =tpl.data.keys # find number to extract in tpl.extract()
    tpl.extract(0)
    tpl.extract(1)
    tpl.extract(2)
    tpl.extract(3)
    tpl.extract(4)
    tpl.extract(5)

    time = tpl.time
    LIQFR = tpl.data[1]
    QGST = tpl.data[2]
    QWST = tpl.data[3]
    QOST = tpl.data[4]
    PT = tpl.data[5]

    return np.array(time), LIQFR, QGST, QWST, QOST, PT


time, LIQFR, QGST, QWST, QOST, PT = getTPLData(sim_folder, filename("tpl")[0])


def getPPLData(sim_folder, f_name):
    # hard coded indexes. check ppl.filter_data("1") etc.
    # ppldata geometry data[index][0]
    # ppldata timeseries data[index][1][timestep]
    pplpath = sim_folder
    fname = f_name
    ppl = fa.Ppl(pplpath + fname)

    ppl.time
    ppl.extract(1)
    ppl.extract(2)
    ppl.extract(3)
    ppl.extract(4)
    ppl.extract(5)
    ppl.extract(6)
    ppl.extract(7)
    ppl.extract(8)
    ppl.extract(9)
    ppl.extract(10)
    ppl.extract(11)
    ppl.extract(14)
    ppl.extract(15)

    time = ppl.time
    ID = ppl.data[1]
    UD = ppl.data[2]
    UG = ppl.data[3]
    UL = ppl.data[4]
    USD = ppl.data[5]
    USG = ppl.data[6]
    USL = ppl.data[7]
    DPZA = ppl.data[8]
    DPZF = ppl.data[9]
    DPZG = ppl.data[10]
    HOL = ppl.data[11]
    PT = ppl.data[14]
    TM = ppl.data[15]

    return np.array(time), ID, UD, UG, UL, USD, USG, USL, DPZA, DPZF, DPZG, HOL, PT, TM


(
    time,
    ID,
    UD,
    UG,
    UL,
    USD,
    USG,
    USL,
    DPZA,
    DPZF,
    DPZG,
    HOL,
    PT_branch,
    TM_branch,
) = getPPLData(sim_folder, filename("ppl")[0])


def getTABdata(simfolder, f_name):
    tabpath = simfolder
    fname = f_name
    tab = fa.Tab(tabpath + fname)
    tab.export_all()
    # p_array = tab.metadata["p_array"]
    # t_array = tab.metadata["t_array"]
    pressure = tab.data.T["PT"].values[0]  # Pa
    temperature = tab.data.T["TM"].values[0]  # C

    try:
        ROG = np.array(tab.data.T["ROG"].values[0])  # dens gas, kg/m3
        ROHL = np.array(tab.data.T["ROHL"].values[0])  # dens oil (liquid), kg/m3
        ROWT = np.array(tab.data.T["ROWT"].values[0])
        RS = np.array(tab.data.T["RS"].values[0])  # gas oil ratio, -
        RSW = np.array(tab.data.T["RSW"].values[0])
        SIGGHL = np.array(tab.data.T["SIGGHL"].values[0])  # IFT gas oil, N/m
        SIGGWT = np.array(tab.data.T["SIGGWT"].values[0])
        SIGGHLWT = np.array(tab.data.T["SIGGHLWT"].values[0])
        VISG = np.array(tab.data.T["VISG"].values[0])  # visc gas, CP
        VISHL = np.array(tab.data.T["VISHL"].values[0])  # visc oil, CP
        VISWT = np.array(tab.data.T["VISWT"].values[0])
        return (
            pressure,
            temperature,
            ROG,
            ROHL,
            ROWT,
            RS,
            RSW,
            SIGGHL,
            SIGGWT,
            SIGGHLWT,
            VISG,
            VISHL,
            VISWT,
        )

    except:
        try:
            ROG = np.array(tab.data.T["ROG"].values[0])  # dens gas, kg/m3
            ROHL = np.array(tab.data.T["ROHL"].values[0])  # dens oil (liquid), kg/m3
            RS = np.array(tab.data.T["RS"].values[0])  # gas oil ratio, -
            SIGGHL = np.array(tab.data.T["SIGGHL"].values[0])  # IFT gas oil, N/m
            VISG = np.array(tab.data.T["VISG"].values[0])  # visc gas, CP
            VISHL = np.array(tab.data.T["VISHL"].values[0])  # visc oil, CP
            return pressure, temperature, ROG, ROHL, RS, SIGGHL, VISG, VISHL

        except:
            ROG = np.array(tab.data.T["ROG"].values[0])  # dens gas, kg/m3
            ROWT = np.array(tab.data.T["ROWT"].values[0])
            RSW = np.array(tab.data.T["RSW"].values[0])
            SIGGWT = np.array(tab.data.T["SIGGWT"].values[0])
            VISG = np.array(tab.data.T["VISG"].values[0])  # visc gas, CP
            VISWT = np.array(tab.data.T["VISWT"].values[0])
            return pressure, temperature, ROG, ROWT, RSW, SIGGWT, VISG, VISWT


try:
    (
        pressure,
        temperature,
        ROG,
        ROHL,
        ROWT,
        RS,
        RSW,
        SIGGHL,
        SIGGWT,
        SIGGHLWT,
        VISG,
        VISHL,
        VISWT,
    ) = getTABdata(sim_folder, filename("tab")[0])
except:
    try:
        pressure, temperature, ROG, ROHL, RS, SIGGHL, VISG, VISHL = getTABdata(
            sim_folder, filename("tab")[0]
        )

    except:
        pressure, temperature, ROG, ROWT, RSW, SIGGWT, VISG, VISWT = getTABdata(
            sim_folder, filename("tab")[0]
        )


def tab_interp(
    p_input, T_input, property, pressure_array=pressure, temperature_array=temperature
):
    x = np.unique(pressure_array) / 1e5
    y = np.unique(temperature_array)
    z = property
    f = interpolate.interp2d(x, y, z, kind="linear")
    return f(p_input, T_input)



def find_nearest(array, value):
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    return idx, array[idx]


def find_slug(array, value):
    array = np.asarray(array)
    idx = int((np.abs(array - value)).argmin() / len(ID[1][1]))
    return idx, array[idx]



def vc(liquid_density, gas_density, surface_tension, model="Turner"):
    # metric units, critical velocity returned in m/s
    model = str.upper(model)
    if model == "TURNER":
        k = 6.558  # turner value
    else:
        k = 5.465  # Coleman value

    rhol = liquid_density
    rhog = gas_density

    return k * (surface_tension * (rhol - rhog) / rhog ** 2) ** (0.25)


def Qc(p1, p2, T1, T2, d, unit=2, model="Turner"):

    A = np.pi / 4 * (d) ** 2  # m2
    Yg = tab_interp(1.01325, 20, ROG)[0] / 1.205
    Z1 = np.array([gas.ZfacStanding(p1[x], T1[x], Yg, 2) for x in range(len(p1))])
    Z2 = gas.ZfacStanding(p2, T2, Yg, 2)
    # Rs = tab_interp(p1, T1, RS)
    rho_g = np.array(
        [tab_interp(p1[x], T1[x], ROG)[0] for x in range(len(p1))]
    )  # kg/m3
    try:
        rho_l = np.array(
            [tab_interp(p1[x], T1[x], ROWT)[0] for x in range(len(p1))]
        )  # kg/m3
        sigma = np.array(
            [tab_interp(p1[x], T1[x], SIGGWT)[0] for x in range(len(p1))]
        )  # N/m

        v_c = vc(rho_l, rho_g, sigma, model)  # m/s
    except:
        rho_l = np.array(
            [tab_interp(p1[x], T1[x], ROHL)[0] for x in range(len(p1))]
        )  # kg/m3
        sigma = np.array(
            [tab_interp(p1[x], T1[x], SIGGHL)[0] for x in range(len(p1))]
        )  # N/m

        v_c = vc(rho_l, rho_g, sigma, model)  # m/s

    Qc = [
        (
            (p1[x] * v_c[x] * A * Z2 * (T2 + 273.15))
            / (Z1[x] * p2 * (T1[x] + 273.15))
            * 86400
        )
        for x in range(len(p1))
    ]  # Sm3/d
    return Qc, v_c

#%%
try:
    rate_gradient_search_idx = (
        # np.where(abs(np.gradient(QGST[50:])) > 0.00001)[0][0] + 50
        np.where(abs(np.gradient(QGST[50:], time[50:])) > 0.00001)[0][0]+50
    )
    rate_gradient_search_rate = QGST[rate_gradient_search_idx]
    rate_gradient_search_pressure = PT[rate_gradient_search_idx]
    print(
        f"Rate gradient search index={rate_gradient_search_idx}, rate={rate_gradient_search_rate*86400}, pressure={rate_gradient_search_pressure/1e5}, time={time[rate_gradient_search_idx]/3600}"
    )
except:
    rate_gradient_search_idx = 0
    rate_gradient_search_rate = QGST[rate_gradient_search_idx] * 86400
    rate_gradient_search_pressure = PT[rate_gradient_search_idx] * 1e-5
    print("dq/dt < 0 does not exist")

slug_identifier_idx = find_slug(ID[1], 3)[0]
slug_identifier_rate = QGST[slug_identifier_idx]
slug_identifier_pressure = PT[slug_identifier_idx]
if slug_identifier_idx == 0:
    print("Slug does not exist")
else:
    print(
        f"Slug identifier index={slug_identifier_idx}, rate={slug_identifier_rate*86400}, pressure={slug_identifier_pressure/1e5}, time={time[slug_identifier_idx]/3600}"
    )

try:
    liquidfilm_reversal_idx = np.where(np.asarray(UL[1]) < 0)[0][0]
    liquidfilm_reversal_rate = QGST[liquidfilm_reversal_idx]
    liquidfilm_reversal_pressure = PT[liquidfilm_reversal_idx]
    print(
        f"Liquid film reversal index={liquidfilm_reversal_idx}, rate={liquidfilm_reversal_rate*86400}, pressure={liquidfilm_reversal_pressure/1e5}, time={time[liquidfilm_reversal_idx]/3600}"
    )
    cr1 = Qc(
        PT_branch[1][liquidfilm_reversal_idx - 10] * 1e-5,
        1.01325,
        TM_branch[1][liquidfilm_reversal_idx - 10],
        15.556,
        0.05715,
    )[0]
    cr2 = Qc(
        PT_branch[1][liquidfilm_reversal_idx + 10] * 1e-5,
        1.01325,
        TM_branch[1][liquidfilm_reversal_idx + 10],
        15.556,
        0.05715,
    )[0]
    print(
        f"Turner critical rate pre/post liquid film reversal = {np.max(cr1), np.max(cr2)}"
    )
except:
    print("UL > 0")
    liquidfilm_reversal_idx = 0

try:
    droplet_reversal_idx = np.where(np.asarray(UD[1]) < 0)[0][0]
    droplet_reversal_rate = QGST[droplet_reversal_idx]
    droplet_reversal_pressure = PT[droplet_reversal_idx]
    print(
        f"Droplet reversal index={droplet_reversal_idx}, rate={droplet_reversal_rate*86400}, pressure={droplet_reversal_pressure/1e5}, time={time[droplet_reversal_idx]/3600}"
    )
    cr1 = Qc(
        PT_branch[1][droplet_reversal_idx - 10] * 1e-5,
        1.01325,
        TM_branch[1][droplet_reversal_idx - 10],
        15.556,
        0.05715,
    )[0]
    cr2 = Qc(
        PT_branch[1][droplet_reversal_idx + 10] * 1e-5,
        1.01325,
        TM_branch[1][droplet_reversal_idx + 10],
        15.556,
        0.05715,
    )[0]
    print(
        f"Turner critical rate pre/post droplet reversal = {np.max(cr1), np.max(cr2)}"
    )

except:
    droplet_reversal_idx = 0
    print("UL > 0")


try:
    oil_backflow_index = len(QoL5)
    for L in (QoL5, QoL4, QoL3, QoL2, QoL1):
        idx = np.where(L < 0)[0][0]
        if idx < oil_backflow_index:
            oil_backflow_index = idx
    if oil_backflow_index != len(QoL5):
        print(
            f"Oil backflow index = {oil_backflow_index}, time={time3[oil_backflow_index]/3600} hr"
        )
except:
    print("Oil does not flow back")

try:
    water_backflow_index = len(QwL5)
    for L in (QwL5, QwL4, QwL3, QwL2, QwL1):
        idx = np.where(L < 0)[0][0]
        if idx < water_backflow_index:
            water_backflow_index = idx
    if water_backflow_index != len(QwL5):
        print(
            f"Water backflow index = {water_backflow_index}, time={time3[water_backflow_index]/3600} hr"
        )
except:
    print("Water does not flow back")

try:
    gas_backflow_index = len(QgL5)
    for L in (QgL5, QgL4, QgL3, QgL2, QgL1):
        idx = np.where(L < 0)[0][0]
        if idx < gas_backflow_index:
            gas_backflow_index = idx
    if gas_backflow_index != len(QgL5):
        print(
            f"Gas backflow index = {gas_backflow_index}, time={time3[gas_backflow_index]/3600} hr"
        )
except:
    print("Gas does not flow back")
# %%
fsize = 16
tsize = 18
tdir = "out"  # "in" "out" "inout"
major = 5
minor = 3
style = "default"  # plt.style.available
xsize = 8
ysize = 5

plt.style.use(style)
plt.rcParams["text.usetex"] = True
plt.rcParams["font.size"] = 12
plt.rcParams["legend.fontsize"] = 12
plt.rcParams["legend.title_fontsize"] = 12
plt.rcParams["legend.loc"] = "best"
plt.rcParams["xtick.direction"] = tdir
plt.rcParams["ytick.direction"] = tdir
plt.rcParams["xtick.major.size"] = major
plt.rcParams["xtick.minor.size"] = minor
plt.rcParams["ytick.major.size"] = major
plt.rcParams["ytick.minor.size"] = minor


timesteps = len(time)


@interact
def ppl_plot(
    ts=widgets.IntSlider(min=0, max=timesteps - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(ID[0])),
):
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    # ax0.grid(False)
    (p0,) = ax0.plot(ID[0], ID[1][ts], label="Flow regime")
    ax0.set_ylabel("Flow regime identification", fontsize=fsize)
    ax0.set_xlabel("Distance along tubing, $z$ $[m]$", fontsize=fsize)
    ax0.set_ylim(1, 4)
    ax0.set_yticks([1, 2, 3, 4])
    ax0.set_yticklabels(["Stratified", "Annular", "Slug", "Bubbly"])

    ax1 = ax0.twinx()
    # ax1.set_ylim(90, 130)
    (p1,) = ax1.plot(HOL[0], HOL[1][ts], "r", label="$H_L$")
    ax1.grid()
    ax1.set_ylabel("Liquid holdup, $H_L$ $[-]$", fontsize=fsize)
    ax1.tick_params(axis="both", labelsize=fsize)
    ax0.set_xlim(xlim)
    fig.legend()
    ax0.set_title(
        "Flow Regime and Liquid Holdup @ time = {} hr".format(round(time[ts] / 3600))
    )
    fig.savefig("plots/flowpattern.png", dpi=1000)

#%%


@interact
def ppl_plot(
    ts=widgets.IntSlider(min=0, max=timesteps - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(ID[0])),
):
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    ax0.grid(True)
    (p0,) = ax0.plot(PT_branch[0], PT_branch[1][ts] / 1e5, label="$p$")
    ax0.set_ylabel("Pressure, $p$ $[bara]$", fontsize=fsize)
    ax0.set_xlabel("Distance along tubing, $z$ $[m]$", fontsize=fsize)
    ax0.set_ylim(np.min(PT_branch[1]) * 1e-5 - 5, np.max(PT_branch[1]) * 1e-5 + 5)
    # ax0.set_ylim(1, 4)
    # ax0.set_yticks([1, 2, 3, 4])
    # ax0.set_yticklabels(["Stratified", "Annular", "Slug", "Bubbly"])

    ax1 = ax0.twinx()
    # ax1.set_ylim(90, 130)
    (p1,) = ax1.plot(TM_branch[0], TM_branch[1][ts], "r", label="$T$")
    # ax1.grid(False)
    ax1.set_ylabel("Temperature, $T$ $[^oC]$", fontsize=fsize)
    ax1.set_ylim(np.min(TM_branch[1]) - 5, np.max(TM_branch[1]) + 5)
    ax1.tick_params(axis="both", labelsize=fsize)
    ax0.set_xlim(xlim)
    ax0.set_title(
        "Pressure and Temperature @ time = {} hr".format(round(time[ts] / 3600))
    )
    fig.legend()


@interact
def ppl_plot(
    ts=widgets.IntSlider(min=0, max=timesteps - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(ID[0])),
):
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    ax0.grid(True)
    (p0,) = ax0.plot(UG[0], UG[1][ts], "r-", label="$u_{g}$")
    (p0,) = ax0.plot(UD[0], UD[1][ts], "g-", label="$u_{d}$")
    (p0,) = ax0.plot(UL[0], UL[1][ts], "b-", label="$u_{l}$")
    ax0.set_ylabel("Velocity, $u$ $[m/s]$", fontsize=fsize)
    ax0.set_xlabel("Distance along tubing , $z$ $[m]$", fontsize=fsize)
    ax0.set_title(
        "Velocity of gas, liquid film, and droplets @ time = {} hr".format(
            round(time[ts] / 3600)
        )
    )
    ax0.set_xlim(xlim)
    fig.legend()
    # ax0.set_ylim(1, 4)
    # ax0.set_yticks([1, 2, 3, 4])
    # ax0.set_yticklabels(["Stratified", "Annular", "Slug", "Bubbly"])


# OLGA Trend data plot
@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time) / 3600),
             ylim=widgets.IntRangeSlider(min=np.min(QGST)*86400, max = np.max(QGST)*86400),
             ylim1=widgets.FloatRangeSlider(min=np.min(QOST)*86400, max = np.max(QOST)*86400)
             ):
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    (p0,) = ax0.plot(time / 3600, QGST * 86400, "r", label="$q_{g}$")
    ax0.set_ylabel("Gas flow rate, $q_{g}$ $[Sm^3/d]$ ", fontsize=fsize)
    ax0.set_xlabel("Time, $t$ $[hr]$", fontsize=fsize)

    ax1 = ax0.twinx()
    (p1,) = ax1.plot(time / 3600, QOST * 86400, "g", label="$q_{o}$")
    # ax1.grid()
    ax1.set_ylabel("Oil flow rate, $q_{o}$ $[Sm^3/d]$ ", fontsize=16)
    ax1.tick_params(axis="both", labelsize=16)
    ax0.set_xlim(xlim)
    ax0.set_ylim(ylim)
    ax1.set_ylim(ylim1)
    fig.legend()
    ax0.set_title("Surface Flow Rates of Oil and Gas")
    fig.savefig("plots/surfacerate.png", dpi=1000)

@interact
def ppl_plot(xlim=widgets.IntRangeSlider(min=0, max=max(time) / 3600)):
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    (p0,) = ax0.plot(time / 3600, PT * 1e-5, "k", label="$p_{wf}$")
    ax0.set_ylabel("Bottomhole pressure, $p_{wf}$ $[bara]$ ", fontsize=fsize)
    ax0.set_xlabel("Time, $t$ $[hr]$", fontsize=fsize)

    ax1 = ax0.twinx()
    (p1,) = ax1.plot(time / 3600, LIQFR, "b", label="$\phi_L$")
    # ax1.grid()
    ax1.set_ylabel("Liquid fraction, $\phi_L$  $[-]$ ", fontsize=16)
    ax1.tick_params(axis="both", labelsize=16)
    ax0.set_xlim(xlim)
    fig.legend()
    ax0.set_title("Bottomhole Pressure and Liquid Fraction")


#%% critical velocity
@interact
def ppl_plot(
    ts=widgets.IntSlider(min=0, max=len(time) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(ID[0])),
    ylim=widgets.FloatRangeSlider(
        min=np.min([UG[1], UD[1], UL[1]]), max=np.max([UG[1], UD[1], UL[1]])
    ),
):
    critical_rate = Qc(
        PT_branch[1][ts] * 1e-5, 1.01325, TM_branch[1][ts], 15.556, 0.05715
    )[1]
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    # ax0.set_ylim(1, 4)
    # ax0.invert_yaxis()
    # ax0.set_yticklabels(["Stratified", "Annular", "Slug", "Bubbly"])
    (p0,) = ax0.plot(PT_branch[0], critical_rate, "k", label="$u_{c, Turner}$")
    (p1,) = ax0.plot(UG[0], UG[1][ts], "r-", label="$u_{g}$")
    (p2,) = ax0.plot(UD[0], UD[1][ts], "g-", label="$u_{d}$")
    (p3,) = ax0.plot(UL[0], UL[1][ts], "b-", label="$u_{l}$")
    ax0.set_title(f"Phase Velocity and Turner Critical Velocity @ t = {round(time[ts]/3600,1)} hr")
    ax0.set_ylabel("Velocity, $u$ $[m/s]$", fontsize=fsize)
    ax0.set_xlabel("Distance along tubing , $z$ $[m]$", fontsize=fsize)
    ax0.set_ylim(ylim)
    ax0.set_xlim(xlim)
    fig.legend()
    fig.savefig("plots/phasevelocity.png", dpi=800)

#
@interact
def ppl_plot(
    ts=widgets.IntSlider(min=0, max=len(time) - 1),
    xlim=widgets.IntRangeSlider(min=0, max=np.max(ID[0])),
    ylim=widgets.FloatRangeSlider(
        min=np.min([UG[1], UD[1], UL[1]]), max=np.max([UG[1], UD[1], UL[1]])
    ),
):
    critical_rate = Qc(
        PT_branch[1][ts] * 1e-5, 1.01325, TM_branch[1][ts], 15.556, 0.05715
    )[1]
    fig, ax0 = plt.subplots(figsize=(xsize, ysize))
    # ax0.set_ylim(1, 4)
    # ax0.invert_yaxis()
    # ax0.set_yticklabels(["Stratified", "Annular", "Slug", "Bubbly"])
    # (p0,) = ax0.plot(PT_branch[0], critical_rate, "k", label="$u_{c, Turner}$")
    (p1,) = ax0.plot(USG[0], USG[1][ts], "r-", label="$u_{sg}$")
    (p2,) = ax0.plot(USD[0], USD[1][ts], "g-", label="$u_{sd}$")
    (p3,) = ax0.plot(USL[0], USL[1][ts], "b-", label="$u_{sl}$")
    ax0.set_ylabel("Superficial velocity, $u_s$ $[m/s]$", fontsize=fsize)
    ax0.set_xlabel("Distance along tubing , $z$ $[m]$", fontsize=fsize)
    ax0.set_title("Superficial Phase Velocity")
    ax0.set_ylim(ylim)
    ax0.set_xlim(xlim)
    fig.legend()

# %%
