#%%
import numpy as np
import matplotlib.pyplot as plt
import datapreprocessing as dpp
import gas


T_R = 50  # degC
P_R = 40  # bara

sD = 1  # 1/sm3/s
k = 100  # md
h = 100  # m
Z = 1
rw = 2.25 / 2  # in
re = 1500  # m
s = 0
n = 1

SGg = 0.61


#%% IPR backpressure equation


def C(k, h, T, vis, Z, sD, re, rw, s, n=1):
    hf = h / 0.3048
    Tf = (T + 273.15) * 1.8
    ref = re / 0.3048
    rwf = rw / 12

    c = (0.703 * k * hf) ** n / (
        (Tf * vis * Z) ** n
        * sD ** (1 - n)
        * (np.log(ref / rwf) - 0.75 + s) ** (2 * n - 1)
    )
    return c * 14.5**(2*n)/35.3146667


def backpressure(P_R, P_wf, C, n=1):
    q = C * (P_R ** 2 - P_wf ** 2) ** n
    return q


#%% TPR from sim

tpl_files = []
ppl_files = []
PT_list = []
QGST_list = []

path_of_directory = str(input("Type in the name of the simulation data folder:"))
if path_of_directory == "":
    path_of_directory = "..\OLGA"
ext = ".tpl"
for files in os.listdir(path_of_directory):
    if files.endswith(".tpl"):
        tpl_files.append(files)
    elif files.endswith(".ppl"):
        ppl_files.append(files)
    else:
        continue

    def find_nearest(array, value):
        array = np.asarray(array)
        idx = (np.abs(array - value)).argmin()
        return idx, array[idx]


P_wf = np.linspace(0, P_R, 100)
# c = C(k, h, T_R, 0.011, 1, sD, re, rw, s, n)
c = 10
QgIPR = [backpressure(P_R, i, c, n) for i in P_wf]

plt.plot(QgIPR, P_wf, "r", label="IPR")

for i in tpl_files:
    time, LIQC, QGST, QWST, PT = dpp.getTPLData(path_of_directory + "/", i)

    PT_list.append(PT*1e-5)
    QGST_list.append(QGST*86400)
    QGST = 86400*QGST
    PT = 1e-5*PT

    # Zg = [gas.ZfacStanding(i * 1e-5, T_R, SGg, 2) for i in PT]
    # deng = [gas.deng(p * 1e-5, T_R, Zg[i], SGg, 2) for i, p in enumerate(PT)]
    # visg = [gas.visg(SGg, deng[i], T_R) for i, p in enumerate(PT)]
    idx = find_nearest(QGST[100:], 0)[0]
    qg0 = find_nearest(QGST[100:], max(QgIPR) )[0]

    plt.plot(QGST[qg0: idx], PT[qg0: idx], label=f"TPR {i[0:2]}")
# plt.xlim(0,max(QgIPR))
# plt.ylim(0,max(P_wf))
plt.grid()
plt.legend()


# %%
