#%%
import numpy as np
import pandas as pd
from openpyxl import load_workbook
import math
import functions
from sklearn import preprocessing
import scipy.optimize as so

#%%
R = 8.31446261815324e-6
"""
Feed and separator definition
"""
# pressure_stages = [500 / 14.5]  # set separator pressure stages, bara
# temperature_stages = [(280 - 32) / 1.8]  # set separator temperature stages, deg C
# nmoles_inlet = [1]  # set number of moles in inlet

"""
Composition and properties
"""
# Component = ["C1", "C4", "C10"]  # Name
# Fraction = np.array([0.5, 0.42, 0.08])  # -
# MolecularWeight = np.array([16.04, 58.12, 142.29])  # g/mol
# S = np.array([-0.1595, -0.0675, 0.0655])
# Tc = np.array([343, 765.3, 1111.8]) * 5 / 9  # R -> K
# pc = np.array([667.8, 550.7, 304]) / 14.503773773  # psia -> bara
# AF = np.array([0.0115, 0.1928, 0.4902])  # -
# # BIP = np.array([[0, 0.014, 0.04361], [0.014, 0, 0], [0.04361, 0, 0]])
# BIP = np.array([[0 for col in range(len(Component))] for row in range(len(Component))])
# Volume_shift = True  # or False


sheetname = str(input("Use properties from 'Input' sheet? (y/n)"))
if sheetname == "":
    sheetname = "Input"
elif sheetname == "y":
    sheetname = "Input"
elif sheetname == "n":
    sheetname = "Manual input"

Volume_shift = str(input("Include volume shift? (y/n)"))
if Volume_shift == "":
    Volume_shift = "y"

inputdata = pd.read_excel("input.xlsx", usecols=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10], sheet_name=sheetname)
S = np.array(inputdata["S"][abs(inputdata["S"]) > 0])
Component = np.array(inputdata["Component"][0 : len(S)])
MolecularWeight = np.array(inputdata["MW"][inputdata["MW"] > 0])
Fraction = np.array(inputdata["Zi"][inputdata["Zi"] > 0])
Tc = np.array(inputdata["Tc"][inputdata["Tc"] > 0])
pc = np.array(inputdata["pc"][inputdata["pc"] > 0])
AF = np.array(inputdata["AF"][inputdata["AF"] > 0])
pressure_stages = np.array(inputdata["P"][inputdata["P"] > 0])
temperature_stages = np.array(inputdata["T"][inputdata["T"] > 0])
BIP = np.array([[0 for col in range(len(Component))] for row in range(len(Component))])
print(inputdata)

"""
Convergence toleranse 
"""
SSQ_tol = 1e-15  # NR for fug


z0 = Fraction

fugacity_loop = []
EoS_loop = []
PVT_loop = []

if Volume_shift != "y":
    S = np.array([0] * len(Component))
#%%
"""
if p and T have several stages, loop through the solver by updating p and T value
"""
feed = Fraction
for idx, pressure in enumerate(pressure_stages):
    T = temperature_stages[idx]
    p = pressure
    print("-----------------------------------------------------------")
    print(f"Pressure = {round(p)} bara \nTemperature = {round(T)} degC")
    print(f"Volume shift (y/n) = {Volume_shift} ")
    # next feed is calculated x
    # etc. ...

    p_sep = p / 10  # bara -> MPa
    T_sep = T + 273.15  # deg C -> K

    """
    EoS calculation start
    """

    def EoS(feed, p, T):
        b = np.array(functions.b(R, Tc, pc / 10))
        B = np.array(b * p_sep / (R * T_sep))
        m = np.array([functions.m(i) for i in AF])
        alpha = np.array(functions.alpha(m, T_sep / Tc))
        a = np.array(functions.a(R, Tc, pc / 10, alpha))
        A = np.array(a * p_sep / (R * T_sep) ** 2)
        c = np.array(S * b)
        k = np.array(range(len(feed)))

        Amix = np.array(functions.A_mix(A, BIP, feed))
        Bmix = np.array(functions.B_mix(B, feed))
        a1 = -(1 - Bmix)
        a2 = Amix - 3 * Bmix ** 2 - 2 * Bmix
        a3 = -(Amix * Bmix - Bmix ** 2 - Bmix ** 3)
        Q = (a1 ** 2 - 3 * a2) / 9
        R_ = (2 * a1 ** 3 - 9 * a1 * a2 + 27 * a3) / 54
        M = R_ ** 2 - Q ** 3

        MW = np.array(np.sum(feed * MolecularWeight))
        Awi = np.array(functions.Awi(A, BIP, feed, k))

        if M < 0:
            theta = math.acos(R_ / (Q ** (3 / 2)))
            Z1 = -2 * np.sqrt(Q) * math.cos(theta / 3) - a1 / 3
            Z2 = -2 * np.sqrt(Q) * math.cos((theta + 2 * np.pi) / 3) - a1 / 3
            Z3 = -2 * np.sqrt(Q) * math.cos((theta - 2 * np.pi) / 3) - a1 / 3

            fugacity_PR = [
                functions.fugacity(p_sep, R, T_sep, i, Amix, Bmix, B, feed, Awi, c)
                for i in [Z1, Z2, Z3]
            ]

            # find Z with lowest gibbs energy
            V = np.array([i * T_sep * R * 1000 / p_sep for i in [Z1, Z2, Z3]])
            V_corrected = V - np.sum(c * feed) * 1000
            V = V_corrected / MW
            gibbs_energy = [[functions.Gibbs_energy(i, feed) for i in fugacity_PR]]
            minind = np.where(
                preprocessing.normalize(gibbs_energy)
                == np.amin(preprocessing.normalize(gibbs_energy))
            )[0][0]
            den = 1 / V[minind]

            Z = [Z1, Z2, Z3][minind]
            fugacity_PR = functions.fugacity(
                p_sep, R, T_sep, Z, Amix, Bmix, B, feed, Awi, c
            )

        else:
            S_ = -np.sign(R_) * ((abs(R_) + (M ** 0.5)) ** (1 / 3))
            if Q == 0:
                T_ = 0
            else:
                T_ = Q / S_
            Z = S_ + T_ - a1 / 3
            fugacity_PR = functions.fugacity(
                p_sep, R, T_sep, Z, Amix, Bmix, B, feed, Awi, c
            )
            V = Z * T_sep * R * 1000 / p_sep
            V_corrected = V - np.sum(c * feed) * 1000
            V = V_corrected / MW
            den = 1 / V

        return (
            b,
            B,
            m,
            alpha,
            a,
            A,
            c,
            k,
            Amix,
            Bmix,
            a1,
            a2,
            a3,
            Q,
            R_,
            M,
            MW,
            den,
            Awi,
            fugacity_PR,
            Z,
        )

    """
    Eos calculation end
    """

    """
    Flash calculation start
    """

    eos = EoS(feed, p, T)
    EoS_loop.append(eos)

    """
    Initial K, Fv, xi, yi calculation
    """
    try:
        Ki = functions.Ki(p, T, Tc, pc, AF)
        h = lambda Fv: np.sum((feed * (Ki - 1)) / (1 + Fv * (Ki - 1)))
        dh = lambda Fv: -np.sum((feed * (Ki - 1) ** 2) / (1 + Fv * (Ki - 1)) ** 2)
        Fv = so.newton(h, 0.5, dh)
        if Fv > 1 / (1 - min(Ki)) or Fv < 1 / (1 - max(Ki)):
            Fv = so.bisect(h, 1 / (1 - max(Ki)), 1 / (1 - min(Ki)))
    except:
        Ki = functions.Ki(p, T, Tc, pc, AF)
        Fv = so.bisect(h, 1 / (1 - max(Ki)), 1 / (1 - min(Ki)))

    xi = functions.xi(feed, Fv, Ki)
    yi = Ki * xi

    eosx = EoS(xi, p, T)
    eosy = EoS(yi, p, T)

    Ki = yi / xi
    fug_x = eosx[-2]
    fug_y = eosy[-2]
    MWl = eosx[-5]
    MWv = eosy[-5]
    denl = eosx[-4]
    denv = eosy[-4]
    Zl = eosx[-1]
    Zv = eosy[-1]
    SSQ = functions.SSQ_fugacity(fug_y, fug_x)

    fugacity_loop.append(
        [Component, p, T, yi, xi, Ki, fug_y, fug_x, fug_x / fug_y, SSQ]
    )
    PVT_loop.append([Component, p, T, yi, xi, Fv, Ki, MWv, MWl, denv, denl, Zv, Zl, SSQ])
    EoS_loop.append(eosy)
    EoS_loop.append(eosx)

    # print(f"xi = {xi}, yi = {yi}, Ki = {Ki}, Fv = {Fv}, MW_liquid = {MWl} g/mol, MW_vapor = {MWv} g/mol, Liquid density = {denl} kg/m3, Vaport density = {denv} kg/m3, Zl = {Zl}, Zv = {Zv}, SSQ = {SSQ}")

    """
    Fugacity loop - update K values using vapor and liquid fugacity
    Calculate new Fv, xi, yi 
    Check fugacity contsraint (SSQ)
    """
    if SSQ > SSQ_tol:
        while SSQ > SSQ_tol:

            try:
                Ki = Ki * fug_x / fug_y
                h = lambda Fv: np.sum((feed * (Ki - 1)) / (1 + Fv * (Ki - 1)))
                dh = lambda Fv: -np.sum(
                    (feed * (Ki - 1) ** 2) / (1 + Fv * (Ki - 1)) ** 2
                )
                Fv = so.newton(h, Fv, fprime=dh)
            except:
                Ki = Ki * fug_x / fug_y
                Fv = so.bisect(h, 1 / (1 - max(Ki)), 1 / (1 - min(Ki)))

            xi = functions.xi(feed, Fv, Ki)
            yi = Ki * xi

            eosx = EoS(xi, p, T)
            eosy = EoS(yi, p, T)

            fug_x = eosx[-2]
            fug_y = eosy[-2]
            SSQ = functions.SSQ_fugacity(fug_y, fug_x)

            EoS_loop.append(eosy)
            EoS_loop.append(eosx)
            fugacity_loop.append(
                [Component, p, T, yi, xi, Ki, fug_y, fug_x, fug_x / fug_y, SSQ]
            )

        MWl = eosx[-5]
        MWv = eosy[-5]
        denl = eosx[-4]
        denv = eosy[-4]
        Zl = eosx[-1]
        Zv = eosy[-1]

        PVT_loop.append([Component, p, T, yi, xi, Fv, Ki, MWv, MWl, denv, denl, Zv, Zl, '{:.5E}'.format(SSQ)])

    """
    If multi-stage separation: feed to the EoS is the liquid fraction xi
    """
    feed = xi

    """
    End of stage in loop
    """

# %%
"""
Prepare data to be written in Excel
"""

PVT_columns = [
    "Component",
    "Pressure",
    "Temperature",
    "yi",
    "xi",
    "Fv",
    "Ki",
    "Vapor Weight/mol",
    "Liquid Weight/mol",
    "Vapor density",
    "Liquid density",
    "Zv",
    "Zl ",
    "SSQ"
]
EoS_columns = [
    "b",
    "B",
    "m",
    "alpha",
    "a",
    "A",
    "c",
    "k",
    "Amix",
    "Bmix",
    "a1",
    "a2",
    "a3",
    "Q",
    "R_",
    "M",
    "MW",
    "den",
    "Awi",
    "fugacity_PR",
    "Z",
]
fug_columns = [
    "Component",
    "Pressure",
    "Temperature",
    "yi",
    "xi",
    "Ki",
    "fug y",
    "fug x",
    "fugx/fugy",
    "SSQ",
]

PVT_data = pd.DataFrame(data=PVT_loop, columns=PVT_columns)
EoS_data = pd.DataFrame(data=EoS_loop, columns=EoS_columns)
fug_data = pd.DataFrame(data=fugacity_loop, columns=fug_columns)

writer = pd.ExcelWriter("VLE.xlsx", engine="xlsxwriter")
PVT_data.to_excel(writer, sheet_name="PVT")
EoS_data.to_excel(writer, sheet_name="EoS")
fug_data.to_excel(writer, sheet_name="Fugacity Loop")
# EoS_loop.to_excel

writer.save()
writer.close()
# %%
PVT_data

# %%
