#%%
import numpy as np

from libraries import gas
from libraries import water
from libraries import datapreprocessing as dpp


def vc(liquid_density, gas_density, surface_tension, model="Turner"):
    # surf tens in dyne/cm, dens in kg/m3, critical velocity returned in m/s
    model = str.upper(model)
    if model == "TURNER":
        k = 1.92  # turner value
    else:
        k = 1.59  # Coleman value

    rhol = liquid_density * 0.0624279605761452
    rhog = gas_density * 0.0624279605761452

    # return 20.4*( surface_tension*(rhol - rhog)/rho_g**2)**(0.25)*0.3048 #1.8949*
    return k * (surface_tension * (rhol - rhog) / rhog ** 2) ** (0.25) * 0.3048


def Qc(p1, p2, T1, T2, Yg, Yw, d, unit=2, model="Turner"):
    if unit == 1:
        p1 = p1 / 14.5
        p2 = p2 / 14.5
        T1 = (T1 - 32) / 1.8
        T2 = (T2 - 32) / 1.8

    A = np.pi / 4 * (0.0254 * d) ** 2  # m2

    Z1 = gas.ZfacStanding(p1, T1, Yg, 2)
    Z2 = gas.ZfacStanding(p2, T2, Yg, 2)

    Bw = water.water_properties(p1, T1, T2)  # m3/Sm3
    Rsw = water.Rsw(p1)[0]

    rho_g = gas.deng(p1, T1, Z1, Yg, 2)  # kg/m3
    rho_l = water.denw1(Yw, Yg, Bw, Rsw) * 1000  # kg/m3
    sigma = water.SurfTens_wg(T1, rho_l / 1000, rho_g / 1000)
    v_c = vc(rho_l, rho_g, sigma, model)  # m/s
    Qc = (
        (p1 * v_c * A * Z2 * (T2 + 273.15)) / (Z1 * p2 * (T1 + 273.15)) * 86400
    )  # Sm3/d
    if unit == 1:
        Qc = Qc / 1000 * 35.3146667
        v_c = v_c / 0.3048
    return Qc, vc


# %% critical velocity plot, f(p) = qc
# p = np.linspace(1,100,101)
# sgg = np.array([0.6,0.7,0.8])
# qc = [[cv.Qc(i, 1.01325, 70, 15.56, j, 1, 2.25,2)[0] for i in p] for j in sgg]

# plt.plot(qc[0], p, label=f"sgg = {sgg[0]}")
# plt.plot(qc[1], p, label=f"sgg = {sgg[1]}")
# plt.plot(qc[2], p, label=f"sgg = {sgg[2]}")
# plt.legend()

# ppl_time, ID, UG, UL, DPZA, DPZF, DPZG_, HOL, PTb = dpp.getPPLData("OLGA" + "/", "02.ppl")

# from libraries import criticalvelocity as cv
# Qc = [cv.Qc(i, 1.01325, T_R, T0, SG_gas, SG_water, ID_tubing, 2)[0] for i in PTb[1][-1]*1e-5]
# Qcavg = np.average(Qc)
# Qcavg, Qc[-1], Qc[0]
