#%%
import gas
import numpy as np
import scipy.optimize as sci


def haaland(v, d, eps, Reynolds=0, rho=1000, vis=1e-3):
    """
    Input: v[m/s], d[m], eps[m], rho[kg/m3], vis[Pa s]
    Output: Friction factor based on Haalands formula
    """
    if Reynolds == 0:
        Re = rho * v * d / vis
    else:
        Re = Reynolds

    a = -1.8 * np.log10((eps / 3.7 / d) * 1.11 + 6.9 / Re)
    f = 1 / a ** 2
    return f


def Bernoulli(guess):
    """
    In solver, replace unknown variable with the actual unknown (e.g. p, h, L ...)
    Input in SI units, xi is inlet, xo is outlet, change known variables as seen fit.
    Output solved the unknown
    """
    # Variables
    Psep = 100 / 14.5
    rho = 0.61
    vis = 0.013e-3
    d = 2.25 * 0.0254
    vi = 5000 / 86400 / (np.pi * (d / 2) ** 2)
    vo = vi
    ho = 3000
    L = 0 + ho
    eps = 2.286e-05
    hi = 0
    g = 9.81
    dp = (
        rho
        * g
        * (
            548
            + 1.1e-3 * ((vi * np.pi * (d / 2) ** 2) * 1e3 * 60)
            - 2.6e-6 * ((vi * np.pi * (d / 2) ** 2) * 1e3 * 60) ** 2
        )
    )  # Added hydraulic head, add dp to Pi if needed.
    [Pi] = guess  # (
    #     200e5 + dp - 10e5 / (10000 / 86400) * (vi * np.pi * (d / 2) ** 2)
    # )  # dP/dQ if rate declines linearly with pressure
    # Re = rho * vi * d / vis
    # f = 1 / (-1.8 * np.log10((eps / 3.7 / d) ** 1.11 + 6.9 / Re)) ** 2
    eq1 = (
        Psep
        - Pi
        + rho * g * (ho - hi)
        + 0.5 * rho * (vo ** 2 - vi ** 2)
        + 0.5
        * (
            1
            / (-1.8 * np.log10((eps / 3.7 / d) ** 1.11 + 6.9 / (rho * vi * d / vis)))
            ** 2
        )
        * rho
        * vi ** 2
        / d
        * L
    )
    return [eq1]


# sol = sci.root(
#     Bernoulli, 60e5
# )  # change guess to whatever seems fitting for the unknown variable. Also change the [] and guess in the function.
# print(sol)

# def pressuredrup(guess):
#     # Variables
#     Psep = 100/14.5
#     ho = 3000
#     hi = 0
#     L = 0 + ho
#     eps = 2.286e-05
#     g = 9.81

#     Pbh = 10.65 #bara
#     Tbh = 70 #deg C
#     SGg = 0.609

#     rho = 0.61
#     vis = 0.013e-3
#     d = 2.25*0.0254
#     vi = 5000/86400/(np.pi*(d/2)**2)
#     vo = vi


def pressuredrop(d, L, eps, m, pbh, pwh, T, Yg):
    # d in in
    # L, eps in m
    # m in kg/s
    # p in bara
    # T in C
    # Yg in rho_g/rho_air
    # NOT COMPLETED!!!
    pressuredrop = []
    v = []
    f = []
    rho = []
    vis = []
    Z = []
    B = []
    x = np.linspace(0, L, 1000)
    d = 0.0254 * d
    A = np.pi / 4 * d ** 2
    p = np.linspace(pbh, pwh, 1000)

    for i in range(1, len(x)):
        Z.append(gas.ZfacStanding(p[i - 1], T, Yg, 2))  # -
        rho.append(gas.deng(p[i - 1], T, Z[i - 1], Yg, 2))  # kg/m3
        vis.append(gas.visg(Yg, rho[i - 1], T))  # cp
        B.append(gas.Bg(p[i - 1], T, Z[i - 1], 2))

        v.append(m / rho[i - 1] / A)  # local conditions
        f.append(haaland(v[i - 1], d, eps, 0, rho[i - 1], vis[i - 1] * 1e-3))

        pressuredrop.append(
            0.5 * f[i - 1] * rho[i - 1] * v[i - 1] ** 2 * (x[i] - x[i - 1]) / d
        )

    return pressuredrop, v, rho, Z, f


# %%
